@@ -1,5 +1,5 @@
 #
-# demo.tcl,v 1.92 2005/04/15 01:11:43 jenglish Exp
+# demo.tcl,v 1.97 2005/10/08 14:24:22 jenglish Exp
 #
 # Tile widget set -- widget demo
 #
@@ -301,29 +301,19 @@ makeThemeControl $::BASE.control
 ### Notebook widget.
 #
 set nb [ttk::notebook $::BASE.nb]
-tile::notebook::enableTraversal $nb
+ttk::notebook::enableTraversal $nb
 
 ### Main demo pane.
 #
 # Side-by comparison of Tile vs. core widgets.
 #
 
-set client [ttk::frame $nb.client]
-$nb add $client -text "Demo" -underline 0
 
-if {0} {
-set l [ttk::labelframe $client.styled -text "Themed" -padding 6 -underline 1]
-set r [labelframe $client.orig -text "Standard" -padx 6 -pady 6]
-grid $l $r -sticky news -padx 6 -pady 6
-grid rowconfigure $client 0 -weight 1
-grid columnconfigure $client {0 1} -weight 1
-} else {
-set pw [ttk::paned $client.pw -orient horizontal]
-pack $pw -expand true -fill both -padx 6 -pady 6
+set pw [ttk::paned $nb.client -orient horizontal]
+$nb add $pw -text "Demo" -underline 0 -padding 6
 set l [ttk::labelframe $pw.l -text "Themed" -padding 6 -underline 1]
 set r [labelframe $pw.r -text "Standard" -padx 6 -pady 6]
 $pw add $l -weight 1; $pw add $r -weight 1
-}
 
 ## menubuttonMenu -- demo menu for menubutton widgets.
 #
@@ -421,19 +411,26 @@ $l.t insert end $txt
 $r.t insert end $txt
 
 ### Scales and sliders pane.
+# (Also: partial test for #1263510)
 #
 proc scales.pane {scales} {
-    ttk::frame $scales
+    #ttk::frame $scales
+    set scales [ttk::frame .scales]
+
+    ttk::paned $scales.pw -orient horizontal
     set l [ttk::labelframe $scales.styled -text "Themed" -padding 6]
     set r [labelframe $scales.orig -text "Standard" -padx 6 -pady 6]
 
     ttk::scale $l.scale -orient horizontal -from 0 -to 100 -variable ::V(SCALE)
-    ttk::scale $l.vscale -orient vertical -from -25 -to 25 -variable ::V(VSCALE)
+    ttk::scale $l.vscale -orient vertical -from 0 -to 100 -variable ::V(VSCALE)
     ttk::progressbar $l.progress -orient horizontal -maximum 100 
-    ttk::progressbar $l.vprogress -orient vertical -maximum 25
+    ttk::progressbar $l.vprogress -orient vertical -maximum 100
     if {1} {
+        proc progress.inverted {w value} {
+            $w configure -value [expr {[$w cget -maximum] - $value}]
+        }
 	$l.scale configure -command [list $l.progress configure -value]
-	$l.vscale configure -command [list $l.vprogress configure -value]
+	$l.vscale configure -command [list progress.inverted $l.vprogress]
     } else {
 	# This would also work, but the Tk scale widgets 
 	# in the right hand pane cause some interference when 
@@ -443,6 +440,9 @@ proc scales.pane {scales} {
         $l.vprogress configure -variable ::V(VSCALE)
     }
 
+    $l.scale set 50
+    $l.vscale set 50
+
     ttk::label $l.lmode -text "Progress bar mode:"
     ttk::radiobutton $l.pbmode0 -variable ::V(PBMODE) \
     	-text determinate -value determinate -command [list pbMode $l]
@@ -483,21 +483,21 @@ proc scales.pane {scales} {
     grid rowconfigure $l 99 -weight 1
 
     scale $r.scale -orient horizontal -from 0 -to 100 -variable ::V(SCALE)
-    scale $r.vscale -orient vertical -from -25 -to 25 -variable ::V(VSCALE)
+    scale $r.vscale -orient vertical -from 0 -to 100 -variable ::V(VSCALE)
     grid $r.scale -sticky news
     grid $r.vscale -sticky nws
 
     grid rowconfigure $r 99 -weight 1
+    grid columnconfigure $r 0 -weight 1
 
     ##
-    grid $l $r -sticky nwes -padx 6 -pady 6
-    grid columnconfigure $scales 0 -weight 1
-    grid columnconfigure $scales 1 -weight 1
-    grid rowconfigure $scales 0 -weight 1
+    $scales.pw add $l -weight 1
+    $scales.pw add $r -weight 1
+    pack $scales.pw -expand true -fill both
 
     return $scales
 }
-$nb add [scales.pane $nb.scales] -text Scales -sticky nwes
+$nb add [scales.pane $nb.scales] -text Scales -sticky nwes -padding 6
 
 ### Combobox demo pane.
 #
@@ -802,7 +802,10 @@ ttk::button $cmd.close -text Close -underline 0 -command [list destroy .]
 ttk::button $cmd.help -text Help -command showHelp
 
 proc showHelp {} {
-    tk_messageBox -message "Tile demo - [package provide tile]" 
+    if {![winfo exists .helpDialog]} {
+	ttk::dialog .helpDialog -type ok -message "Tile demo" \
+	    -detail "Tile version [package provide tile]"
+    }
 }
 
 grid x $cmd.close $cmd.help -pady 6 -padx 6
