@@ -1,12 +1,8 @@
-/* altTheme.c,v 1.55 2005/03/26 20:07:06 jenglish Exp
+/* altTheme.c,v 1.58 2005/10/07 16:21:40 jenglish Exp
  *
  * Copyright (c) 2003, Joe English
  *
- * Tk alternate theme, intended to match the MSUE and GTk's default theme.
- *
- * @@@ I need a good name for this theme.  Calling it "new"
- * @@@ is just asking for built-in obsolescense (it won't be "new"
- * @@@ forever), plus that look and feel is SO 1997 :-)
+ * Tk alternate theme, intended to match the MSUE and Gtk's (old) default theme
  */
 
 #include <math.h>
@@ -19,6 +15,12 @@
 
 #include "gunk.h"
 
+#if defined(WIN32)
+static const int WIN32_XDRAWLINE_HACK = 1;
+#else
+static const int WIN32_XDRAWLINE_HACK = 0;
+#endif
+
 #define MIN(a,b) (a < b ? a : b)
 
 #define BORDERWIDTH  2
@@ -205,16 +207,25 @@ void ArrowSize(int h, ArrowDirection dir, int *widthPtr, int *heightPtr)
 }
 
 /*
- * DrawArrow --
+ * DrawArrow, FillArrow --
  * 	Draw an arrow in the indicated direction inside the specified box.
  */
+/*public*/ 
+void FillArrow(
+    Display *display, Drawable d, GC gc, Ttk_Box b, ArrowDirection dir)
+{
+    XPoint points[4];
+    ArrowPoints(b, dir, points);
+    XFillPolygon(display, d, gc, points, 3, Convex, CoordModeOrigin);
+    XDrawLines(display, d, gc, points, 4, CoordModeOrigin);
+}
+
 /*public*/ 
 void DrawArrow(
     Display *display, Drawable d, GC gc, Ttk_Box b, ArrowDirection dir)
 {
     XPoint points[4];
     ArrowPoints(b, dir, points);
-    XFillPolygon(display, d, gc, points, 3, Convex, CoordModeOrigin);
     XDrawLines(display, d, gc, points, 4, CoordModeOrigin);
 }
 
@@ -253,10 +264,9 @@ static Ttk_ElementOptionSpec BorderElementOptions[] =
     {NULL}
 };
 
-static void
-BorderElementGeometry(
-    void *clientData, void *elementRecord,
-    Tk_Window tkwin, int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
+static void BorderElementSize(
+    void *clientData, void *elementRecord, Tk_Window tkwin,
+    int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
 {
     BorderElement *bd = elementRecord;
     int borderWidth = 0;
@@ -309,7 +319,7 @@ static Ttk_ElementSpec BorderElementSpec =
     TK_STYLE_VERSION_2,
     sizeof(BorderElement),
     BorderElementOptions,
-    BorderElementGeometry,
+    BorderElementSize,
     BorderElementDraw
 };
 
@@ -332,16 +342,14 @@ static Ttk_ElementOptionSpec FieldElementOptions[] =
     {NULL}
 };
 
-static void
-FieldElementGeometry(
+static void FieldElementSize(
     void *clientData, void *elementRecord, Tk_Window tkwin,
     int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
 {
     *paddingPtr = Ttk_UniformPadding(2);
 }
 
-static void
-FieldElementDraw(
+static void FieldElementDraw(
     void *clientData, void *elementRecord, Tk_Window tkwin,
     Drawable d, Ttk_Box b, unsigned int state)
 {
@@ -359,7 +367,7 @@ static Ttk_ElementSpec FieldElementSpec =
     TK_STYLE_VERSION_2,
     sizeof(FieldElement),
     FieldElementOptions,
-    FieldElementGeometry,
+    FieldElementSize,
     FieldElementDraw
 };
 
@@ -489,10 +497,9 @@ static Ttk_ElementOptionSpec IndicatorElementOptions[] =
     {NULL}
 };
 
-static void
-IndicatorElementGeometry(
-    void *clientData, void *elementRecord,
-    Tk_Window tkwin, int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
+static void IndicatorElementSize(
+    void *clientData, void *elementRecord, Tk_Window tkwin,
+    int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
 {
     IndicatorSpec *spec = clientData;
     IndicatorElement *indicator = elementRecord;
@@ -501,9 +508,9 @@ IndicatorElementGeometry(
     *heightPtr = spec->height;
 }
 
-static void
-IndicatorElementDraw(void *clientData, void *elementRecord,
-    Tk_Window tkwin, Drawable d, Ttk_Box b, unsigned int state)
+static void IndicatorElementDraw(
+    void *clientData, void *elementRecord, Tk_Window tkwin,
+    Drawable d, Ttk_Box b, unsigned int state)
 {
     IndicatorSpec *spec = clientData;
     IndicatorElement *indicator = elementRecord;
@@ -595,7 +602,7 @@ static Ttk_ElementSpec IndicatorElementSpec =
     TK_STYLE_VERSION_2,
     sizeof(IndicatorElement),
     IndicatorElementOptions,
-    IndicatorElementGeometry,
+    IndicatorElementSize,
     IndicatorElementDraw
 };
 
@@ -638,10 +645,9 @@ static Ttk_ElementOptionSpec ArrowElementOptions[] =
  */
 static Ttk_Padding ArrowPadding = { 3,3,4,4 };
 
-static void
-ArrowElementGeometry(
-    void *clientData, void *elementRecord,
-    Tk_Window tkwin, int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
+static void ArrowElementSize(
+    void *clientData, void *elementRecord, Tk_Window tkwin,
+    int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
 {
     ArrowElement *arrow = elementRecord;
     int direction = *(int *)clientData;
@@ -653,9 +659,9 @@ ArrowElementGeometry(
     *paddingPtr = ArrowPadding;
 }
 
-static void
-ArrowElementDraw(void *clientData, void *elementRecord,
-    Tk_Window tkwin, Drawable d, Ttk_Box b, unsigned int state)
+static void ArrowElementDraw(
+    void *clientData, void *elementRecord, Tk_Window tkwin,
+    Drawable d, Ttk_Box b, unsigned int state)
 {
     int direction = *(int *)clientData;
     ArrowElement *arrow = elementRecord;
@@ -671,7 +677,7 @@ ArrowElementDraw(void *clientData, void *elementRecord,
 	tkwin, d, border, b.x, b.y, b.width, b.height, 0, TK_RELIEF_FLAT);
     DrawBorder(tkwin,d,border,borderColor,b,borderWidth,relief);
 
-    DrawArrow(Tk_Display(tkwin), d, Tk_GCForColor(arrowColor, d),
+    FillArrow(Tk_Display(tkwin), d, Tk_GCForColor(arrowColor, d),
 	Ttk_PadBox(b, ArrowPadding), direction);
 }
 
@@ -680,7 +686,7 @@ static Ttk_ElementSpec ArrowElementSpec =
     TK_STYLE_VERSION_2,
     sizeof(ArrowElement),
     ArrowElementOptions,
-    ArrowElementGeometry,
+    ArrowElementSize,
     ArrowElementDraw
 };
 
@@ -715,10 +721,9 @@ static Ttk_ElementOptionSpec MenubuttonArrowElementOptions[] =
 
 static Ttk_Padding MenubuttonArrowPadding = { 3, 0, 3, 0 };
 
-static void
-MenubuttonArrowElementGeometry(
-    void *clientData, void *elementRecord,
-    Tk_Window tkwin, int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
+static void MenubuttonArrowElementSize(
+    void *clientData, void *elementRecord, Tk_Window tkwin,
+    int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
 {
     MenubuttonArrowElement *arrow = elementRecord;
     int size = MENUBUTTON_ARROW_SIZE;
@@ -727,9 +732,9 @@ MenubuttonArrowElementGeometry(
     *paddingPtr = MenubuttonArrowPadding;
 }
 
-static void
-MenubuttonArrowElementDraw(void *clientData, void *elementRecord,
-    Tk_Window tkwin, Drawable d, Ttk_Box b, unsigned int state)
+static void MenubuttonArrowElementDraw(
+    void *clientData, void *elementRecord, Tk_Window tkwin,
+    Drawable d, Ttk_Box b, unsigned int state)
 {
     MenubuttonArrowElement *arrow = elementRecord;
     XColor *arrowColor = Tk_GetColorFromObj(tkwin, arrow->colorObj);
@@ -755,7 +760,7 @@ MenubuttonArrowElementDraw(void *clientData, void *elementRecord,
     ArrowSize(size, arrowDirection, &width, &height);
     b = Ttk_PadBox(b, MenubuttonArrowPadding);
     b = Ttk_AnchorBox(b, width, height, TK_ANCHOR_CENTER);
-    DrawArrow(Tk_Display(tkwin), d, gc, b, arrowDirection);
+    FillArrow(Tk_Display(tkwin), d, gc, b, arrowDirection);
 }
 
 static Ttk_ElementSpec MenubuttonArrowElementSpec =
@@ -763,7 +768,7 @@ static Ttk_ElementSpec MenubuttonArrowElementSpec =
     TK_STYLE_VERSION_2,
     sizeof(MenubuttonArrowElement),
     MenubuttonArrowElementOptions,
-    MenubuttonArrowElementGeometry,
+    MenubuttonArrowElementSize,
     MenubuttonArrowElementDraw
 };
 
@@ -804,10 +809,9 @@ static Ttk_ElementOptionSpec TroughElementOptions[] =
     { NULL }
 };
 
-static void
-TroughElementGeometry(
-    void *clientData, void *elementRecord,
-    Tk_Window tkwin, int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
+static void TroughElementSize(
+    void *clientData, void *elementRecord, Tk_Window tkwin,
+    int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
 {
     TroughElement *troughPtr = elementRecord;
     int borderWidth = 2, grooveWidth = 0;
@@ -820,9 +824,9 @@ TroughElementGeometry(
     }
 }
 
-static void
-TroughElementDraw(void *clientData, void *elementRecord,
-    Tk_Window tkwin, Drawable d, Ttk_Box b, unsigned int state)
+static void TroughElementDraw(
+    void *clientData, void *elementRecord, Tk_Window tkwin,
+    Drawable d, Ttk_Box b, unsigned int state)
 {
     TroughElement *troughPtr = elementRecord;
     Tk_3DBorder border = NULL;
@@ -853,7 +857,7 @@ static Ttk_ElementSpec TroughElementSpec =
     TK_STYLE_VERSION_2,
     sizeof(TroughElement),
     TroughElementOptions,
-    TroughElementGeometry,
+    TroughElementSize,
     TroughElementDraw
 };
 
@@ -886,10 +890,9 @@ static Ttk_ElementOptionSpec ThumbElementOptions[] =
     { NULL }
 };
 
-static void
-ThumbElementGeometry(
-    void *clientData, void *elementRecord,
-    Tk_Window tkwin, int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
+static void ThumbElementSize(
+    void *clientData, void *elementRecord, Tk_Window tkwin,
+    int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
 {
     ThumbElement *thumb = elementRecord;
     int orient, size;
@@ -905,9 +908,9 @@ ThumbElementGeometry(
     }
 }
 
-static void
-ThumbElementDraw(void *clientData, void *elementRecord,
-    Tk_Window tkwin, Drawable d, Ttk_Box b, unsigned int state)
+static void ThumbElementDraw(
+    void *clientData, void *elementRecord, Tk_Window tkwin,
+    Drawable d, Ttk_Box b, unsigned int state)
 {
     ThumbElement *thumb = elementRecord;
     Tk_3DBorder border = Tk_Get3DBorderFromObj(tkwin, thumb->borderObj);
@@ -934,7 +937,7 @@ static Ttk_ElementSpec ThumbElementSpec =
     TK_STYLE_VERSION_2,
     sizeof(ThumbElement),
     ThumbElementOptions,
-    ThumbElementGeometry,
+    ThumbElementSize,
     ThumbElementDraw
 };
 
@@ -951,19 +954,19 @@ static Ttk_ElementSpec ThumbElementSpec =
 
 typedef struct
 {
-    Tcl_Obj *lengthObj;  /* the length of the slider (if a flat style) */
-    Tcl_Obj *widthObj;   /* the width of the slider (height if horizontal) */
-    Tcl_Obj *reliefObj;  /* the relief for this object */
-    Tcl_Obj *borderObj;
+    Tcl_Obj *lengthObj;		/* Long axis dimension */
+    Tcl_Obj *thicknessObj;	/* Short axis dimension */
+    Tcl_Obj *reliefObj;		/* Relief for this object */
+    Tcl_Obj *borderObj;		/* Border / background color */
     Tcl_Obj *borderWidthObj;
-    Tcl_Obj *orientObj;  /* orientation of overall slider */
+    Tcl_Obj *orientObj;		/* Orientation of overall slider */
 } SliderElement;
 
 static Ttk_ElementOptionSpec SliderElementOptions[] =
 {
     { "-sliderlength", TK_OPTION_PIXELS, Tk_Offset(SliderElement,lengthObj),
 	"15" },
-    { "-width", TK_OPTION_PIXELS, Tk_Offset(SliderElement,widthObj),
+    { "-sliderthickness",TK_OPTION_PIXELS,Tk_Offset(SliderElement,thicknessObj),
 	"15" },
     { "-sliderrelief", TK_OPTION_RELIEF, Tk_Offset(SliderElement,reliefObj),
 	"raised" },
@@ -976,35 +979,34 @@ static Ttk_ElementOptionSpec SliderElementOptions[] =
     { NULL }
 };
 
-static void
-SliderElementGeometry(
-    void *clientData, void *elementRecord,
-    Tk_Window tkwin, int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
+static void SliderElementSize(
+    void *clientData, void *elementRecord, Tk_Window tkwin,
+    int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
 {
     SliderElement *slider = elementRecord;
-    int borderWidth, sliderwidth, orient, sliderlength;
+    int orient, length, thickness, borderWidth;
 
     Ttk_GetOrientFromObj(NULL, slider->orientObj, &orient);
     Tk_GetPixelsFromObj(NULL, tkwin, slider->borderWidthObj, &borderWidth);
-    Tk_GetPixelsFromObj(NULL, tkwin, slider->lengthObj, &sliderlength);
-    Tk_GetPixelsFromObj(NULL, tkwin, slider->widthObj, &sliderwidth);
+    Tk_GetPixelsFromObj(NULL, tkwin, slider->lengthObj, &length);
+    Tk_GetPixelsFromObj(NULL, tkwin, slider->thicknessObj, &thickness);
 
     switch (orient) {
 	case TTK_ORIENT_VERTICAL:
-	    *widthPtr = sliderwidth + (borderWidth *2);
+	    *widthPtr = thickness + (borderWidth *2);
 	    *heightPtr = *widthPtr/2;
 	    break;
 
 	case TTK_ORIENT_HORIZONTAL:
-	    *heightPtr = sliderwidth + (borderWidth *2);
+	    *heightPtr = thickness + (borderWidth *2);
 	    *widthPtr = *heightPtr/2;
 	    break;
     }
 }
 
-static void
-SliderElementDraw(void *clientData, void *elementRecord,
-    Tk_Window tkwin, Drawable d, Ttk_Box b, unsigned int state)
+static void SliderElementDraw(
+    void *clientData, void *elementRecord, Tk_Window tkwin,
+    Drawable d, Ttk_Box b, unsigned int state)
 {
     SliderElement *slider = elementRecord;
     Tk_3DBorder border = NULL;
@@ -1052,10 +1054,90 @@ static Ttk_ElementSpec SliderElementSpec =
     TK_STYLE_VERSION_2,
     sizeof(SliderElement),
     SliderElementOptions,
-    SliderElementGeometry,
+    SliderElementSize,
     SliderElementDraw
 };
 
+/*------------------------------------------------------------------------
+ * +++ Tree indicator element.
+ */
+
+#define TTK_STATE_OPEN TTK_STATE_USER1		/* XREF: treeview.c */
+#define TTK_STATE_LEAF TTK_STATE_USER2
+
+typedef struct
+{
+    Tcl_Obj *colorObj;
+    Tcl_Obj *marginObj;
+    Tcl_Obj *diameterObj;
+} TreeitemIndicator;
+
+static Ttk_ElementOptionSpec TreeitemIndicatorOptions[] =
+{
+    { "-foreground", TK_OPTION_COLOR,
+	Tk_Offset(TreeitemIndicator,colorObj), DEFAULT_FOREGROUND },
+    { "-diameter", TK_OPTION_PIXELS,
+	Tk_Offset(TreeitemIndicator,diameterObj), "6" },
+    { "-indicatormargins", TK_OPTION_STRING,
+	Tk_Offset(TreeitemIndicator,marginObj), "0 2 4 2" },
+    {NULL}
+};
+
+static void TreeitemIndicatorSize(
+    void *clientData, void *elementRecord, Tk_Window tkwin,
+    int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
+{
+    TreeitemIndicator *indicator = elementRecord;
+    int diameter = 0;
+    Ttk_GetPaddingFromObj(NULL, tkwin, indicator->marginObj, paddingPtr);
+    Tk_GetPixelsFromObj(NULL, tkwin, indicator->diameterObj, &diameter);
+    *widthPtr = *heightPtr = diameter;
+}
+
+static void TreeitemIndicatorDraw(
+    void *clientData, void *elementRecord, Tk_Window tkwin,
+    Drawable d, Ttk_Box b, Ttk_State state)
+{
+    TreeitemIndicator *indicator = elementRecord;
+    XColor *color = Tk_GetColorFromObj(tkwin, indicator->colorObj);
+    GC gc = Tk_GCForColor(color, d);
+    Ttk_Padding padding = Ttk_UniformPadding(0);
+    int w = WIN32_XDRAWLINE_HACK;
+    int cx, cy;
+
+    if (state & TTK_STATE_LEAF) {
+	/* don't draw anything ... */
+	return;
+    }
+
+    Ttk_GetPaddingFromObj(NULL,tkwin,indicator->marginObj,&padding);
+    b = Ttk_PadBox(b, padding);
+
+    XDrawRectangle(Tk_Display(tkwin), d, gc,
+	    b.x, b.y, b.width, b.height);
+
+    cx = b.x + b.width / 2;
+    cy = b.y + b.height / 2;
+    XDrawLine(Tk_Display(tkwin), d, gc, b.x+2, cy, b.x+b.width-2+w, cy);
+
+    if (!(state & TTK_STATE_OPEN)) {
+	/* turn '-' into a '+' */
+	XDrawLine(Tk_Display(tkwin), d, gc, cx, b.y+2, cx, b.y+b.height-2+w);
+    }
+}
+
+static Ttk_ElementSpec TreeitemIndicatorElementSpec =
+{
+    TK_STYLE_VERSION_2,
+    sizeof(TreeitemIndicator),
+    TreeitemIndicatorOptions,
+    TreeitemIndicatorSize,
+    TreeitemIndicatorDraw
+};
+
+
+
+
 /*------------------------------------------------------------------------
  * AltTheme_Init --
  * 	Install alternate theme.
@@ -1097,6 +1179,9 @@ int AltTheme_Init(Tcl_Interp *interp)
     Ttk_RegisterElement(interp, theme, "arrow",
 	    &ArrowElementSpec, &ArrowElements[0]);
 
+    Ttk_RegisterElement(interp, theme, "Treeitem.indicator",
+	    &TreeitemIndicatorElementSpec, 0);
+
     Tcl_PkgProvide(interp, "tile::theme::alt", TILE_VERSION);
 
     return TCL_OK;
