@@ -1,7 +1,8 @@
-/* button.c,v 1.39 2005/03/31 17:27:10 jenglish Exp
+/* button.c,v 1.45 2005/10/08 14:02:47 jenglish Exp
  * Copyright (c) 2003, Joe English
  *
- * Tile package: button, checkbutton, radiobutton, and menubutton widgets.
+ * Tile widget set: label, button, checkbutton, radiobutton, and 
+ * menubutton widgets.
  */
 
 #include <string.h>
@@ -28,7 +29,7 @@ typedef struct
     Tcl_Obj *underlineObj;
     Tcl_Obj *widthObj;
 
-    TraceHandle	*textVariableTrace;
+    Ttk_TraceHandle	*textVariableTrace;
     Tk_Image	*images;
 
     /*
@@ -40,10 +41,10 @@ typedef struct
      * Compound label/image resources:
      */
     Tcl_Obj *compoundObj;
-    Tcl_Obj *paddingObj;	/* Possibly does not belong here */
+    Tcl_Obj *paddingObj;
 
     /*
-     * Compatibility/legacy resources:
+     * Compatibility/legacy options:
      */
     Tcl_Obj *stateObj;
 
@@ -72,14 +73,14 @@ static Tk_OptionSpec BaseOptionSpecs[] =
 	TK_OPTION_NULL_OK,0,GEOMETRY_CHANGED },
 
     /*
-     * Image resources
+     * Image options
      */
     {TK_OPTION_STRING, "-image", "image", "Image", NULL/*default*/,
 	Tk_Offset(Base,base.imageObj), -1, 
 	TK_OPTION_NULL_OK,0,GEOMETRY_CHANGED },
 
     /*
-     * Compound base/image resources.
+     * Compound base/image options
      */
     {TK_OPTION_STRING_TABLE, "-compound", "compound", "Compound",
 	 "none", Tk_Offset(Base,base.compoundObj), -1,
@@ -89,7 +90,7 @@ static Tk_OptionSpec BaseOptionSpecs[] =
 	TK_OPTION_NULL_OK,0,GEOMETRY_CHANGED},
 
     /*
-     * Compatibility/legacy resources:
+     * Compatibility/legacy options
      */
     {TK_OPTION_STRING, "-state", "state", "State",
 	 "normal", Tk_Offset(Base,base.stateObj), -1,
@@ -101,23 +102,22 @@ static Tk_OptionSpec BaseOptionSpecs[] =
 /*
  * Variable trace procedure for -textvariable option:
  */
-static int
-TextVariableChanged(
-    Tcl_Interp *interp, 	/* interpreter containing variable. */
-    void *clientData,		/* widget record pointer */
-    const char *value)		/* variable's new value */
+static void TextVariableChanged(void *clientData, const char *value)
 {
     Base *basePtr = clientData;
-    Tcl_Obj *newText = 
-	value ? Tcl_NewStringObj(value, -1) : Tcl_NewStringObj("", 0);
+    Tcl_Obj *newText;
+
+    if (WidgetDestroyed(&basePtr->core)) {
+	return;
+    }
+
+    newText = value ? Tcl_NewStringObj(value, -1) : Tcl_NewStringObj("", 0);
 
     Tcl_IncrRefCount(newText);
     Tcl_DecrRefCount(basePtr->base.textObj);
     basePtr->base.textObj = newText;
 
-    WidgetChanged(&basePtr->core, REDISPLAY_REQUIRED|RELAYOUT_REQUIRED);
-
-    return TCL_OK;
+    TtkResizeWidget(&basePtr->core);
 }
 
 /*
@@ -127,7 +127,7 @@ static void CoreImageChangedProc(ClientData clientData,
     int x, int y, int width, int height, int imageWidth, int imageHeight)
 {
     WidgetCore *corePtr = (WidgetCore *)clientData;
-    WidgetChanged(corePtr, REDISPLAY_REQUIRED);
+    TtkRedisplayWidget(corePtr);
 }
 
 /* GetImageList --
@@ -228,21 +228,20 @@ BaseCleanup(void *recordPtr)
 {
     Base *basePtr = recordPtr;
     if (basePtr->base.textVariableTrace)
-	UntraceVariable(basePtr->base.textVariableTrace);
+	Ttk_UntraceVariable(basePtr->base.textVariableTrace);
     if (basePtr->base.images) 
     	FreeImageList(basePtr->base.images);
 }
 
-static int
-BaseConfigure(Tcl_Interp *interp, void *recordPtr, int mask)
+static int BaseConfigure(Tcl_Interp *interp, void *recordPtr, int mask)
 {
     Base *basePtr = recordPtr;
     Tcl_Obj *textVarName = basePtr->base.textVariableObj;
-    TraceHandle *vt = 0;
+    Ttk_TraceHandle *vt = 0;
     Tk_Image *images = NULL;
 
     if (textVarName != NULL && *Tcl_GetString(textVarName) != '\0') {
-	vt = TraceVariable(interp, textVarName, TextVariableChanged, basePtr);
+	vt = Ttk_TraceVariable(interp,textVarName,TextVariableChanged,basePtr);
 	if (!vt) return TCL_ERROR;
     }
 
@@ -255,16 +254,18 @@ BaseConfigure(Tcl_Interp *interp, void *recordPtr, int mask)
     if (CoreConfigure(interp, recordPtr, mask) != TCL_OK) {
 error:
 	if (images) FreeImageList(images);
-	if (vt) UntraceVariable(vt);
+	if (vt) Ttk_UntraceVariable(vt);
 	return TCL_ERROR;
     }
 
-    if (basePtr->base.textVariableTrace)
-	UntraceVariable(basePtr->base.textVariableTrace);
+    if (basePtr->base.textVariableTrace) {
+	Ttk_UntraceVariable(basePtr->base.textVariableTrace);
+    }
     basePtr->base.textVariableTrace = vt;
 
-    if (basePtr->base.images) 
+    if (basePtr->base.images) {
 	FreeImageList(basePtr->base.images);
+    }
     basePtr->base.images = images;
 
     if (mask & STATE_CHANGED) {
@@ -280,8 +281,9 @@ BasePostConfigure(Tcl_Interp *interp, void *recordPtr, int mask)
     Base *basePtr = recordPtr;
     int status = TCL_OK;
 
-    if (basePtr->base.textVariableTrace)
-	status = FireTrace(basePtr->base.textVariableTrace);
+    if (basePtr->base.textVariableTrace) {
+	status = Ttk_FireTrace(basePtr->base.textVariableTrace);
+    }
 
     return status;
 }
@@ -289,7 +291,7 @@ BasePostConfigure(Tcl_Interp *interp, void *recordPtr, int mask)
 
 /*------------------------------------------------------------------------
  * +++ Label widget:
- * Just a base widget that adds a few appearance-related resources.
+ * Just a base widget that adds a few appearance-related options
  */
 
 typedef struct
@@ -324,7 +326,7 @@ static Tk_OptionSpec LabelOptionSpecs[] =
 	NULL, Tk_Offset(Label,label.fontObj), -1,
 	TK_OPTION_NULL_OK,0,GEOMETRY_CHANGED },
     {TK_OPTION_PIXELS, "-borderwidth", "borderWidth", "BorderWidth", 
-	"2", Tk_Offset(Label,label.borderWidthObj), -1,
+	NULL, Tk_Offset(Label,label.borderWidthObj), -1,
 	TK_OPTION_NULL_OK,0,GEOMETRY_CHANGED },
     {TK_OPTION_RELIEF, "-relief", "relief", "Relief",
 	"flat", Tk_Offset(Label,label.reliefObj), -1,
@@ -366,13 +368,12 @@ WidgetSpec LabelWidgetSpec =
     WidgetGetLayout, 		/* getLayoutProc */
     WidgetSize, 		/* sizeProc */
     WidgetDoLayout,		/* layoutProc */
-    WidgetDisplay,		/* displayProc */
-    WIDGET_SPEC_END		/* sentinel */
+    WidgetDisplay		/* displayProc */
 };
 
 /*------------------------------------------------------------------------
  * +++ Button widget.
- * Adds a new subcommand "invoke", and resources "-command" and "-default"
+ * Adds a new subcommand "invoke", and options "-command" and "-default"
  */
 
 typedef struct
@@ -430,17 +431,21 @@ static int ButtonConfigure(Tcl_Interp *interp, void *recordPtr, int mask)
     return TCL_OK;
 }
 
+/* $button invoke --
+ * 	Evaluate the button's -command.
+ */
 static int
 ButtonInvokeCommand(
-    Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[], void *recordPtr)
+    Tcl_Interp *interp, int objc, Tcl_Obj *const objv[], void *recordPtr)
 {
     Button *buttonPtr = recordPtr;
     if (objc > 2) {
 	Tcl_WrongNumArgs(interp, 1, objv, "invoke");
 	return TCL_ERROR;
     }
-    if (buttonPtr->core.state & TTK_STATE_DISABLED)
+    if (buttonPtr->core.state & TTK_STATE_DISABLED) {
 	return TCL_OK;
+    }
     return Tcl_EvalObjEx(interp, buttonPtr->button.commandObj, TCL_EVAL_GLOBAL);
 }
 
@@ -467,9 +472,7 @@ WidgetSpec ButtonWidgetSpec =
     WidgetGetLayout, 		/* getLayoutProc */
     WidgetSize, 		/* sizeProc */
     WidgetDoLayout,		/* layoutProc */
-    WidgetDisplay,		/* displayProc */
-
-    WIDGET_SPEC_END		/* sentinel */
+    WidgetDisplay		/* displayProc */
 };
 
 /*------------------------------------------------------------------------
@@ -482,7 +485,7 @@ typedef struct
     Tcl_Obj *offValueObj;
     Tcl_Obj *commandObj;
 
-    TraceHandle *variableTrace;
+    Ttk_TraceHandle *variableTrace;
 
 } CheckbuttonPart;
 
@@ -516,19 +519,19 @@ static Tk_OptionSpec CheckbuttonOptionSpecs[] =
 CHECKBUTTON_COMPAT_OPTIONS
 
 /*
- * Variable trace procedure for checkbutton -variable resource:
+ * Variable trace procedure for checkbutton -variable option
  */
-static int
-CheckbuttonVariableChanged(
-    Tcl_Interp *interp, 	/* Interpreter containing variable. */
-    void *clientData,		/* widget record pointer */
-    const char *value)		/* Variable's new value */
+static void CheckbuttonVariableChanged(void *clientData, const char *value)
 {
     Checkbutton *checkPtr = clientData;
 
+    if (WidgetDestroyed(&checkPtr->core)) {
+	return;
+    }
+
     if (!value) {
 	WidgetChangeState(&checkPtr->core, TTK_STATE_ALTERNATE, 0);
-	return TCL_OK;
+	return;
     }
     /* else */
     WidgetChangeState(&checkPtr->core, 0, TTK_STATE_ALTERNATE);
@@ -537,19 +540,19 @@ CheckbuttonVariableChanged(
     } else {
 	WidgetChangeState(&checkPtr->core, 0, TTK_STATE_SELECTED);
     }
-
-    return TCL_OK;
 }
 
-static int
-CheckbuttonInitialize(Tcl_Interp *interp, void *recordPtr)
+static int CheckbuttonInitialize(Tcl_Interp *interp, void *recordPtr)
 {
     Checkbutton *checkPtr = recordPtr;
-    checkPtr->checkbutton.variableTrace = TraceVariable(
-	interp, checkPtr->checkbutton.variableObj,
-	CheckbuttonVariableChanged, checkPtr);
+    Tcl_Obj *objPtr;
 
-    if (!checkPtr->checkbutton.variableTrace) return TCL_ERROR;
+    /* default -variable is the widget name:
+     */
+    objPtr = Tcl_NewStringObj(Tk_PathName(checkPtr->core.tkwin), -1);
+    Tcl_IncrRefCount(objPtr);
+    Tcl_DecrRefCount(checkPtr->checkbutton.variableObj);
+    checkPtr->checkbutton.variableObj = objPtr;
 
     return BaseInitialize(interp, recordPtr);
 }
@@ -558,7 +561,7 @@ static void
 CheckbuttonCleanup(void *recordPtr)
 {
     Checkbutton *checkPtr = recordPtr;
-    UntraceVariable(checkPtr->checkbutton.variableTrace);
+    Ttk_UntraceVariable(checkPtr->checkbutton.variableTrace);
     checkPtr->checkbutton.variableTrace = 0;
     BaseCleanup(recordPtr);
 }
@@ -567,18 +570,20 @@ static int
 CheckbuttonConfigure(Tcl_Interp *interp, void *recordPtr, int mask)
 {
     Checkbutton *checkPtr = recordPtr;
-    TraceHandle *vt = TraceVariable(interp, checkPtr->checkbutton.variableObj,
+    Ttk_TraceHandle *vt = Ttk_TraceVariable(
+	interp, checkPtr->checkbutton.variableObj,
 	CheckbuttonVariableChanged, checkPtr);
 
-    if (!vt) 
-	return TCL_ERROR;
-
-    if (BaseConfigure(interp, recordPtr, mask) != TCL_OK){
-	UntraceVariable(vt);
+    if (!vt) {
 	return TCL_ERROR;
     }
 
-    UntraceVariable(checkPtr->checkbutton.variableTrace);
+    if (BaseConfigure(interp, recordPtr, mask) != TCL_OK){
+	Ttk_UntraceVariable(vt);
+	return TCL_ERROR;
+    }
+
+    Ttk_UntraceVariable(checkPtr->checkbutton.variableTrace);
     checkPtr->checkbutton.variableTrace = vt;
 
     return TCL_OK;
@@ -591,7 +596,7 @@ CheckbuttonPostConfigure(Tcl_Interp *interp, void *recordPtr, int mask)
     int status = TCL_OK;
 
     if (checkPtr->checkbutton.variableTrace)
-	status = FireTrace(checkPtr->checkbutton.variableTrace);
+	status = Ttk_FireTrace(checkPtr->checkbutton.variableTrace);
     if (status == TCL_OK && !WidgetDestroyed(&checkPtr->core))
 	status = BasePostConfigure(interp, recordPtr, mask);
     return status;
@@ -661,9 +666,7 @@ WidgetSpec CheckbuttonWidgetSpec =
     WidgetGetLayout, 		/* getLayoutProc */
     WidgetSize, 		/* sizeProc */
     WidgetDoLayout,		/* layoutProc */
-    WidgetDisplay,		/* displayProc */
-
-    WIDGET_SPEC_END		/* sentinel */
+    WidgetDisplay		/* displayProc */
 };
 
 /*------------------------------------------------------------------------
@@ -676,7 +679,7 @@ typedef struct
     Tcl_Obj *valueObj;
     Tcl_Obj *commandObj;
 
-    TraceHandle	*variableTrace;
+    Ttk_TraceHandle	*variableTrace;
 
 } RadiobuttonPart;
 
@@ -696,11 +699,14 @@ static Tk_OptionSpec RadiobuttonOptionSpecs[] =
     WIDGET_TAKES_FOCUS,
 
     {TK_OPTION_STRING, "-variable", "variable", "Variable",
-	"", Tk_Offset(Radiobutton, radiobutton.variableObj), -1, 0,0,0},
+	"::selectedButton", Tk_Offset(Radiobutton, radiobutton.variableObj),-1,
+	0,0,0},
     {TK_OPTION_STRING, "-value", "Value", "Value",
-	"1", Tk_Offset(Radiobutton, radiobutton.valueObj), -1, 0,0,0},
+	"1", Tk_Offset(Radiobutton, radiobutton.valueObj), -1,
+	0,0,0},
     {TK_OPTION_STRING, "-command", "command", "Command",
-	"", Tk_Offset(Radiobutton, radiobutton.commandObj), -1, 0,0,0},
+	"", Tk_Offset(Radiobutton, radiobutton.commandObj), -1,
+	0,0,0},
 
     WIDGET_INHERIT_OPTIONS(BaseOptionSpecs)
 };
@@ -710,17 +716,18 @@ RADIOBUTTON_COMPAT_OPTIONS
 /*
  * Variable trace procedure for radiobuttons.
  */
-static int
-RadiobuttonVariableChanged(
-    Tcl_Interp *interp, 	/* Interpreter containing variable. */
-    void *clientData,		/* widget record pointer */
-    const char *value)		/* Variable's new value */
+static void 
+RadiobuttonVariableChanged(void *clientData, const char *value)
 {
     Radiobutton *radioPtr = clientData;
 
+    if (WidgetDestroyed(&radioPtr->core)) {
+	return;
+    }
+
     if (!value) {
 	WidgetChangeState(&radioPtr->core, TTK_STATE_ALTERNATE, 0);
-	return TCL_OK;
+	return;
     }
     /* else */
     WidgetChangeState(&radioPtr->core, 0, TTK_STATE_ALTERNATE);
@@ -729,29 +736,13 @@ RadiobuttonVariableChanged(
     } else {
 	WidgetChangeState(&radioPtr->core, 0, TTK_STATE_SELECTED);
     }
-
-    return TCL_OK;
-}
-
-static int
-RadiobuttonInitialize(Tcl_Interp *interp, void *recordPtr)
-{
-    Radiobutton *radioPtr = recordPtr;
-
-    radioPtr->radiobutton.variableTrace = TraceVariable(
-	interp, radioPtr->radiobutton.variableObj,
-	RadiobuttonVariableChanged, radioPtr);
-    if (!radioPtr->radiobutton.variableTrace) 
-	return TCL_ERROR;
-
-    return BaseInitialize(interp, recordPtr);
 }
 
 static void
 RadiobuttonCleanup(void *recordPtr)
 {
     Radiobutton *radioPtr = recordPtr;
-    UntraceVariable(radioPtr->radiobutton.variableTrace);
+    Ttk_UntraceVariable(radioPtr->radiobutton.variableTrace);
     radioPtr->radiobutton.variableTrace = 0;
     BaseCleanup(recordPtr);
 }
@@ -760,18 +751,20 @@ static int
 RadiobuttonConfigure(Tcl_Interp *interp, void *recordPtr, int mask)
 {
     Radiobutton *radioPtr = recordPtr;
-    TraceHandle *vt = TraceVariable(interp, radioPtr->radiobutton.variableObj,
+    Ttk_TraceHandle *vt = Ttk_TraceVariable(
+	interp, radioPtr->radiobutton.variableObj,
 	RadiobuttonVariableChanged, radioPtr);
 
-    if (!vt) 
-	return TCL_ERROR;
-
-    if (BaseConfigure(interp, recordPtr, mask) != TCL_OK) {
-	UntraceVariable(vt);
+    if (!vt) {
 	return TCL_ERROR;
     }
 
-    UntraceVariable(radioPtr->radiobutton.variableTrace);
+    if (BaseConfigure(interp, recordPtr, mask) != TCL_OK) {
+	Ttk_UntraceVariable(vt);
+	return TCL_ERROR;
+    }
+
+    Ttk_UntraceVariable(radioPtr->radiobutton.variableTrace);
     radioPtr->radiobutton.variableTrace = vt;
 
     return TCL_OK;
@@ -784,7 +777,7 @@ RadiobuttonPostConfigure(Tcl_Interp *interp, void *recordPtr, int mask)
     int status = TCL_OK;
 
     if (radioPtr->radiobutton.variableTrace)
-	status = FireTrace(radioPtr->radiobutton.variableTrace);
+	status = Ttk_FireTrace(radioPtr->radiobutton.variableTrace);
     if (status == TCL_OK && !WidgetDestroyed(&radioPtr->core))
 	status = BasePostConfigure(interp, recordPtr, mask);
     return status;
@@ -839,15 +832,14 @@ WidgetSpec RadiobuttonWidgetSpec =
     sizeof(Radiobutton),	/* recordSize */
     RadiobuttonCompatOptionSpecs,	/* optionSpecs */
     RadiobuttonCommands,	/* subcommands */
-    RadiobuttonInitialize,	/* initializeProc */
+    BaseInitialize,		/* initializeProc */
     RadiobuttonCleanup,		/* cleanupProc */
     RadiobuttonConfigure,	/* configureProc */
     RadiobuttonPostConfigure,	/* postConfigureProc */
     WidgetGetLayout, 		/* getLayoutProc */
     WidgetSize, 		/* sizeProc */
     WidgetDoLayout,		/* layoutProc */
-    WidgetDisplay,		/* displayProc */
-    WIDGET_SPEC_END		/* sentinel */
+    WidgetDisplay		/* displayProc */
 };
 
 /*------------------------------------------------------------------------
@@ -911,7 +903,6 @@ WidgetSpec MenubuttonWidgetSpec =
     WidgetGetLayout, 		/* getLayoutProc */
     WidgetSize, 		/* sizeProc */
     WidgetDoLayout,		/* layoutProc */
-    WidgetDisplay,		/* displayProc */
-    WIDGET_SPEC_END		/* sentinel */
+    WidgetDisplay		/* displayProc */
 };
 
