@@ -1,5 +1,5 @@
 /*
- * clamTheme.c,v 1.22 2005/04/11 01:53:59 jenglish Exp
+ * clamTheme.c,v 1.24 2005/09/04 17:24:29 jenglish Exp
  *
  * Copyright (C) 2004 Joe English
  *
@@ -395,6 +395,65 @@ static Ttk_ElementSpec CheckIndicatorElementSpec = {
     CheckIndicatorElementDraw
 };
 
+#define MENUBUTTON_ARROW_SIZE 5
+
+typedef struct {
+    Tcl_Obj *sizeObj;
+    Tcl_Obj *colorObj;
+    Tcl_Obj *paddingObj;
+} MenuIndicatorElement;
+
+static Ttk_ElementOptionSpec MenuIndicatorElementOptions[] =
+{
+    { "-arrowsize", TK_OPTION_PIXELS,
+	Tk_Offset(MenuIndicatorElement,sizeObj), 
+	STR(MENUBUTTON_ARROW_SIZE)},
+    { "-arrowcolor",TK_OPTION_COLOR,
+	Tk_Offset(MenuIndicatorElement,colorObj),
+	"black" },
+    { "-arrowpadding",TK_OPTION_STRING,
+	Tk_Offset(MenuIndicatorElement,paddingObj),
+	"3" },
+    { NULL }
+};
+
+static void MenuIndicatorElementSize(
+    void *clientData, void *elementRecord, Tk_Window tkwin,
+    int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
+{
+    MenuIndicatorElement *indicator = elementRecord;
+    int size = MENUBUTTON_ARROW_SIZE;
+    Tk_GetPixelsFromObj(NULL, tkwin, indicator->sizeObj, &size);
+    ArrowSize(size, ARROW_DOWN, widthPtr, heightPtr);
+    Ttk_GetPaddingFromObj(NULL, tkwin, indicator->paddingObj, paddingPtr);
+}
+
+static void MenuIndicatorElementDraw(
+    void *clientData, void *elementRecord, Tk_Window tkwin,
+    Drawable d, Ttk_Box b, unsigned int state)
+{
+    MenuIndicatorElement *indicator = elementRecord;
+    XColor *arrowColor = Tk_GetColorFromObj(tkwin, indicator->colorObj);
+    GC gc = Tk_GCForColor(arrowColor, d);
+    int size = MENUBUTTON_ARROW_SIZE;
+    int width, height;
+
+    Tk_GetPixelsFromObj(NULL, tkwin, indicator->sizeObj, &size);
+
+    ArrowSize(size, ARROW_DOWN, &width, &height);
+    b = Ttk_StickBox(b, width, height, 0);
+    FillArrow(Tk_Display(tkwin), d, gc, b, ARROW_DOWN);
+}
+
+static Ttk_ElementSpec MenuIndicatorElementSpec =
+{
+    TK_STYLE_VERSION_2,
+    sizeof(MenuIndicatorElement),
+    MenuIndicatorElementOptions,
+    MenuIndicatorElementSize,
+    MenuIndicatorElementDraw
+};
+
 /*------------------------------------------------------------------------
  * +++ Grips.
  *
@@ -706,7 +765,7 @@ static void ArrowElementDraw(
     ArrowSize(h/2, dir, &cx, &cy);
     b = Ttk_AnchorBox(b, cx, cy, TK_ANCHOR_CENTER);
 
-    DrawArrow(Tk_Display(tkwin), d, gc, b, dir);
+    FillArrow(Tk_Display(tkwin), d, gc, b, dir);
 }
 
 static Ttk_ElementSpec ArrowElementSpec = {
@@ -858,10 +917,7 @@ TTK_END_LAYOUT
 int DLLEXPORT
 ClamTheme_Init(Tcl_Interp *interp)
 {
-    Ttk_Theme theme, parentPtr;
-
-    parentPtr = Ttk_GetTheme(interp, "alt");
-    theme = Ttk_CreateTheme(interp, "clam", parentPtr);
+    Ttk_Theme theme = Ttk_CreateTheme(interp, "clam", 0);
 
     if (!theme) {
         return TCL_ERROR;
@@ -890,6 +946,8 @@ ClamTheme_Init(Tcl_Interp *interp)
 	theme, "Radiobutton.indicator", &RadioIndicatorElementSpec, NULL);
     Ttk_RegisterElement(interp,
 	theme, "Checkbutton.indicator", &CheckIndicatorElementSpec, NULL);
+    Ttk_RegisterElement(interp,
+	theme, "Menubutton.indicator", &MenuIndicatorElementSpec, NULL);
 
     Ttk_RegisterElement(interp, theme, "tab", &TabElementSpec, NULL);
     Ttk_RegisterElement(interp, theme, "client", &ClientElementSpec, NULL);
