@@ -1,5 +1,5 @@
 /*
- * entry.c,v 1.29 2005/05/22 22:26:33 jenglish Exp
+ * entry.c,v 1.35 2005/10/08 14:02:47 jenglish Exp
  *
  * DERIVED FROM: tk/generic/tkEntry.c r1.35.
  *
@@ -8,8 +8,6 @@
  * Copyright (c) 2000 Ajuba Solutions.
  * Copyright (c) 2002 ActiveState Corporation.
  * Copyright (c) 2004 Joe English
- *
- * TODO: Validation system not completely right yet.
  */
 
 #include <string.h>
@@ -133,7 +131,7 @@ typedef struct
     /*
      * Derived resources:
      */
-    TraceHandle *textVariableTrace;
+    Ttk_TraceHandle *textVariableTrace;
     int avgWidth;		/* Average character width of text font */
 
     char *displayString;	/* String to use when displaying */
@@ -156,7 +154,8 @@ typedef struct
 /*
  * Extra mask bits for Tk_SetOptions()
  */
-#define STATE_CHANGED	 	(0x100)		/* -state option changed */
+#define STATE_CHANGED	 	(0x100)	/* -state option changed */
+#define TEXTVAR_CHANGED	 	(0x200)	/* -textvariable option changed */
 
 /*
  * Default option values:
@@ -191,7 +190,7 @@ static Tk_OptionSpec EntryOptionSpecs[] =
         0,0,STATE_CHANGED},
     {TK_OPTION_STRING, "-textvariable", "textVariable", "Variable",
 	NULL, Tk_Offset(Entry, entry.textVariableObj), -1,
-	TK_OPTION_NULL_OK, 0, 0},
+	TK_OPTION_NULL_OK,0,TEXTVAR_CHANGED},
     {TK_OPTION_STRING_TABLE, "-validate", "validate", "Validate",
 	"none", -1, Tk_Offset(Entry, entry.validate),
 	0, (ClientData) validateStrings, 0},
@@ -401,7 +400,7 @@ static void EntryLostSelection(ClientData clientData)
     Entry *entryPtr = (Entry *) clientData;
     entryPtr->core.flags &= ~GOT_SELECTION;
     entryPtr->entry.selectFirst = entryPtr->entry.selectLast = -1;
-    WidgetChanged(&entryPtr->core, REDISPLAY_REQUIRED);
+    TtkRedisplayWidget(&entryPtr->core);
 }
 
 /* EntryOwnSelection --
@@ -664,20 +663,20 @@ static int EntryNeedsValidation(VMODE vmode, VREASON reason)
  *	TCL_BREAK if the change is rejected
  *      TCL_ERROR if any errors occured
  *
- * The change will be rejected if the -validatecommand returns 0,
+ * The change will be rejected if -validatecommand returns 0, 
  * or if -validatecommand or -invalidcommand modifies the value.
  */
 static int
 EntryValidateChange(
     Entry *entryPtr,		/* Entry that needs validation. */
-    const char *new,		/* Potential new value of entry string */
+    const char *newValue,	/* Potential new value of entry string */
     int index,			/* index of insert/delete, -1 otherwise */
     int count,			/* #changed characters */
     VREASON reason)		/* Reason for change */
 {
     Tcl_Interp *interp = entryPtr->core.interp;
     VMODE vmode = entryPtr->entry.validate;
-    int code, ok;
+    int code, change_ok;
 
     if (   (entryPtr->entry.validateCmd == NULL)
 	|| (entryPtr->core.flags & VALIDATING)
@@ -692,11 +691,12 @@ EntryValidateChange(
      */
     code = RunValidationScript(interp, entryPtr,
 	    entryPtr->entry.validateCmd, "-validatecommand",
-	    new, index, count, reason);
-    if (code != TCL_OK)
+	    newValue, index, count, reason);
+    if (code != TCL_OK) {
 	goto done;
+    }
 
-    code = Tcl_GetBooleanFromObj(interp,Tcl_GetObjResult(interp), &ok);
+    code = Tcl_GetBooleanFromObj(interp,Tcl_GetObjResult(interp), &change_ok);
     if (code != TCL_OK) {
 	entryPtr->entry.validate = VMODE_NONE;	/* Disable validation */
 	Tcl_AddErrorInfo(interp,
@@ -706,19 +706,21 @@ EntryValidateChange(
 
     /* Run the -invalidcommand if validation failed:
      */
-    if (!ok && entryPtr->entry.invalidCmd != NULL) {
+    if (!change_ok && entryPtr->entry.invalidCmd != NULL) {
 	code = RunValidationScript(interp, entryPtr,
 		entryPtr->entry.invalidCmd, "-invalidcommand",
-		new, index, count, reason);
-	if (code != TCL_OK)
+		newValue, index, count, reason);
+	if (code != TCL_OK) {
 	    goto done;
+	}
     }
 
-    /* If validation failed, or if a validation script changed the value,
-     * reject the pending change.
+    /* Reject the pending change if validation failed
+     * or if a validation script changed the value.
      */
-    if (!ok || (entryPtr->core.flags & VALIDATION_SET_VALUE))
+    if (!change_ok || (entryPtr->core.flags & VALIDATION_SET_VALUE)) {
 	code = TCL_BREAK;
+    }
 
 done:
     entryPtr->core.flags &= ~(VALIDATING|VALIDATION_SET_VALUE);
@@ -726,18 +728,33 @@ done:
 }
 
 /* EntryRevalidate --
- * 	Revalidate the contents of an entry widget.
+ * 	Revalidate the current value of an entry widget,
+ * 	update the TTK_STATE_INVALID bit.
+ *
+ * Returns:
+ * 	TCL_OK if valid, TCL_BREAK if invalid, TCL_ERROR on error.
  */
-static void EntryRevalidate(Entry *entryPtr, VREASON reason)
+static int EntryRevalidate(Tcl_Interp *interp, Entry *entryPtr, VREASON reason)
 {
-    Tcl_Interp *interp = entryPtr->core.interp;
-    int code;
-
-    code = EntryValidateChange(entryPtr, entryPtr->entry.string, -1,0, reason);
+    int code = EntryValidateChange(
+		    entryPtr, entryPtr->entry.string, -1,0, reason);
 
     if (code == TCL_BREAK) {
-	/* @@@ What should happen here? There's no change to reject. */
-    } else if (code == TCL_ERROR) {
+	WidgetChangeState(&entryPtr->core, TTK_STATE_INVALID, 0);
+    } else if (code == TCL_OK) {
+	WidgetChangeState(&entryPtr->core, 0, TTK_STATE_INVALID);
+    }
+
+    return code;
+}
+
+/* EntryRevalidateBG --
+ * 	Revalidate in the background (called from event handler).
+ */
+static void EntryRevalidateBG(Entry *entryPtr, VREASON reason)
+{
+    Tcl_Interp *interp = entryPtr->core.interp;
+    if (EntryRevalidate(interp, entryPtr, reason) == TCL_ERROR) {
 	Tcl_BackgroundError(interp);
     }
 }
@@ -823,7 +840,7 @@ EntryStoreValue(Entry *entryPtr, const char *value)
     /* Update layout, schedule redisplay:
      */
     EntryUpdateTextLayout(entryPtr);
-    WidgetChanged(&entryPtr->core, REDISPLAY_REQUIRED);
+    TtkRedisplayWidget(&entryPtr->core);
 }
 
 /* EntrySetValue --
@@ -858,26 +875,22 @@ static int EntrySetValue(Entry *entryPtr, const char *value)
 /* EntryTextVariableTrace --
  *	Variable trace procedure for entry -textvariable
  */
-static int
-EntryTextVariableTrace(
-    Tcl_Interp *interp, void *recordPtr, const char *value)
+static void EntryTextVariableTrace(void *recordPtr, const char *value)
 {
     Entry *entryPtr = recordPtr;
 
-    if (WidgetDestroyed(&entryPtr->core))
-	return TCL_ERROR;
+    if (WidgetDestroyed(&entryPtr->core)) {
+	return;
+    }
 
     if (entryPtr->core.flags & SYNCING_VARIABLE) {
 	/* Trace was fired due to Tcl_SetVar call in EntrySetValue.
 	 * Don't do anything.
 	 */
-	return TCL_OK;
+	return;
     }
 
-    /* HERE: Schedule revalidation.  Maybe. */
-
     EntryStoreValue(entryPtr, value ? value : "");
-    return TCL_OK;
 }
 
 /*------------------------------------------------------------------------
@@ -996,10 +1009,10 @@ EntryEventProc(ClientData clientData, XEvent *eventPtr)
 	    entryPtr->core.flags |= UPDATE_SCROLLBAR;
 	    break;
 	case FocusIn:
-	    EntryRevalidate(entryPtr, VALIDATE_FOCUSIN);
+	    EntryRevalidateBG(entryPtr, VALIDATE_FOCUSIN);
 	    break;
 	case FocusOut:
-	    EntryRevalidate(entryPtr, VALIDATE_FOCUSOUT);
+	    EntryRevalidateBG(entryPtr, VALIDATE_FOCUSOUT);
 	    break;
     }
     Tcl_Release(clientData);
@@ -1045,7 +1058,7 @@ EntryCleanup(void *recordPtr)
     Entry *entryPtr = recordPtr;
 
     if (entryPtr->entry.textVariableTrace)
-	UntraceVariable(entryPtr->entry.textVariableTrace);
+	Ttk_UntraceVariable(entryPtr->entry.textVariableTrace);
 
     EntryFreeStyleDefaults(&entryPtr->entry.styleDefaults);
 
@@ -1060,30 +1073,34 @@ EntryCleanup(void *recordPtr)
 /* EntryConfigure --
  * 	Configure hook for Entry widgets.
  */
-static int
-EntryConfigure(Tcl_Interp *interp, void *recordPtr, int mask)
+static int EntryConfigure(Tcl_Interp *interp, void *recordPtr, int mask)
 {
     Entry *entryPtr = recordPtr;
     Tcl_Obj *textVarName = entryPtr->entry.textVariableObj;
-    TraceHandle *vt = 0;
+    Ttk_TraceHandle *vt = 0;
     Tk_Window tkwin = entryPtr->core.tkwin;
     Tk_Font font;
 
-    if (textVarName && *Tcl_GetString(textVarName)) {
-	vt = TraceVariable(interp,textVarName,EntryTextVariableTrace,entryPtr);
-	if (!vt) return TCL_ERROR;
+    if (mask & TEXTVAR_CHANGED) {
+	if (textVarName && *Tcl_GetString(textVarName)) {
+	    vt = Ttk_TraceVariable(interp,
+		    textVarName,EntryTextVariableTrace,entryPtr);
+	    if (!vt) return TCL_ERROR;
+	}
     }
 
     if (CoreConfigure(interp, recordPtr, mask) != TCL_OK) {
-	if (vt) UntraceVariable(vt);
+	if (vt) Ttk_UntraceVariable(vt);
 	return TCL_ERROR;
     }
 
     /* Update derived resources:
      */
-    if (entryPtr->entry.textVariableTrace)
-	UntraceVariable(entryPtr->entry.textVariableTrace);
-    entryPtr->entry.textVariableTrace = vt;
+    if (mask & TEXTVAR_CHANGED) {
+	if (entryPtr->entry.textVariableTrace)
+	    Ttk_UntraceVariable(entryPtr->entry.textVariableTrace);
+	entryPtr->entry.textVariableTrace = vt;
+    }
 
     font = Tk_GetFontFromObj(tkwin, entryPtr->entry.fontObj);
     entryPtr->entry.avgWidth = Tk_TextWidth(font, "0", 1);
@@ -1123,16 +1140,14 @@ EntryConfigure(Tcl_Interp *interp, void *recordPtr, int mask)
 /* EntryPostConfigure --
  * 	Post-configuration hook for entry widgets.
  */
-static int
-EntryPostConfigure(Tcl_Interp *interp, void *recordPtr, int mask)
+static int EntryPostConfigure(Tcl_Interp *interp, void *recordPtr, int mask)
 {
     Entry *entryPtr = recordPtr;
     int status = TCL_OK;
 
-    if (entryPtr->entry.textVariableTrace)
-	status = FireTrace(entryPtr->entry.textVariableTrace);
-
-    /* @@@ Possibly: Revalidate if '-validatecommand' or '-validate' changed */
+    if ((mask & TEXTVAR_CHANGED) && entryPtr->entry.textVariableTrace != NULL) {
+	status = Ttk_FireTrace(entryPtr->entry.textVariableTrace);
+    }
 
     return status;
 }
@@ -1611,7 +1626,7 @@ EntryICursorCommand(
 	    &entryPtr->entry.insertPos) != TCL_OK) {
 	return TCL_ERROR;
     }
-    WidgetChanged(&entryPtr->core, REDISPLAY_REQUIRED);
+    TtkRedisplayWidget(&entryPtr->core);
     return TCL_OK;
 }
 
@@ -1673,7 +1688,7 @@ static int EntrySelectionClearCommand(
 	return TCL_ERROR;
     }
     entryPtr->entry.selectFirst = entryPtr->entry.selectLast = -1;
-    WidgetChanged(&entryPtr->core, REDISPLAY_REQUIRED);
+    TtkRedisplayWidget(&entryPtr->core);
     return TCL_OK;
 }
 
@@ -1734,7 +1749,7 @@ static int EntrySelectionRangeCommand(
 	entryPtr->entry.selectLast = end;
 	EntryOwnSelection(entryPtr);
     }
-    WidgetChanged(&entryPtr->core, REDISPLAY_REQUIRED);
+    TtkRedisplayWidget(&entryPtr->core);
     return TCL_OK;
 }
 
@@ -1788,8 +1803,7 @@ static int EntryValidateCommand(
 	return TCL_ERROR;
     }
 
-    code = EntryValidateChange(
-	    entryPtr, entryPtr->entry.string, -1, 0, VALIDATE_FORCED);
+    code = EntryRevalidate(interp, entryPtr, VALIDATE_FORCED);
 
     if (code == TCL_ERROR)
 	return code;
@@ -1857,7 +1871,7 @@ static int EntryXViewCommand(
     entryPtr->entry.leftIndex = index;
 
     EntryPlaceTextLayout(entryPtr);
-    WidgetChanged(&entryPtr->core, REDISPLAY_REQUIRED);
+    TtkRedisplayWidget(&entryPtr->core);
     return TCL_OK;
 }
 
@@ -1897,8 +1911,7 @@ WidgetSpec EntryWidgetSpec =
     WidgetGetLayout, 		/* getLayoutProc */
     WidgetSize, 		/* sizeProc */
     EntryDoLayout,		/* layoutProc */
-    EntryDisplay,		/* displayProc */
-    WIDGET_SPEC_END		/* sentinel */
+    EntryDisplay		/* displayProc */
 };
 
 /*------------------------------------------------------------------------
@@ -2054,11 +2067,9 @@ WidgetSpec ComboboxWidgetSpec =
     WidgetGetLayout, 		/* getLayoutProc */
     WidgetSize, 		/* sizeProc */
     EntryDoLayout,		/* layoutProc */
-    EntryDisplay,		/* displayProc */
-    WIDGET_SPEC_END		/* sentinel */
+    EntryDisplay		/* displayProc */
 };
 
-
 /*------------------------------------------------------------------------
  * +++ Textarea element.
  *
