@@ -1,11 +1,7 @@
-/* frame.c,v 1.35 2005/05/22 22:20:37 jenglish Exp
+/* frame.c,v 1.41 2005/10/08 14:02:47 jenglish Exp
  * Copyright (c) 2004, Joe English
  *
  * Tile widget set: frame and labelframe widgets
- *
- * Note to maintainers: UNDER NO CIRCUMSTANCES is anyone to add
- * the -colormap, -visual, or -container resources to this widget.
- * Thank you.
  */
 
 #include <tk.h>
@@ -13,6 +9,7 @@
 #include "tkTheme.h"
 #include "widget.h"
 #include "compat.h"
+#include "manager.h"
 
 /* ======================================================================
  * +++ Frame widget:
@@ -36,9 +33,9 @@ typedef struct
 
 static Tk_OptionSpec FrameOptionSpecs[] =
 {
-    {TK_OPTION_PIXELS, "-borderwidth", "borderWidth", "BorderWidth", "0",
+    {TK_OPTION_PIXELS, "-borderwidth", "borderWidth", "BorderWidth", NULL,
 	Tk_Offset(Frame,frame.borderWidthObj), -1,
-	0,0,GEOMETRY_CHANGED },
+	TK_OPTION_NULL_OK,0,GEOMETRY_CHANGED },
     {TK_OPTION_STRING, "-padding", "padding", "Pad", NULL,
 	Tk_Offset(Frame,frame.paddingObj), -1, 
 	TK_OPTION_NULL_OK,0,GEOMETRY_CHANGED },
@@ -70,13 +67,10 @@ static WidgetCommandSpec FrameCommands[] =
  * FrameMargins --
  * 	Compute internal margins for a frame widget.
  * 	This includes the -borderWidth, plus any additional -padding.
- * 	The Labelframe widget adjusts margins further.
  */
 static Ttk_Padding FrameMargins(Frame *framePtr)
 {
     Ttk_Padding margins = Ttk_UniformPadding(0);
-    int borderWidth = 0;
-    Ttk_Padding border;
 
     /* Check -padding:
      */
@@ -88,12 +82,13 @@ static Ttk_Padding FrameMargins(Frame *framePtr)
     /* Add padding for border:
      */
     if (framePtr->frame.borderWidthObj) {
+	int border = 0;
 	Tk_GetPixelsFromObj(NULL, 
-	    framePtr->core.tkwin, framePtr->frame.borderWidthObj, &borderWidth);
-	border = Ttk_UniformPadding((short)borderWidth);
+	    framePtr->core.tkwin, framePtr->frame.borderWidthObj, &border);
+	margins = Ttk_AddPadding(margins, Ttk_UniformPadding((short)border));
     }
 
-    return Ttk_AddPadding(margins, border);
+    return margins;
 }
 
 /* FrameSize procedure --
@@ -103,8 +98,7 @@ static Ttk_Padding FrameMargins(Frame *framePtr)
 static int FrameSize(void *recordPtr, int *widthPtr, int *heightPtr)
 {
     Frame *framePtr = recordPtr;
-    Ttk_Padding margins = FrameMargins(framePtr);
-    Ttk_SetMargins(framePtr->core.tkwin, margins);
+    Ttk_SetMargins(framePtr->core.tkwin, FrameMargins(framePtr));
     return 0;
 }
 
@@ -170,15 +164,16 @@ WidgetSpec FrameWidgetSpec =
     WidgetGetLayout, 		/* getLayoutProc */
     FrameSize,			/* sizeProc */
     WidgetDoLayout,		/* layoutProc */
-    WidgetDisplay,		/* displayProc */
-    WIDGET_SPEC_END		/* sentinel */
+    WidgetDisplay		/* displayProc */
 };
 
 /* ======================================================================
  * +++ Labelframe widget:
  */
 
-#define LABELFRAME_SPACE 4
+#define LABELFRAME_SPACE	4
+#define DEFAULT_LABELINSET	8
+#define DEFAULT_BORDERWIDTH	2
 
 typedef enum {
     LABELANCHOR_E, LABELANCHOR_EN, LABELANCHOR_ES,
@@ -229,9 +224,8 @@ static Ttk_Sticky LabelAnchorSticky(LabelAnchor labelAnchor)
  * Labelframe widget record:
  */
 typedef struct {
-    LabelAnchor	labelAnchor;
+    Tcl_Obj 	*labelAnchorObj;
     Tcl_Obj	*textObj;
-    Tcl_Obj	*fontObj;
     Tcl_Obj 	*underlineObj;
     Tcl_Obj	*labelWidgetObj;
 
@@ -250,14 +244,11 @@ typedef struct
 static Tk_OptionSpec LabelframeOptionSpecs[] =
 {
     {TK_OPTION_STRING_TABLE, "-labelanchor", "labelAnchor", "LabelAnchor",
-	"nw", -1, Tk_Offset(Labelframe, label.labelAnchor),
+	"nw", Tk_Offset(Labelframe, label.labelAnchorObj),-1,
         0, (ClientData) labelAnchorStrings, GEOMETRY_CHANGED},
     {TK_OPTION_STRING, "-text", "text", "Text", "",
 	Tk_Offset(Labelframe,label.textObj), -1, 
 	0,0,GEOMETRY_CHANGED }, 
-    {TK_OPTION_FONT, "-font", "font", "Font", NULL,
-	Tk_Offset(Labelframe,label.fontObj), -1, 
-	TK_OPTION_NULL_OK,0,GEOMETRY_CHANGED },
     {TK_OPTION_INT, "-underline", "underline", "Underline",
 	"-1", Tk_Offset(Labelframe,label.underlineObj), -1, 
 	0,0,0 },
@@ -265,64 +256,58 @@ static Tk_OptionSpec LabelframeOptionSpecs[] =
 	Tk_Offset(Labelframe,label.labelWidgetObj), -1,
 	TK_OPTION_NULL_OK,0,GEOMETRY_CHANGED },
 
-    /*
-     * Override Frame defaults:
-     */
-    {TK_OPTION_RELIEF, "-relief", "relief", "Relief", "groove",
-	Tk_Offset(Labelframe,frame.reliefObj), -1, 
-	TK_OPTION_NULL_OK,0,0 },
-    {TK_OPTION_PIXELS, "-borderwidth", "borderWidth", "BorderWidth", "2",
-	Tk_Offset(Labelframe,frame.borderWidthObj), -1,
-	TK_OPTION_NULL_OK,0,GEOMETRY_CHANGED },
-
     WIDGET_INHERIT_OPTIONS(FrameOptionSpecs)
 };
 
 LABELFRAME_COMPAT_OPTIONS
 
 /*
- * Maintainable(interp, slave, master) --
- * 	Utility routine.  Verifies that 'master' may be used to maintain
- *	the geometry of 'slave' (via Tk_MaintainGeometry):
- * 
- * 	+ 'master' is either 'slave's parent -OR-
- * 	+ 'master is a descendant of 'slave's parent.
- * 	+ 'slave' is not a toplevel window
- * 	+ 'slave' belongs to the same toplevel as 'master'
- *
- * Side effects:
- * 	+ If 'slave' is not a direct child of 'master', raises 'slave'
- * 	  above the sibling window containing 'master' 
+ * Labelframe style parameters:
  */
-
-static int Maintainable(Tcl_Interp *interp, Tk_Window slave, Tk_Window master)
+typedef struct
 {
-    Tk_Window ancestor = master, parent = Tk_Parent(slave), sibling = NULL;
+    int 	borderWidth;	/* border width */
+    Ttk_Padding	padding;	/* internal padding */
+    LabelAnchor	labelAnchor;	/* corner/side to place label */
+    int 	labelOutside;	/* true=>place label outside border */
+    int 	labelInset;	/* distance between label and adjacent corner */
+    int 	labelSpace;	/* distance between label and border */
+} LabelframeStyle;
 
-    if (Tk_IsTopLevel(slave) || slave == master) {
-	goto badWindow;
+static void LabelframeStyleOptions(Labelframe *lf, LabelframeStyle *style)
+{
+    Ttk_Layout layout = lf->core.layout;
+    Tcl_Obj *objPtr;
+
+    style->borderWidth = DEFAULT_BORDERWIDTH;
+    style->padding = Ttk_UniformPadding(0);
+    style->labelAnchor = LABELANCHOR_NW;
+    style->labelOutside = 0;
+    style->labelSpace = 1;
+    style->labelInset = DEFAULT_LABELINSET;
+
+    if ((objPtr = Ttk_QueryOption(layout, "-borderwidth", 0)) != NULL) {
+	Tk_GetPixelsFromObj(NULL, lf->core.tkwin, objPtr, &style->borderWidth);
+    }
+    if ((objPtr = Ttk_QueryOption(layout, "-padding", 0)) != NULL) {
+	Ttk_GetPaddingFromObj(NULL, lf->core.tkwin, objPtr, &style->padding);
+    }
+    if ((objPtr = Ttk_QueryOption(layout,"-labelanchor", 0)) != NULL) {
+	Tcl_GetIndexFromObj(NULL, objPtr, labelAnchorStrings, "label anchor",
+		TCL_EXACT, (int*)&style->labelAnchor);
+    }
+    if ((objPtr = Ttk_QueryOption(layout,"-labeloutside", 0)) != NULL) {
+	Tcl_GetBooleanFromObj(NULL, objPtr, &style->labelOutside);
+    }
+    if ((objPtr = Ttk_QueryOption(layout,"-labelspace", 0)) != NULL) {
+	Tk_GetPixelsFromObj(NULL, lf->core.tkwin, objPtr, &style->labelSpace);
     }
 
-    while (ancestor != parent) {
-	if (Tk_IsTopLevel(ancestor)) {
-	    goto badWindow;
-	}
-	sibling = ancestor;
-	ancestor = Tk_Parent(ancestor);
+    if ((objPtr = Ttk_QueryOption(layout,"-labelinset", 0)) != NULL) {
+	Tk_GetPixelsFromObj(NULL, lf->core.tkwin, objPtr, &style->labelInset);
     }
 
-    if (sibling != NULL) {
-	Tk_RestackWindow(slave, Above, sibling);
-    }
-
-    return 1;
-
-badWindow:
-    Tcl_AppendResult(interp, 
-	"can't use ", Tk_PathName(slave),
-	" as slave of ", Tk_PathName(master),
-	NULL);
-    return 0;
+    return;
 }
 
 /*
@@ -348,7 +333,7 @@ static void
 LabelframeGeometryRequestProc(ClientData clientData, Tk_Window slave)
 {
     Labelframe *lframePtr = clientData;
-    WidgetChanged(&lframePtr->core, RELAYOUT_REQUIRED);
+    TtkResizeWidget(&lframePtr->core);
 }
 
 static void
@@ -365,7 +350,7 @@ LabelframeGeometryLostSlaveProc(ClientData clientData, Tk_Window slave)
 	slave, StructureNotifyMask, LabelframeSlaveEventHandler, clientData);
     Tk_UnmaintainGeometry(slave, lframePtr->core.tkwin);
 
-    WidgetChanged(&lframePtr->core, RELAYOUT_REQUIRED);
+    TtkResizeWidget(&lframePtr->core);
 }
 
 static Tk_GeomMgr LabelframeGeometryManager = {
@@ -455,11 +440,16 @@ static int LabelframeSize(void *recordPtr, int *widthPtr, int *heightPtr)
 {
     Labelframe *lframePtr = recordPtr;
     WidgetCore *corePtr = &lframePtr->core;
-    Ttk_Padding margins = FrameMargins((Frame*)recordPtr);
-    Tcl_Obj *outsideObj = Ttk_QueryOption(corePtr->layout,"-labeloutside", 0);
-    int labelOutside = 0;
+    Ttk_Padding margins;
+    LabelframeStyle style;
     int labelWidth, labelHeight;
-    int borderWidth = 2;
+
+    LabelframeStyleOptions(lframePtr, &style);
+
+    /* Compute base margins (See also: FrameMargins)
+     */
+    margins = Ttk_AddPadding(
+		style.padding, Ttk_UniformPadding((short)style.borderWidth));
 
     /* Adjust margins based on label size and position:
      */
@@ -467,17 +457,11 @@ static int LabelframeSize(void *recordPtr, int *widthPtr, int *heightPtr)
 
     /* Adjust for -labeloutside / -labelspace:
     */
-    if (outsideObj)
-	Tcl_GetBooleanFromObj(NULL, outsideObj, &labelOutside);
-    if (labelOutside) {
-	Tcl_Obj *labelspaceObj=Ttk_QueryOption(corePtr->layout,"-labelspace",0);
-	int labelspace = 1;
-	if (labelspaceObj)
-	    Tk_GetPixelsFromObj(NULL,corePtr->tkwin,labelspaceObj,&labelspace);
-	labelHeight += labelspace;
+    if (style.labelOutside) {
+	labelHeight += style.labelSpace;
     }
 
-    switch (LabelAnchorSide(lframePtr->label.labelAnchor)) {
+    switch (LabelAnchorSide(style.labelAnchor)) {
 	case TTK_SIDE_LEFT:	margins.left   += labelWidth;	break;
 	case TTK_SIDE_RIGHT:	margins.right  += labelWidth;	break;
 	case TTK_SIDE_TOP:	margins.top    += labelHeight;	break;
@@ -488,30 +472,24 @@ static int LabelframeSize(void *recordPtr, int *widthPtr, int *heightPtr)
 
     /* Request minimum size based on border width and label size:
      */
-    Tk_GetPixelsFromObj(NULL, 
-	    corePtr->tkwin, lframePtr->frame.borderWidthObj, &borderWidth);
     Tk_SetMinimumRequestSize(corePtr->tkwin, 
-	    labelWidth + 2*borderWidth + LABELFRAME_SPACE,
-	    labelHeight + 2*borderWidth + LABELFRAME_SPACE);
+	    labelWidth + 2*style.borderWidth + LABELFRAME_SPACE,
+	    labelHeight + 2*style.borderWidth + LABELFRAME_SPACE);
 
     return 0;
 }
 
+
 static void LabelframeDoLayout(void *recordPtr)
 {
     Labelframe *lframePtr = recordPtr;
     WidgetCore *corePtr = &lframePtr->core;
     Ttk_Box borderParcel = Ttk_WinBox(corePtr->tkwin);
-    Ttk_Side side = LabelAnchorSide(lframePtr->label.labelAnchor);
-    Ttk_Sticky sticky = LabelAnchorSticky(lframePtr->label.labelAnchor);
     Ttk_LayoutNode
 	*textNode = Ttk_LayoutFindNode(corePtr->layout, "text"),
 	*borderNode = Ttk_LayoutFindNode(corePtr->layout, "border");
-    Tcl_Obj *outsideObj = Ttk_QueryOption(corePtr->layout,"-labeloutside", 0);
-    Tcl_Obj *insetObj = Ttk_QueryOption(corePtr->layout,"-labelinset", 0);
-    int labelOutside = 0; 	/* true=>place label outside border */
-    int labelinset = 0;		/* distance between label and adjacent corner */
     int lw, lh;			/* Label width and height */
+    LabelframeStyle style;
     Ttk_Box labelParcel;
 
     /*
@@ -522,19 +500,20 @@ static void LabelframeDoLayout(void *recordPtr)
     /*
      * Compute label parcel:
      */
+    LabelframeStyleOptions(lframePtr, &style);
     LabelframeLabelSize(lframePtr, &lw, &lh);
-    labelParcel = Ttk_PlaceBox(&borderParcel, lw, lh, side, sticky);
+    labelParcel = Ttk_PlaceBox(
+	    &borderParcel, lw, lh,
+	    LabelAnchorSide(style.labelAnchor),
+	    LabelAnchorSticky(style.labelAnchor));
 
     /*
      * Place label on top of border, unless -labeloutside is set:
      */
-    if (outsideObj) {
-	Tcl_GetBooleanFromObj(NULL, outsideObj, &labelOutside);
-    }
-    if (!labelOutside) {
+    if (!style.labelOutside) {
 	/* Move border edge so it's over label:
 	*/
-	switch (side) {
+	switch (LabelAnchorSide(style.labelAnchor)) {
 	    case TTK_SIDE_LEFT: 	borderParcel.x -= lw / 2;
 	    case TTK_SIDE_RIGHT:	borderParcel.width += lw/2; 	break;
 	    case TTK_SIDE_TOP:  	borderParcel.y -= lh / 2;
@@ -543,43 +522,22 @@ static void LabelframeDoLayout(void *recordPtr)
     } else {
 	/* Add extra space between label and border:
 	 */
-	Tcl_Obj *labelspaceObj=Ttk_QueryOption(corePtr->layout,"-labelspace",0);
-	int labelspace = 1;
-	if (labelspaceObj)
-	    Tk_GetPixelsFromObj(NULL,corePtr->tkwin,labelspaceObj,&labelspace);
-
-	borderParcel.y += labelspace;
-	borderParcel.height -= labelspace;
+	borderParcel.y += style.labelSpace;
+	borderParcel.height -= style.labelSpace;
     }
 
     /* Move label away from corner:
     */
-    if (insetObj) {
-	/* Inset specified by style:
-	 */
-	Tk_GetPixelsFromObj(NULL, corePtr->tkwin, insetObj, &labelinset);
-    } else if (!labelOutside) {		
-	/* Compute default based on -borderwidth:
-	 */
-	Tk_GetPixelsFromObj(NULL, 
-	    corePtr->tkwin, lframePtr->frame.borderWidthObj, &labelinset);
-	labelinset = labelinset * 2 + LABELFRAME_SPACE;
-    } else {			
-	/* No extra inset
-	 */
-	labelinset = 0;
-    }
-
-    switch (lframePtr->label.labelAnchor) {
-	default:			break; /*no-op*/
+    switch (style.labelAnchor) {
+	default:		break; /*no-op*/
 	case LABELANCHOR_NW:
-	case LABELANCHOR_SW:	labelParcel.x += labelinset; break;
+	case LABELANCHOR_SW:	labelParcel.x += style.labelInset; break;
 	case LABELANCHOR_NE:
-	case LABELANCHOR_SE:	labelParcel.x -= labelinset; break;
+	case LABELANCHOR_SE:	labelParcel.x -= style.labelInset; break;
 	case LABELANCHOR_EN:
-	case LABELANCHOR_WN:	labelParcel.y += labelinset; break;
+	case LABELANCHOR_WN:	labelParcel.y += style.labelInset; break;
 	case LABELANCHOR_ES:
-	case LABELANCHOR_WS:	labelParcel.y -= labelinset; break;
+	case LABELANCHOR_WS:	labelParcel.y -= style.labelInset; break;
     }
 
     /* 
@@ -613,7 +571,7 @@ static int LabelframeConfigure(Tcl_Interp *interp,void *recordPtr,int mask)
     if (lframePtr->label.labelWidgetObj) {
 	const char *pathName = Tcl_GetString(lframePtr->label.labelWidgetObj);
 	if (pathName && *pathName) {
-	    labelWidget = 
+	    labelWidget =
 		Tk_NameToWindow(interp, pathName, lframePtr->core.tkwin);
 	    if (!labelWidget) {
 		return TCL_ERROR;
@@ -659,7 +617,6 @@ WidgetSpec LabelframeWidgetSpec =
     WidgetGetLayout, 		/* getLayoutProc */
     LabelframeSize,		/* sizeProc */
     LabelframeDoLayout,		/* layoutProc */
-    WidgetDisplay,		/* displayProc */
-    WIDGET_SPEC_END		/* sentinel */
+    WidgetDisplay		/* displayProc */
 };
 
