@@ -1,9 +1,9 @@
-/* label.c,v 1.11 2005/03/31 17:27:10 jenglish Exp
+/* label.c,v 1.14 2005/10/08 15:53:42 jenglish Exp
  *
  * Tile widget set: text, image, and label elements.
  *
  * The label element combines text and image elements,
- * with layout determined by the "-compound" option.  
+ * with layout determined by the "-compound" option.
  *
  */
 
@@ -24,7 +24,7 @@
 typedef struct
 {
     /*
-     * Element resources:
+     * Element options:
      */
     Tcl_Obj	*textObj;
     Tcl_Obj	*fontObj;
@@ -171,7 +171,7 @@ static void TextDraw(TextElement *text, Tk_Window tkwin, Drawable d, Ttk_Box b)
     Tk_FreeGC(Tk_Display(tkwin), gc2);
 }
 
-static void TextElementGeometry(
+static void TextElementSize(
     void *clientData, void *elementRecord, Tk_Window tkwin,
     int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
 {
@@ -190,7 +190,7 @@ static void TextElementGeometry(
 
 static void TextElementDraw(
     void *clientData, void *elementRecord, Tk_Window tkwin,
-    Drawable d, Ttk_Box b, unsigned int state)
+    Drawable d, Ttk_Box b, Ttk_State state)
 {
     TextElement *text = elementRecord;
     if (TextSetup(text, tkwin)) {
@@ -204,7 +204,7 @@ static void TextElementDraw(
     TK_STYLE_VERSION_2,
     sizeof(TextElement),
     TextElementOptions,
-    TextElementGeometry,
+    TextElementSize,
     TextElementDraw
 };
 
@@ -214,7 +214,7 @@ static void TextElementDraw(
  */
 static void ImageTextElementDraw(
     void *clientData, void *elementRecord, Tk_Window tkwin,
-    Drawable d, Ttk_Box b, unsigned int state)
+    Drawable d, Ttk_Box b, Ttk_State state)
 {
     TextElement *text = elementRecord;
     Tk_3DBorder bd = Tk_Get3DBorderFromObj(tkwin,text->backgroundObj);
@@ -235,7 +235,7 @@ static void ImageTextElementDraw(
     TK_STYLE_VERSION_2,
     sizeof(TextElement),
     TextElementOptions,
-    TextElementGeometry,
+    TextElementSize,
     ImageTextElementDraw
 };
 
@@ -292,8 +292,8 @@ static void NullImageChanged(ClientData clientData,
  * 	or the image resource was not specified.
  */
 
-static int ImageSetup(ImageElement *image, Tk_Window tkwin, Tcl_Interp *interp,
-    unsigned state)
+static int ImageSetup(
+    ImageElement *image, Tk_Window tkwin, Tcl_Interp *interp, Ttk_State state)
 {
     const char *imageName;
     Tcl_Obj *imageObj = image->imageObj;
@@ -357,7 +357,7 @@ static void ImageCleanup(ImageElement *image)
  */
 static void StippleOver(
     ImageElement *image, Tk_Window tkwin, Drawable d, int x, int y)
-{ 
+{
     Pixmap stipple = Tk_AllocBitmapFromObj(NULL, tkwin, image->stippleObj);
     XColor *color = Tk_GetColorFromObj(tkwin, image->backgroundObj);
 
@@ -376,7 +376,7 @@ static void StippleOver(
 }
 
 static void ImageDraw(
-    ImageElement *image, Tk_Window tkwin, Drawable d, Ttk_Box b, unsigned state)
+    ImageElement *image, Tk_Window tkwin,Drawable d,Ttk_Box b,Ttk_State state)
 {
     int width = image->width, height = image->height;
 
@@ -396,7 +396,7 @@ static void ImageDraw(
     }
 }
 
-static void ImageElementGeometry(
+static void ImageElementSize(
     void *clientData, void *elementRecord, Tk_Window tkwin,
     int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
 {
@@ -412,7 +412,7 @@ static void ImageElementGeometry(
 
 static void ImageElementDraw(
     void *clientData, void *elementRecord, Tk_Window tkwin,
-    Drawable d, Ttk_Box b, unsigned int state)
+    Drawable d, Ttk_Box b, Ttk_State state)
 {
     ImageElement *image = elementRecord;
     Tcl_Interp *interp = clientData;
@@ -428,7 +428,7 @@ static void ImageElementDraw(
     TK_STYLE_VERSION_2,
     sizeof(ImageElement),
     ImageElementOptions,
-    ImageElementGeometry,
+    ImageElementSize,
     ImageElementDraw
 };
 
@@ -455,6 +455,13 @@ static void ImageElementDraw(
  * space to leave between the text and image; Tk 8.4 reuses the
  * -padx or -pady option for this purpose.
  *
+ * -width always specifies the length in characters of the text part;
+ *  in Tk 8.4 it's either characters or pixels, depending on the
+ *  value of -compound.
+ *
+ * Negative values of -width are interpreted as a minimum width
+ * on all platforms, not just on Windows.
+ *
  * Tk 8.4 ignores -padx and -pady if -compound is set to "none".
  * Here, padding is handled by a different element.
  */
@@ -473,7 +480,7 @@ typedef struct
     /*
      * Computed values (see LabelSetup)
      */
-    Ttk_CompoundLayout 	layout;
+    Ttk_Compound	compound;
     int  		space;
     int 		totalWidth, totalHeight;
 } LabelElement;
@@ -531,60 +538,60 @@ static Ttk_ElementOptionSpec LabelElementOptions[] =
 
 #define MAX(a,b) ((a) > (b) ? a : b);
 static void LabelSetup(
-    LabelElement *c, Tk_Window tkwin, Tcl_Interp *interp, unsigned state)
+    LabelElement *c, Tk_Window tkwin, Tcl_Interp *interp, Ttk_State state)
 {
     Tk_GetPixelsFromObj(NULL,tkwin,c->spaceObj,&c->space);
-    Ttk_GetCompoundLayoutFromObj(NULL,c->compoundObj,(int*)&c->layout);
+    Ttk_GetCompoundFromObj(NULL,c->compoundObj,(int*)&c->compound);
 
     /*
-     * Deal with TK_COMPOUND_LAYOUT_NONE.
+     * Deal with TTK_COMPOUND_NONE.
      */
-    if (c->layout == TK_COMPOUND_LAYOUT_NONE) {
+    if (c->compound == TTK_COMPOUND_NONE) {
 	if (ImageSetup(&c->image, tkwin, interp, state)) {
-	    c->layout = TK_COMPOUND_LAYOUT_IMAGE;
+	    c->compound = TTK_COMPOUND_IMAGE;
 	} else {
-	    c->layout = TK_COMPOUND_LAYOUT_TEXT;
+	    c->compound = TTK_COMPOUND_TEXT;
 	}
-    } else if (c->layout != TK_COMPOUND_LAYOUT_TEXT) {
+    } else if (c->compound != TTK_COMPOUND_TEXT) {
     	if (!ImageSetup(&c->image, tkwin, interp, state)) {
-	    c->layout = TK_COMPOUND_LAYOUT_TEXT;
+	    c->compound = TTK_COMPOUND_TEXT;
 	}
     }
-    if (c->layout != TK_COMPOUND_LAYOUT_IMAGE)
+    if (c->compound != TTK_COMPOUND_IMAGE)
 	TextSetup(&c->text, tkwin);
 
     /*
      * ASSERT:
-     * if c->layout != LAYOUT_IMAGE, then TextSetup() has been called
-     * if c->layout != LAYOUT_TEXT, then ImageSetup() has returned successfully
-     * c->layout != LAYOUT_NONE.
+     * if c->compound != IMAGE, then TextSetup() has been called
+     * if c->compound != TEXT, then ImageSetup() has returned successfully
+     * c->compound != COMPOUND_NONE.
      */
 
-    switch (c->layout)
+    switch (c->compound)
     {
-	case TK_COMPOUND_LAYOUT_NONE:
-	    Tcl_Panic("This can't happen");
+	case TTK_COMPOUND_NONE:
+	    /* Can't happen */
 	    break;
-	case TK_COMPOUND_LAYOUT_TEXT:
+	case TTK_COMPOUND_TEXT:
 	    c->totalWidth  = c->text.width;
 	    c->totalHeight = c->text.height;
 	    break;
-	case TK_COMPOUND_LAYOUT_IMAGE:
+	case TTK_COMPOUND_IMAGE:
 	    c->totalWidth  = c->image.width;
 	    c->totalHeight = c->image.height;
 	    break;
-	case TK_COMPOUND_LAYOUT_CENTER:
+	case TTK_COMPOUND_CENTER:
 	    c->totalWidth  = MAX(c->image.width, c->text.width);
 	    c->totalHeight = MAX(c->image.height, c->text.height);
 	    break;
-	case TK_COMPOUND_LAYOUT_TOP:
-	case TK_COMPOUND_LAYOUT_BOTTOM:
+	case TTK_COMPOUND_TOP:
+	case TTK_COMPOUND_BOTTOM:
 	    c->totalWidth  = MAX(c->image.width, c->text.width);
 	    c->totalHeight = c->image.height + c->text.height + c->space;
 	    break;
 
-	case TK_COMPOUND_LAYOUT_LEFT:
-	case TK_COMPOUND_LAYOUT_RIGHT:
+	case TTK_COMPOUND_LEFT:
+	case TTK_COMPOUND_RIGHT:
 	    c->totalWidth  = c->image.width + c->text.width + c->space;
 	    c->totalHeight = MAX(c->image.height, c->text.height);
 	    break;
@@ -593,45 +600,60 @@ static void LabelSetup(
 
 static void LabelCleanup(LabelElement *c)
 {
-    if (c->layout != TK_COMPOUND_LAYOUT_TEXT)
+    if (c->compound != TTK_COMPOUND_TEXT)
 	ImageCleanup(&c->image);
-    if (c->layout != TK_COMPOUND_LAYOUT_IMAGE)
+    if (c->compound != TTK_COMPOUND_IMAGE)
 	TextCleanup(&c->text);
 }
 
-static void LabelElementGeometry(
+static void LabelElementSize(
     void *clientData, void *elementRecord, Tk_Window tkwin,
     int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
 {
     LabelElement *label = elementRecord;
     Tcl_Interp *interp = clientData;
+    int textReqWidth = 0;
 
     LabelSetup(label, tkwin, interp, 0);
-    if (label->layout == TK_COMPOUND_LAYOUT_TEXT) {
-	*widthPtr = TextReqWidth(&label->text);
-    } else {
-	*widthPtr = label->totalWidth;
-    }
-    *heightPtr = label->totalHeight;
-    LabelCleanup(label);
-}
 
-/*
- * Ttk_PlaceBox --
- * 	Combine Ttk_PackBox() and Ttk_StickBox().
- */
-Ttk_Box Ttk_PlaceBox(Ttk_Box *cavity, int width, int height, Ttk_Side side, unsigned sticky)
-{
-    return Ttk_StickBox(Ttk_PackBox(cavity, width, height, side), width, height, sticky);
+    *heightPtr = label->totalHeight;
+
+    /* Requested width based on -width option, not actual text width:
+     */
+    if (label->compound != TTK_COMPOUND_IMAGE)
+	textReqWidth = TextReqWidth(&label->text);
+
+    switch (label->compound) 
+    {
+	case TTK_COMPOUND_TEXT:
+	    *widthPtr = textReqWidth;
+	    break;
+	case TTK_COMPOUND_IMAGE:
+	    *widthPtr = label->image.width;
+	    break;
+	case TTK_COMPOUND_TOP:
+	case TTK_COMPOUND_BOTTOM:
+	case TTK_COMPOUND_CENTER:
+	    *widthPtr = MAX(label->image.width, textReqWidth); 
+	    break;
+	case TTK_COMPOUND_LEFT:
+	case TTK_COMPOUND_RIGHT:
+	    *widthPtr = label->image.width + textReqWidth + label->space; 
+	    break;
+	case TTK_COMPOUND_NONE:
+	    break; /* Can't happen */
+    }
+
+    LabelCleanup(label);
 }
 
 /*
  * DrawCompound --
  * 	Helper routine for LabelElementDraw;
- * 	Handles layout for -compound {left,right,top,bottom} 
+ * 	Handles layout for -compound {left,right,top,bottom}
  */
 static void DrawCompound(
-    LabelElement *l, Ttk_Box b, Tk_Window tkwin, Drawable d, unsigned state,
+    LabelElement *l, Ttk_Box b, Tk_Window tkwin, Drawable d, Ttk_State state,
     int imageSide, int textSide)
 {
     Ttk_Box imageBox =
@@ -644,7 +666,7 @@ static void DrawCompound(
 
 static void LabelElementDraw(
     void *clientData, void *elementRecord, Tk_Window tkwin,
-    Drawable d, Ttk_Box b, unsigned int state)
+    Drawable d, Ttk_Box b, Ttk_State state)
 {
     LabelElement *l = elementRecord;
     Tcl_Interp *interp = clientData;
@@ -661,18 +683,18 @@ static void LabelElementDraw(
     /*
      * Draw text and/or image parts based on -compound:
      */
-    switch (l->layout)
+    switch (l->compound)
     {
-	case TK_COMPOUND_LAYOUT_NONE:
-	    Tcl_Panic("This can't happen");
+	case TTK_COMPOUND_NONE:
+	    /* Can't happen */
 	    break;
-	case TK_COMPOUND_LAYOUT_TEXT:
+	case TTK_COMPOUND_TEXT:
 	    TextDraw(&l->text,tkwin,d,b);
 	    break;
-	case TK_COMPOUND_LAYOUT_IMAGE:
+	case TTK_COMPOUND_IMAGE:
 	    ImageDraw(&l->image,tkwin,d,b,state);
 	    break;
-	case TK_COMPOUND_LAYOUT_CENTER: 
+	case TTK_COMPOUND_CENTER:
 	{
 	    Ttk_Box pb = Ttk_AnchorBox(
 		b, l->image.width, l->image.height, TK_ANCHOR_CENTER);
@@ -682,16 +704,16 @@ static void LabelElementDraw(
 	    TextDraw(&l->text, tkwin, d, pb);
 	    break;
 	}
-	case TK_COMPOUND_LAYOUT_TOP:
+	case TTK_COMPOUND_TOP:
 	    DrawCompound(l, b, tkwin, d, state, TTK_SIDE_TOP, TTK_SIDE_BOTTOM);
 	    break;
-	case TK_COMPOUND_LAYOUT_BOTTOM:
+	case TTK_COMPOUND_BOTTOM:
 	    DrawCompound(l, b, tkwin, d, state, TTK_SIDE_BOTTOM, TTK_SIDE_TOP);
 	    break;
-	case TK_COMPOUND_LAYOUT_LEFT:
+	case TTK_COMPOUND_LEFT:
 	    DrawCompound(l, b, tkwin, d, state, TTK_SIDE_LEFT, TTK_SIDE_RIGHT);
 	    break;
-	case TK_COMPOUND_LAYOUT_RIGHT:
+	case TTK_COMPOUND_RIGHT:
 	    DrawCompound(l, b, tkwin, d, state, TTK_SIDE_RIGHT, TTK_SIDE_LEFT);
 	    break;
     }
@@ -704,7 +726,7 @@ static void LabelElementDraw(
     TK_STYLE_VERSION_2,
     sizeof(LabelElement),
     LabelElementOptions,
-    LabelElementGeometry,
+    LabelElementSize,
     LabelElementDraw
 };
 
