@@ -5,7 +5,7 @@
  *
  * Copyright (c) 2003 Joe English.  Freely redistributable.
  *
- * layout.c,v 1.61 2004/12/31 04:09:53 jenglish Exp
+ * layout.c,v 1.65 2005/09/04 18:05:43 jenglish Exp
  */
 
 #include <string.h>
@@ -34,10 +34,10 @@ int Ttk_BoxContains(Ttk_Box box, int x, int y)
 
 /*
  * packTop, packBottom, packLeft, packRight --
- * 	Carve out a parcel of the specified height (resp width) 
+ * 	Carve out a parcel of the specified height (resp width)
  * 	from the specified cavity.
  *
- * Returns: 
+ * Returns:
  * 	The new parcel.
  *
  * Side effects:
@@ -77,7 +77,7 @@ static Ttk_Box packRight(Ttk_Box *cavity, int width)
 {
     width = MIN(width, cavity->width);
     cavity->width -= width;
-    return Ttk_MakeBox(cavity->x + cavity->width, 
+    return Ttk_MakeBox(cavity->x + cavity->width,
 	    cavity->y, width, cavity->height);
 }
 
@@ -86,7 +86,7 @@ static Ttk_Box packRight(Ttk_Box *cavity, int width)
  * 	Carve out a parcel of the specified size on the specified side
  * 	in the specified cavity.
  *
- * Returns: 
+ * Returns:
  * 	The new parcel.
  *
  * Side effects:
@@ -114,6 +114,8 @@ Ttk_Box Ttk_PadBox(Ttk_Box b, Ttk_Padding p)
     b.y += p.top;
     b.width -= (p.left + p.right);
     b.height -= (p.top + p.bottom);
+    if (b.width <= 0) b.width = 1;
+    if (b.height <= 0) b.height = 1;
     return b;
 }
 
@@ -148,7 +150,7 @@ Ttk_Box Ttk_StickBox(Ttk_Box parcel, int width, int height, unsigned sticky)
     /*
      * X coordinate adjustment:
      */
-    switch (sticky & (TTK_STICK_W | TTK_STICK_E)) 
+    switch (sticky & (TTK_STICK_W | TTK_STICK_E))
     {
 	case TTK_STICK_W | TTK_STICK_E:
 	    /* no-op -- use entire parcel width */
@@ -169,7 +171,7 @@ Ttk_Box Ttk_StickBox(Ttk_Box parcel, int width, int height, unsigned sticky)
     /*
      * Y coordinate adjustment:
      */
-    switch (sticky & (TTK_STICK_N | TTK_STICK_S)) 
+    switch (sticky & (TTK_STICK_N | TTK_STICK_S))
     {
 	case TTK_STICK_N | TTK_STICK_S:
 	    /* use entire parcel height */
@@ -218,6 +220,17 @@ Ttk_Box Ttk_AnchorBox(Ttk_Box parcel, int width, int height, int anchor)
     return Ttk_StickBox(parcel, width, height, AnchorToSticky(anchor));
 }
 
+/*
+ * Ttk_PlaceBox --
+ * 	Combine Ttk_PackBox() and Ttk_StickBox().
+ */
+Ttk_Box Ttk_PlaceBox(
+    Ttk_Box *cavity, int width, int height, Ttk_Side side, unsigned sticky)
+{
+    return Ttk_StickBox(
+	    Ttk_PackBox(cavity, width, height, side), width, height, sticky);
+}
+
 /*
  * TTKInitPadding --
  * 	Common factor of Ttk_GetPaddingFromObj and Ttk_GetBorderFromObj.
@@ -239,7 +252,6 @@ static void TTKInitPadding(int padc, int pixels[4], Ttk_Padding *pad)
     pad->bottom	= (short)pixels[3];
 }
 
-
 /*
  * Ttk_GetPaddingFromObj --
  *
@@ -376,7 +388,7 @@ Ttk_Padding Ttk_AddPadding(Ttk_Padding p1, Ttk_Padding p2)
 
 /* Ttk_RelievePadding --
  * 	Add an extra n pixels of padding according to specified relief.
- * 	This may be used in element geometry procedures to simulate 
+ * 	This may be used in element geometry procedures to simulate
  * 	a "pressed-in" look for pushbuttons.
  */
 Ttk_Padding Ttk_RelievePadding(Ttk_Padding padding, int relief, int n)
@@ -409,12 +421,15 @@ Ttk_Padding Ttk_RelievePadding(Ttk_Padding padding, int relief, int n)
  * 	Returns a stickiness specification from the specified Tcl_Obj*,
  * 	consisting of any combination of n, s, e, and w.
  *
- * Returns: TCL_OK, or TCL_ERROR.  If interp is non-NULL
+ * Returns: TCL_OK if objPtr holds a valid stickiness specification,
+ *	otherwise TCL_ERROR.  interp is used for error reporting if non-NULL.
+ *
  */
-int Ttk_GetStickyFromObj(Tcl_Interp *interp, Tcl_Obj *objPtr, unsigned *result)
+int Ttk_GetStickyFromObj(
+    Tcl_Interp *interp, Tcl_Obj *objPtr, Ttk_Sticky *result)
 {
     const char *string = Tcl_GetString(objPtr);
-    unsigned sticky = 0;
+    Ttk_Sticky sticky = 0;
     char c;
 
     while ((c = *string++) != '\0') {
@@ -426,7 +441,7 @@ int Ttk_GetStickyFromObj(Tcl_Interp *interp, Tcl_Obj *objPtr, unsigned *result)
 	    default:
 	    	if (interp) {
 		    Tcl_ResetResult(interp);
-		    Tcl_AppendResult(interp, 
+		    Tcl_AppendResult(interp,
 			"Bad -sticky specification ",
 			Tcl_GetString(objPtr),
 			NULL);
@@ -435,10 +450,27 @@ int Ttk_GetStickyFromObj(Tcl_Interp *interp, Tcl_Obj *objPtr, unsigned *result)
 	}
     }
 
-    *result = sticky; 
+    *result = sticky;
     return TCL_OK;
 }
 
+/* Ttk_NewStickyObj --
+ * 	Construct a new Tcl_Obj * containing a stickiness specification.
+ */
+Tcl_Obj *Ttk_NewStickyObj(Ttk_Sticky sticky)
+{
+    char buf[5];
+    char *p = buf;
+
+    if (sticky & TTK_STICK_N)	*p++ = 'n';
+    if (sticky & TTK_STICK_S)	*p++ = 's';
+    if (sticky & TTK_STICK_W)	*p++ = 'w';
+    if (sticky & TTK_STICK_E)	*p++ = 'e';
+
+    *p = '\0';
+    return Tcl_NewStringObj(buf, p - buf);
+}
+
 /*------------------------------------------------------------------------
  * +++ Layout nodes.
  */
@@ -505,7 +537,7 @@ void Ttk_FreeLayoutTemplate(Ttk_LayoutTemplate op)
 }
 
 /* InstantiateLayout --
- *	Create a layout tree from a template. 
+ *	Create a layout tree from a template.
  */
 static Ttk_LayoutNode *
 Ttk_InstantiateLayout(Ttk_Theme theme, Ttk_TemplateNode *op)
@@ -530,16 +562,17 @@ Ttk_InstantiateLayout(Ttk_Theme theme, Ttk_TemplateNode *op)
  * Syntax:
  * 	layoutSpec ::= { elementName ?-option value ...? }+
  */
+
+/* NB: This must match bit definitions TTK_PACK_LEFT etc. */
+static const char *packSideStrings[] =
+    { "left", "right", "top", "bottom", NULL };
+
 Ttk_LayoutTemplate Ttk_ParseLayoutTemplate(Tcl_Interp *interp, Tcl_Obj *objPtr)
 {
     enum {  OP_SIDE, OP_STICKY, OP_EXPAND, OP_BORDER, OP_UNIT, OP_CHILDREN };
     static const char *optStrings[] = {
 	    "-side", "-sticky", "-expand", "-border", "-unit", "-children", 0 };
 
-    /* NB: This must match bit definitions TTK_PACK_LEFT etc. */
-    static const char *sideStrings[] = 
-	{ "left", "right", "top", "bottom", NULL };
-
     int i = 0, objc;
     Tcl_Obj **objv;
     Ttk_TemplateNode *head = 0, *tail = 0;
@@ -560,11 +593,11 @@ Ttk_LayoutTemplate Ttk_ParseLayoutTemplate(Tcl_Interp *interp, Tcl_Obj *objPtr)
 	    const char *optName = Tcl_GetString(objv[i]);
 	    int option, value;
 
-	    if (optName[0] != '-') 
+	    if (optName[0] != '-')
 		break;
 
 	    if (Tcl_GetIndexFromObj(
-		    interp, objv[i], optStrings, "option", 0, &option) 
+		    interp, objv[i], optStrings, "option", 0, &option)
 		!= TCL_OK)
 	    {
 		goto error;
@@ -572,15 +605,15 @@ Ttk_LayoutTemplate Ttk_ParseLayoutTemplate(Tcl_Interp *interp, Tcl_Obj *objPtr)
 
 	    if (++i >= objc) {
 		Tcl_ResetResult(interp);
-		Tcl_AppendResult(interp, 
+		Tcl_AppendResult(interp,
 			"Missing value for option ",Tcl_GetString(objv[i-1]),
 			NULL);
 		goto error;
 	    }
 
 	    switch (option) {
-		case OP_SIDE:
-		    if (Tcl_GetIndexFromObj(interp, objv[i], sideStrings, 
+		case OP_SIDE:	/* <<NOTE-PACKSIDE>> */
+		    if (Tcl_GetIndexFromObj(interp, objv[i], packSideStrings,
 				"side", 0, &value) != TCL_OK)
 		    {
 			goto error;
@@ -592,22 +625,22 @@ Ttk_LayoutTemplate Ttk_ParseLayoutTemplate(Tcl_Interp *interp, Tcl_Obj *objPtr)
 		    if (Ttk_GetStickyFromObj(interp,objv[i],&sticky) != TCL_OK)
 			goto error;
 		    break;
-		case OP_EXPAND: 
+		case OP_EXPAND:
 		    if (Tcl_GetBooleanFromObj(interp,objv[i],&value) != TCL_OK)
 			goto error;
-		    if (value) 
+		    if (value)
 			flags |= TTK_EXPAND;
 		    break;
-		case OP_BORDER: 
+		case OP_BORDER:
 		    if (Tcl_GetBooleanFromObj(interp,objv[i],&value) != TCL_OK)
 			goto error;
-		    if (value) 
+		    if (value)
 			flags |= TTK_BORDER;
 		    break;
-		case OP_UNIT: 
+		case OP_UNIT:
 		    if (Tcl_GetBooleanFromObj(interp,objv[i],&value) != TCL_OK)
 			goto error;
-		    if (value) 
+		    if (value)
 			flags |= TTK_UNIT;
 		    break;
 		case OP_CHILDREN:
@@ -642,7 +675,7 @@ error:
 }
 
 /* Ttk_BuildLayoutTemplate --
- * 	Build a layout template tree from a statically defined 
+ * 	Build a layout template tree from a statically defined
  * 	Ttk_LayoutSpec array.
  */
 Ttk_LayoutTemplate Ttk_BuildLayoutTemplate(Ttk_LayoutSpec spec)
@@ -673,6 +706,62 @@ Ttk_LayoutTemplate Ttk_BuildLayoutTemplate(Ttk_LayoutSpec spec)
     return first;
 }
 
+Tcl_Obj *Ttk_UnparseLayoutTemplate(Ttk_TemplateNode *node) 
+{
+    Tcl_Obj *result = Tcl_NewListObj(0,0);
+
+#   define APPENDOBJ(obj) Tcl_ListObjAppendElement(NULL, result, obj)
+#   define APPENDSTR(str) APPENDOBJ(Tcl_NewStringObj(str,-1))
+
+    while (node) {
+	unsigned flags = node->flags;
+
+	APPENDSTR(node->name);
+
+	/* Back-compute -side.  <<NOTE-PACKSIDE>>
+	 * @@@ NOTES: Ick.
+	 * @@@ CHECK: ? '-expand true' equivalent to omitted -side?
+	 */
+	if (flags & TTK_EXPAND) {
+	    APPENDSTR("-expand");
+	    APPENDSTR("1"); 
+	} else {
+	    if (flags & _TTK_MASK_PACK) {
+		int side = 0;
+		unsigned sideFlags = flags & _TTK_MASK_PACK;
+
+		while ((sideFlags & TTK_PACK_LEFT) == 0) {
+		    ++side; 
+		    sideFlags >>= 1;
+		}
+		APPENDSTR("-side");
+		APPENDSTR(packSideStrings[side]);
+	    }
+	}
+
+	/* Default -sticky is "nsew", so always include this
+	 * even if no sticky bits are set.
+	 */
+	APPENDSTR("-sticky");
+	APPENDOBJ(Ttk_NewStickyObj(flags & _TTK_MASK_STICK));
+
+	/* @@@ Check again: are these necessary? */
+	if (flags & TTK_BORDER)	{ APPENDSTR("-border"); APPENDSTR("1"); }
+	if (flags & TTK_UNIT) 	{ APPENDSTR("-unit"); APPENDSTR("1"); }
+
+	if (node->child) {
+	    APPENDSTR("-children");
+	    APPENDOBJ(Ttk_UnparseLayoutTemplate(node->child));
+	}
+	node = node->next;
+    }
+
+#   undef APPENDOBJ
+#   undef APPENDSTR
+
+    return result;
+}
+
 /*------------------------------------------------------------------------
  * +++ Layouts.
  */
@@ -720,7 +809,7 @@ Ttk_Layout Ttk_CreateLayout(
     Tk_Window tkwin)
 {
     Ttk_Style style = Ttk_GetStyle(themePtr, styleName);
-    Ttk_LayoutTemplate layoutTemplate = 
+    Ttk_LayoutTemplate layoutTemplate =
 	Ttk_FindLayoutTemplate(themePtr,styleName);
     Ttk_Element bgelement = Ttk_GetElement(themePtr, "background");
     Ttk_LayoutNode *bgnode;
@@ -740,7 +829,7 @@ Ttk_Layout Ttk_CreateLayout(
 /* Ttk_CreateSubLayout --
  * 	Creates a new sublayout.
  */
-Ttk_Layout 
+Ttk_Layout
 Ttk_CreateSubLayout(
     Tcl_Interp *interp,
     Ttk_Theme themePtr,
@@ -750,7 +839,7 @@ Ttk_CreateSubLayout(
     Tk_Window tkwin)
 {
     Ttk_Style style = Ttk_GetStyle(themePtr, styleName);
-    Ttk_LayoutTemplate layoutTemplate = 
+    Ttk_LayoutTemplate layoutTemplate =
 	Ttk_FindLayoutTemplate(themePtr,styleName);
     Ttk_LayoutNode *root;
 
@@ -764,18 +853,12 @@ Ttk_CreateSubLayout(
     return TTKNewLayout(style, recordPtr, optionTable, tkwin, root);
 }
 
-/*
- * Ttk_RebindLayout --
+/* Ttk_RebindSublayout --
+ * 	Rebind sublayout to new data source.
  */
-void Ttk_RebindLayout(
-    Ttk_Layout layout,
-    void *recordPtr, 
-    Tk_OptionTable optionTable,
-    Tk_Window tkwin)
+void Ttk_RebindSublayout(Ttk_Layout layout, void *recordPtr)
 {
     layout->recordPtr = recordPtr;
-    layout->optionTable = optionTable;
-    layout->tkwin = tkwin;
 }
 
 /*
@@ -818,13 +901,13 @@ static void Ttk_NodeSize(
 }
 
 static void Ttk_NodeListSize(
-    Ttk_Layout layout, Ttk_LayoutNode *node, 
+    Ttk_Layout layout, Ttk_LayoutNode *node,
     Ttk_State state, int *widthPtr, int *heightPtr)
 {
     if (!node) {
 	*widthPtr = *heightPtr = 0;
     } else {
-	int width, height, restWidth, restHeight; 
+	int width, height, restWidth, restHeight;
 	Ttk_Padding unused;
 
 	Ttk_NodeSize(layout, node, state, &width, &height, &unused);
@@ -854,7 +937,7 @@ Ttk_Padding Ttk_LayoutNodeInternalPadding(
     int unused;
     Ttk_Padding padding;
     Ttk_ElementSize(node->element,
-	layout->style, layout->recordPtr, layout->optionTable, layout->tkwin, 
+	layout->style, layout->recordPtr, layout->optionTable, layout->tkwin,
 	0/*state*/, &unused, &unused, &padding);
     return padding;
 }
@@ -928,7 +1011,7 @@ static void Ttk_PlaceNodeList(
 	 * Place element within parcel:
 	 */
 	node->parcel = Ttk_StickBox(
-		parcel, elementWidth, elementHeight, node->flags); 
+		parcel, elementWidth, elementHeight, node->flags);
 
 	/*
 	 * Place child nodes:
@@ -948,7 +1031,7 @@ void Ttk_PlaceLayout(Ttk_Layout layout, Ttk_State state, Ttk_Box b)
 /*------------------------------------------------------------------------
  * +++ Layout drawing.
  */
- 
+
 /*
  * Ttk_DrawLayout --
  * 	Draw a layout tree.
@@ -956,7 +1039,7 @@ void Ttk_PlaceLayout(Ttk_Layout layout, Ttk_State state, Ttk_Box b)
 static void Ttk_DrawNodeList(
     Ttk_Layout layout, Ttk_State state, Ttk_LayoutNode *node, Drawable d)
 {
-    for (; node; node = node->next) 
+    for (; node; node = node->next)
     {
 	int border = node->flags & TTK_BORDER;
 	int substate = state;
@@ -968,7 +1051,7 @@ static void Ttk_DrawNodeList(
 	    Ttk_DrawNodeList(layout, substate, node->child, d);
 
 	Ttk_DrawElement(
-	    node->element, 
+	    node->element,
 	    layout->style,layout->recordPtr,layout->optionTable,layout->tkwin,
 	    d, node->parcel, state | node->state);
 
@@ -999,7 +1082,7 @@ Ttk_LayoutNodeIdentify(Ttk_LayoutNode *node, int x, int y)
 	if (Ttk_BoxContains(node->parcel, x, y)) {
 	    closest = node;
 	    if (node->child && !(node->flags & TTK_UNIT)) {
-		Ttk_LayoutNode *childNode = 
+		Ttk_LayoutNode *childNode =
 			Ttk_LayoutNodeIdentify(node->child, x,y);
 		if (childNode) {
 		    closest = childNode;
@@ -1040,9 +1123,9 @@ Ttk_LayoutNodeFind(Ttk_LayoutNode *node, const char *nodeName)
 	    return node;
 
 	if (node->child) {
-	    Ttk_LayoutNode *childNode = 
+	    Ttk_LayoutNode *childNode =
 		Ttk_LayoutNodeFind(node->child, nodeName);
-	    if (childNode) 
+	    if (childNode)
 		return childNode;
 	}
     }
@@ -1073,7 +1156,7 @@ void Ttk_PlaceLayoutNode(Ttk_Layout layout, Ttk_LayoutNode *node, Ttk_Box b)
 {
     node->parcel = b;
     if (node->child) {
-	Ttk_PlaceNodeList(layout, node->child, 0,  
+	Ttk_PlaceNodeList(layout, node->child, 0,
 	    Ttk_PadBox(b, Ttk_LayoutNodeInternalPadding(layout, node)));
     }
 }
@@ -1083,6 +1166,4 @@ void Ttk_ChangeElementState(Ttk_LayoutNode *node,unsigned set,unsigned clr)
     node->state = (node->state | set) & ~clr;
 }
 
-
-
 /*EOF*/
