@@ -1,4 +1,4 @@
-/* paned.c,v 1.3 2005/04/11 01:53:59 jenglish Exp
+/* paned.c,v 1.12 2005/10/19 20:30:49 jenglish Exp
  *
  * Copyright (c) 2005, Joe English.  Freely redistributable.
  *
@@ -15,10 +15,6 @@
 
 #define MIN_SASH_THICKNESS 5
 
-#if !defined(SHOVE_SASHES)
-#define SHOVE_SASHES 1	/* TEMP: for testing different sash drag behaviors */
-#endif
-
 /*------------------------------------------------------------------------
  * +++ Layout algorithm.
  *
@@ -67,7 +63,7 @@
 typedef struct {
     Tcl_Obj 	*orientObj;
     int 	orient;
-    Manager	*mgr;
+    Ttk_Manager	*mgr;
     Ttk_Layout	sashLayout;
     int 	sashThickness;
 } PanedPart;
@@ -116,75 +112,72 @@ static Tk_OptionSpec PaneOptionSpecs[] = {
 
 static int ShoveUp(Paned *pw, int i, int pos)
 {
-    Pane *pane = pw->paned.mgr->slaves[i]->slaveData;
+    Pane *pane = Ttk_SlaveData(pw->paned.mgr, i);
     int sashThickness = pw->paned.sashThickness;
 
     if (i == 0) {
 	if (pos < 0)
 	    pos = 0;
     } else {
-	Pane *prevPane =  pw->paned.mgr->slaves[i-1]->slaveData;
+	Pane *prevPane = Ttk_SlaveData(pw->paned.mgr, i-1);
 	if (pos < prevPane->sashPos + sashThickness)
 	    pos = ShoveUp(pw, i-1, pos - sashThickness) + sashThickness;
     }
     return pane->sashPos = pos;
 }
 
-#if SHOVE_SASHES
-
 /* ShoveDown --
  * 	Same as ShoveUp, but going in the opposite direction
  * 	and stopping at the sentinel sash.
  */
 static int ShoveDown(Paned *pw, int i, int pos)
 {
-    Pane *pane = pw->paned.mgr->slaves[i]->slaveData;
+    Pane *pane = Ttk_SlaveData(pw->paned.mgr,i);
     int sashThickness = pw->paned.sashThickness;
 
     if (i == pw->paned.mgr->nSlaves - 1) {
 	pos = pane->sashPos; /* Sentinel value == master window size */
     } else {
-	Pane *nextPane =  pw->paned.mgr->slaves[i+1]->slaveData;
+	Pane *nextPane = Ttk_SlaveData(pw->paned.mgr,i+1);
 	if (pos + sashThickness > nextPane->sashPos)
 	    pos = ShoveDown(pw, i+1, pos + sashThickness) - sashThickness;
     }
     return pane->sashPos = pos;
 }
 
-#endif /* SHOVE_SASHES */
-
 /* PanedSize --
+ * 	Compute the requested size of the paned widget.
  * 	Used as both the WidgetSpec sizeProc and the ManagerSpec sizeProc.
  */
-int PanedSize(void *recordPtr, int *widthPtr, int *heightPtr)
+static int PanedSize(void *recordPtr, int *widthPtr, int *heightPtr)
 {
     Paned *pw = recordPtr;
+    int nPanes = Ttk_NumberSlaves(pw->paned.mgr);
+    int nSashes = nPanes - 1;
     int sashThickness = pw->paned.sashThickness;
     int width = 0, height = 0;
     int index;
 
     if (pw->paned.orient == TTK_ORIENT_HORIZONTAL) {
-	for (index = 0; index < pw->paned.mgr->nSlaves; ++index) {
-	    Slave *slave = pw->paned.mgr->slaves[index];
-	    Pane *pane = slave->slaveData;
-
-	    if (height < Tk_ReqHeight(slave->slaveWindow))
-		height = Tk_ReqHeight(slave->slaveWindow);
+	for (index = 0; index < nPanes; ++index) {
+	    Pane *pane = Ttk_SlaveData(pw->paned.mgr, index);
+	    Tk_Window slaveWindow = Ttk_SlaveWindow(pw->paned.mgr, index);
 
+	    if (height < Tk_ReqHeight(slaveWindow))
+		height = Tk_ReqHeight(slaveWindow);
 	    width += pane->reqSize;
 	}
-	width += pw->paned.mgr->nSlaves * sashThickness;
+	width += nSashes * sashThickness;
     } else {
-	for (index = 0; index < pw->paned.mgr->nSlaves; ++index) {
-	    Slave *slave = pw->paned.mgr->slaves[index];
-	    Pane *pane = slave->slaveData;
-
-	    if (width < Tk_ReqWidth(slave->slaveWindow))
-		width = Tk_ReqWidth(slave->slaveWindow);
+	for (index = 0; index < nPanes; ++index) {
+	    Pane *pane = Ttk_SlaveData(pw->paned.mgr, index);
+	    Tk_Window slaveWindow = Ttk_SlaveWindow(pw->paned.mgr, index);
 
+	    if (width < Tk_ReqWidth(slaveWindow))
+		width = Tk_ReqWidth(slaveWindow);
 	    height += pane->reqSize;
 	}
-	height += pw->paned.mgr->nSlaves * sashThickness;
+	height += nSashes * sashThickness;
     }
 
     *widthPtr = width;
@@ -198,7 +191,7 @@ int PanedSize(void *recordPtr, int *widthPtr, int *heightPtr)
  * NOTE:
  * 	AdjustPanes followed by ComputePositions (called during relayout)
  * 	will leave the sashes in the same place, as long as available size
- * 	remains contants.
+ * 	remains contant.
  */
 static void AdjustPanes(Paned *pw)
 {
@@ -206,8 +199,8 @@ static void AdjustPanes(Paned *pw)
     int pos = 0;
     int index;
 
-    for (index = 0; index < pw->paned.mgr->nSlaves; ++index) {
-	Pane *pane = pw->paned.mgr->slaves[index]->slaveData;
+    for (index = 0; index < Ttk_NumberSlaves(pw->paned.mgr); ++index) {
+	Pane *pane = Ttk_SlaveData(pw->paned.mgr, index);
 	int size = pane->sashPos - pos;
 	pane->reqSize = size >= 0 ? size : 0;
 	pos = pane->sashPos + sashThickness;
@@ -231,7 +224,8 @@ static void AdjustPanes(Paned *pw)
  */
 static void ComputePositions(Paned *pw)
 {
-    Manager *mgr = pw->paned.mgr;
+    Ttk_Manager *mgr = pw->paned.mgr;
+    int nPanes = Ttk_NumberSlaves(mgr);
     int sashThickness = pw->paned.sashThickness;
     int available
 	= pw->paned.orient == TTK_ORIENT_HORIZONTAL
@@ -239,20 +233,20 @@ static void ComputePositions(Paned *pw)
     int reqSize = 0, totalWeight = 0;
     int difference, delta, remainder, pos, i;
 
-    if (mgr->nSlaves == 0)
+    if (nPanes == 0)
 	return;
 
     /* Compute total required size and total available weight:
      */
-    for (i = 0; i < mgr->nSlaves; ++i) {
-	Pane *pane = mgr->slaves[i]->slaveData;
+    for (i = 0; i < nPanes; ++i) {
+	Pane *pane = Ttk_SlaveData(mgr, i);
 	reqSize += pane->reqSize;
 	totalWeight += pane->weight * (pane->reqSize != 0);
     }
 
     /* Compute difference to be redistributed:
      */
-    difference = available - reqSize - sashThickness*(mgr->nSlaves-1);
+    difference = available - reqSize - sashThickness*(nPanes-1);
     if (totalWeight != 0) {
 	delta = difference / totalWeight;
 	remainder = difference % totalWeight;
@@ -268,8 +262,8 @@ static void ComputePositions(Paned *pw)
     /* Place sashes:
      */
     pos = 0;
-    for (i = 0; i < mgr->nSlaves; ++i) {
-	Pane *pane = mgr->slaves[i]->slaveData;
+    for (i = 0; i < nPanes; ++i) {
+	Pane *pane = Ttk_SlaveData(mgr, i);
 	int weight = pane->weight * (pane->reqSize != 0);
 	int size = pane->reqSize + delta * weight;
 
@@ -289,7 +283,7 @@ static void ComputePositions(Paned *pw)
      * Set sentinel sash position to end of widget,
      * shove preceding sashes up.
      */
-    ShoveUp(pw, mgr->nSlaves - 1, available);
+    ShoveUp(pw, nPanes - 1, available);
 }
 
 /* PlacePanes --
@@ -303,21 +297,18 @@ static void PlacePanes(Paned *pw)
     int pos = 0;
     int index;
 
-    for (index = 0; index < pw->paned.mgr->nSlaves; ++index) {
-	Slave *slave = pw->paned.mgr->slaves[index];
-	Pane *pane = slave->slaveData;
+    for (index = 0; index < Ttk_NumberSlaves(pw->paned.mgr); ++index) {
+	Pane *pane = Ttk_SlaveData(pw->paned.mgr, index);
 	int size = pane->sashPos - pos;
 
 	if (size > 0) {
 	    if (horizontal) {
-		Tk_MoveResizeWindow(slave->slaveWindow, pos, 0, size, height);
+		Ttk_PlaceSlave(pw->paned.mgr, index, pos, 0, size, height);
 	    } else {
-		Tk_MoveResizeWindow(slave->slaveWindow, 0, pos, width, size);
+		Ttk_PlaceSlave(pw->paned.mgr, index, 0, pos, width, size);
 	    }
-	    Tk_MapWindow(slave->slaveWindow);
 	} else {
-	    size = 0;
-	    Tk_UnmapWindow(slave->slaveWindow);
+	    Ttk_UnmapSlave(pw->paned.mgr, index);
 	}
 
 	pos = pane->sashPos + sashThickness;
@@ -335,21 +326,22 @@ static void PanedPlaceSlaves(void *managerData)
     PlacePanes(pw);
 }
 
-static void PaneAdded(Manager *mgr, Slave *slave)
+static void PaneAdded(Ttk_Manager *mgr, int index)
 {
-    Pane *pane = slave->slaveData;
+    Pane *pane = Ttk_SlaveData(mgr, index);
+    Tk_Window slaveWindow = Ttk_SlaveWindow(mgr, index);
     Paned *pw = mgr->managerData;
 
     /* See also: PanedGeometryRequestProc */
     pane->reqSize
 	= pw->paned.orient == TTK_ORIENT_HORIZONTAL
-	? Tk_ReqWidth(slave->slaveWindow) : Tk_ReqHeight(slave->slaveWindow);
+	? Tk_ReqWidth(slaveWindow) : Tk_ReqHeight(slaveWindow);
 }
 
-static void PaneRemoved(Manager *mgr, Slave *slave) { /* no-op */ }
+static void PaneRemoved(Ttk_Manager *mgr, int i) { /*no-op*/ }
 
 static int PaneConfigured(
-    Tcl_Interp *interp, Manager *mgr, Slave *slave, unsigned mask)
+    Tcl_Interp *interp, Ttk_Manager *mgr, Ttk_Slave *slave, unsigned mask)
 {
     Pane *pane = slave->slaveData;
     if (pane->weight < 0) {
@@ -361,32 +353,35 @@ static int PaneConfigured(
 }
 
 /* PanedGeometryRequestProc --
- * 	Update pane request size.
+ * 	Update pane request size, but only if slave is currently unmapped.
+ * 	Geometry requests from mapped slaves are not directly honored,
+ * 	in order to avoid unexpected pane resizes (esp. while the
+ * 	user is dragging a sash [#1325286]).
  */
+
 static void PanedGeometryRequestProc(
     ClientData clientData, Tk_Window slaveWindow)
 {
-    Slave *slave = clientData;
+    Ttk_Slave *slave = clientData;
     Pane *pane = slave->slaveData;
     Paned *pw = slave->manager->managerData;
 
-    /* See also: PaneAdded */
-    pane->reqSize
-	= pw->paned.orient == TTK_ORIENT_HORIZONTAL
-	? Tk_ReqWidth(slave->slaveWindow) : Tk_ReqHeight(slave->slaveWindow);
+    if (!Tk_IsMapped(slaveWindow)) {
+	pane->reqSize
+	    = pw->paned.orient == TTK_ORIENT_HORIZONTAL
+	    ? Tk_ReqWidth(slaveWindow) : Tk_ReqHeight(slaveWindow);
+    }
 
     /* Continue with default GeometryRequestProc:
      */
-    ManagerGeometryRequestProc(clientData, slaveWindow);
+    Ttk_GeometryRequestProc(clientData, slaveWindow);
 }
 
-static ManagerSpec PanedManagerSpec = {
-    { "paned", PanedGeometryRequestProc, ManagerLostSlaveProc },
+static Ttk_ManagerSpec PanedManagerSpec = {
+    { "paned", PanedGeometryRequestProc, Ttk_LostSlaveProc },
     PaneOptionSpecs, sizeof(Pane),
-
     PanedSize,
     PanedPlaceSlaves,
-
     PaneAdded,
     PaneRemoved,
     PaneConfigured,
@@ -425,7 +420,7 @@ static int PanedInitialize(Tcl_Interp *interp, void *recordPtr)
 
     Tk_CreateEventHandler(pw->core.tkwin,
 	PanedEventMask, PanedEventProc, recordPtr);
-    pw->paned.mgr = CreateManager(&PanedManagerSpec, pw, pw->core.tkwin);
+    pw->paned.mgr = Ttk_CreateManager(&PanedManagerSpec, pw, pw->core.tkwin);
     pw->paned.sashLayout = 0;
     pw->paned.sashThickness = 1;
 
@@ -440,13 +435,13 @@ static void PanedCleanup(void *recordPtr)
 	Ttk_FreeLayout(pw->paned.sashLayout);
     Tk_DeleteEventHandler(pw->core.tkwin,
 	PanedEventMask, PanedEventProc, recordPtr);
-    DeleteManager(pw->paned.mgr);
+    Ttk_DeleteManager(pw->paned.mgr);
 }
 
 /*------------------------------------------------------------------------
  * +++ Layout management hooks.
  */
-Ttk_Layout PanedGetLayout(
+static Ttk_Layout PanedGetLayout(
     Tcl_Interp *interp, Ttk_Theme themePtr, void *recordPtr)
 {
     Paned *pw = recordPtr;
@@ -471,19 +466,20 @@ Ttk_Layout PanedGetLayout(
 	if (pw->paned.sashThickness < MIN_SASH_THICKNESS)
 	    pw->paned.sashThickness = MIN_SASH_THICKNESS;
 
-	SizeChanged(pw->paned.mgr);
+	Ttk_ManagerSizeChanged(pw->paned.mgr);
     }
 
     return WidgetGetLayout(interp, themePtr, recordPtr);
 }
 
-void PanedDisplay(void *recordPtr, Drawable d)
+static void PanedDisplay(void *recordPtr, Drawable d)
 {
     Paned *pw = recordPtr;
     int horizontal = pw->paned.orient == TTK_ORIENT_HORIZONTAL;
     Ttk_Layout sashLayout = pw->paned.sashLayout;
     int sashThickness = pw->paned.sashThickness;
     Ttk_State state = pw->core.state;
+    int nPanes = Ttk_NumberSlaves(pw->paned.mgr);
     int i;
 
     WidgetDisplay(recordPtr, d);
@@ -492,18 +488,18 @@ void PanedDisplay(void *recordPtr, Drawable d)
      */
     if (horizontal) {
 	int height = Tk_Height(pw->core.tkwin);
-	for (i = 0; i < pw->paned.mgr->nSlaves; ++i) {
-	    Pane *pane = pw->paned.mgr->slaves[i]->slaveData;
+	for (i = 0; i < nPanes; ++i) {
+	    Pane *pane = Ttk_SlaveData(pw->paned.mgr, i);
 	    Ttk_PlaceLayout(sashLayout, state,
-		Ttk_MakeBox(pane->sashPos,0, sashThickness, height));
+		Ttk_MakeBox(pane->sashPos, 0, sashThickness, height));
 	    Ttk_DrawLayout(sashLayout, state, d);
 	}
     } else {
 	int width = Tk_Width(pw->core.tkwin);
-	for (i = 0; i < pw->paned.mgr->nSlaves; ++i) {
-	    Pane *pane = pw->paned.mgr->slaves[i]->slaveData;
+	for (i = 0; i < nPanes; ++i) {
+	    Pane *pane = Ttk_SlaveData(pw->paned.mgr, i);
 	    Ttk_PlaceLayout(sashLayout, state,
-		Ttk_MakeBox(0,pane->sashPos, width,sashThickness));
+		Ttk_MakeBox(0, pane->sashPos, width, sashThickness));
 	    Ttk_DrawLayout(sashLayout, state, d);
 	}
     }
@@ -529,19 +525,12 @@ static int PanedAddCommand(
     slaveWindow = Tk_NameToWindow(
 	interp, Tcl_GetString(objv[2]), pw->core.tkwin);
 
-    if (!slaveWindow || !Manageable(interp, slaveWindow, pw->core.tkwin)) {
+    if (!slaveWindow) {
 	return TCL_ERROR;
     }
 
-    if (SlaveIndex(pw->paned.mgr, slaveWindow) >= 0) {
-	Tcl_AppendResult(interp,
-	    Tk_PathName(slaveWindow), " already added",
-	    NULL);
-	return TCL_ERROR;
-    }
-
-    return AddSlave(interp, pw->paned.mgr, slaveWindow, pw->paned.mgr->nSlaves,
-	    objc - 3, objv + 3);
+    return Ttk_AddSlave(interp, pw->paned.mgr, slaveWindow,
+	    Ttk_NumberSlaves(pw->paned.mgr), objc - 3, objv + 3);
 }
 
 /* $pw insert $index $slave ?options...?
@@ -561,28 +550,28 @@ static int PanedInsertCommand(
 
     slaveWindow = Tk_NameToWindow(
 	interp, Tcl_GetString(objv[3]), pw->core.tkwin);
-    if (!slaveWindow || !Manageable(interp, slaveWindow, pw->core.tkwin)) {
+    if (!slaveWindow) {
 	return TCL_ERROR;
     }
 
     if (!strcmp(Tcl_GetString(objv[2]), "end")) {
-	destIndex = pw->paned.mgr->nSlaves;
-    } else if (!GetSlaveFromObj(interp, pw->paned.mgr, objv[2], &destIndex)) {
+	destIndex = Ttk_NumberSlaves(pw->paned.mgr);
+    } else if (!Ttk_GetSlaveFromObj(interp,pw->paned.mgr,objv[2],&destIndex)) {
 	return TCL_ERROR;
     }
 
-    srcIndex = SlaveIndex(pw->paned.mgr, slaveWindow);
+    srcIndex = Ttk_SlaveIndex(pw->paned.mgr, slaveWindow);
     if (srcIndex < 0) { /* New slave: */
-	return AddSlave(interp, pw->paned.mgr, slaveWindow, destIndex,
-		objc - 4, objv + 4);
+	return Ttk_AddSlave(interp, pw->paned.mgr, slaveWindow,
+		destIndex, objc - 4, objv + 4);
     } /* else -- move existing slave: */
 
     if (destIndex >= pw->paned.mgr->nSlaves)
 	destIndex  = pw->paned.mgr->nSlaves - 1;
-    MoveSlave(pw->paned.mgr, srcIndex, destIndex);
+    Ttk_ReorderSlave(pw->paned.mgr, srcIndex, destIndex);
 
     return objc == 4 ? TCL_OK :
-	ConfigureSlave(interp, pw->paned.mgr,
+	Ttk_ConfigureSlave(interp, pw->paned.mgr,
 	     pw->paned.mgr->slaves[destIndex], objc-4,objv+4);
 }
 
@@ -599,10 +588,10 @@ static int PanedForgetCommand(
 	return TCL_ERROR;
     }
 
-    if (!GetSlaveFromObj(interp, pw->paned.mgr, objv[2], &paneIndex)) {
+    if (!Ttk_GetSlaveFromObj(interp, pw->paned.mgr, objv[2], &paneIndex)) {
 	return TCL_ERROR;
     }
-    ForgetSlave(pw->paned.mgr, paneIndex);
+    Ttk_ForgetSlave(pw->paned.mgr, paneIndex);
 
     return TCL_OK;
 }
@@ -630,7 +619,7 @@ static int PanedIdentifyCommand(
 
     pos = pw->paned.orient == TTK_ORIENT_HORIZONTAL ? x : y;
     for (index = 0; index < pw->paned.mgr->nSlaves - 1; ++index) {
-	Pane *pane = pw->paned.mgr->slaves[index]->slaveData;
+	Pane *pane = Ttk_SlaveData(pw->paned.mgr, index);
 	if (pane->sashPos <= pos && pos <= pane->sashPos + sashThickness) {
 	    Tcl_SetObjResult(interp, Tcl_NewIntObj(index));
 	    return TCL_OK;
@@ -648,14 +637,14 @@ static int PanedPaneCommand(
 {
     Paned *pw = recordPtr;
     int paneIndex;
-    Slave *slave;
+    Ttk_Slave *slave;
 
     if (objc < 3) {
 	Tcl_WrongNumArgs(interp, 2,objv, "pane ?-option value...?");
 	return TCL_ERROR;
     }
 
-    slave = GetSlaveFromObj(interp,pw->paned.mgr,objv[2],&paneIndex);
+    slave = Ttk_GetSlaveFromObj(interp,pw->paned.mgr,objv[2],&paneIndex);
     if (!slave) {
 	return TCL_ERROR;
     }
@@ -668,7 +657,8 @@ static int PanedPaneCommand(
 	    return GetOptionValue(interp, slave->slaveData,objv[3],
 			pw->paned.mgr->slaveOptionTable, slave->slaveWindow);
 	default:
-	    return ConfigureSlave(interp, pw->paned.mgr, slave, objc-3,objv+3);
+	    return Ttk_ConfigureSlave(
+		    interp, pw->paned.mgr, slave, objc-3,objv+3);
     }
 }
 
@@ -689,14 +679,14 @@ static int PanedSashposCommand(
     if (Tcl_GetIntFromObj(interp, objv[2], &sashIndex) != TCL_OK) {
 	return TCL_ERROR;
     }
-    if (sashIndex < 0 || sashIndex >= pw->paned.mgr->nSlaves - 1) {
+    if (sashIndex < 0 || sashIndex >= Ttk_NumberSlaves(pw->paned.mgr) - 1) {
 	Tcl_AppendResult(interp,
 	    "sash index ", Tcl_GetString(objv[2]), " out of range",
 	    NULL);
 	return TCL_ERROR;
     }
 
-    pane = pw->paned.mgr->slaves[sashIndex]->slaveData;
+    pane = Ttk_SlaveData(pw->paned.mgr, sashIndex);
 
     if (objc == 3) {
 	Tcl_SetObjResult(interp, Tcl_NewIntObj(pane->sashPos));
@@ -708,35 +698,14 @@ static int PanedSashposCommand(
 	return TCL_ERROR;
     }
 
-#if SHOVE_SASHES
     if (position < pane->sashPos) {
 	ShoveUp(pw, sashIndex, position);
     } else {
 	ShoveDown(pw, sashIndex, position);
     }
 
-#else /* !SHOVE_SASHES */
-
-    if (position < pane->sashPos && sashIndex > 0) {
-	Pane *prevPane = pw->paned.mgr->slaves[sashIndex - 1]->slaveData;
-	int sashThickness = pw->paned.sashThickness;
-	if (position < prevPane->sashPos + sashThickness)
-	    position = prevPane->sashPos + sashThickness;
-    } else if (position < pane->sashPos) {
-	if (position < 0)
-	    position = 0;
-    } else {
-	Pane *nextPane = pw->paned.mgr->slaves[sashIndex + 1]->slaveData;
-	int sashThickness = pw->paned.sashThickness;
-	if (position > nextPane->sashPos - sashThickness)
-	    position = nextPane->sashPos - sashThickness;
-    }
-    pane->sashPos = position;
-
-#endif /* SHOVE_SASHES */
-
     AdjustPanes(pw);
-    LayoutChanged(pw->paned.mgr);
+    Ttk_ManagerLayoutChanged(pw->paned.mgr);
 
     Tcl_SetObjResult(interp, Tcl_NewIntObj(pane->sashPos));
     return TCL_OK;
@@ -774,9 +743,7 @@ static WidgetSpec PanedWidgetSpec =
     PanedGetLayout,		/* getLayoutProc */
     PanedSize, 			/* sizeProc */
     WidgetDoLayout,		/* layoutProc */
-    PanedDisplay,		/* displayProc */
-
-    WIDGET_SPEC_END		/* sentinel */
+    PanedDisplay		/* displayProc */
 };
 
 /*------------------------------------------------------------------------
