@@ -1,4 +1,4 @@
-/* scroll.c,v 1.2 2005/03/08 20:54:13 jenglish Exp
+/* scroll.c,v 1.3 2005/10/08 14:02:47 jenglish Exp
  *
  * Copyright 2004, Joe English
  *
@@ -211,7 +211,7 @@ int ScrollviewCommand(
 
     if (newFirst != s->first) {
 	s->first = newFirst;
-	WidgetChanged(h->corePtr, REDISPLAY_REQUIRED);
+	TtkRedisplayWidget(h->corePtr);
     }
 
     return TCL_OK;
