@@ -1,4 +1,4 @@
-/* tkElements.c,v 1.72 2005/05/11 20:00:51 jenglish Exp
+/* tkElements.c,v 1.75 2005/10/07 16:21:40 jenglish Exp
  *
  * Copyright (c) 2003, Joe English
  *
@@ -12,7 +12,6 @@
 #include "tkTheme.h"
 
 #define DEFAULT_BORDERWIDTH "2"
-#define DEFAULT_SLIDER_LENGTH "30"
 #define DEFAULT_ARROW_SIZE "15"
 #define MIN_THUMB_SIZE 10
 
@@ -252,8 +251,9 @@ PaddingElementGeometry(
 }
 
 static void
-PaddingElementDraw(void *clientData, void *elementRecord,
-    Tk_Window tkwin, Drawable d, Ttk_Box b, unsigned int state)
+PaddingElementDraw(
+    void *clientData, void *elementRecord, Tk_Window tkwin,
+    Drawable d, Ttk_Box b, unsigned int state)
 {
     /* No-op */
 }
@@ -492,8 +492,8 @@ static Ttk_ElementOptionSpec IndicatorElementOptions[] =
  */
 static void
 SquareIndicatorElementGeometry(
-    void *clientData, void *elementRecord,
-    Tk_Window tkwin, int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
+    void *clientData, void *elementRecord, Tk_Window tkwin,
+    int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
 {
     IndicatorElement *indicator = elementRecord;
     int diameter = 0;
@@ -503,8 +503,9 @@ SquareIndicatorElementGeometry(
 }
 
 static void
-SquareIndicatorElementDraw(void *clientData, void *elementRecord,
-    Tk_Window tkwin, Drawable d, Ttk_Box b, unsigned int state)
+SquareIndicatorElementDraw(
+    void *clientData, void *elementRecord, Tk_Window tkwin,
+    Drawable d, Ttk_Box b, unsigned int state)
 {
     IndicatorElement *indicator = elementRecord;
     Tk_3DBorder border = 0, interior = 0;
@@ -533,8 +534,8 @@ SquareIndicatorElementDraw(void *clientData, void *elementRecord,
  */
 static void
 DiamondIndicatorElementGeometry(
-    void *clientData, void *elementRecord,
-    Tk_Window tkwin, int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
+    void *clientData, void *elementRecord, Tk_Window tkwin,
+    int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
 {
     IndicatorElement *indicator = elementRecord;
     int diameter = 0;
@@ -544,8 +545,9 @@ DiamondIndicatorElementGeometry(
 }
 
 static void
-DiamondIndicatorElementDraw(void *clientData, void *elementRecord,
-    Tk_Window tkwin, Drawable d, Ttk_Box b, unsigned int state)
+DiamondIndicatorElementDraw(
+    void *clientData, void *elementRecord, Tk_Window tkwin,
+    Drawable d, Ttk_Box b, unsigned int state)
 {
     IndicatorElement *indicator = elementRecord;
     Tk_3DBorder border = 0, interior = 0;
@@ -636,8 +638,8 @@ static Ttk_ElementOptionSpec MenuIndicatorElementOptions[] =
 
 static void
 MenuIndicatorElementGeometry(
-    void *clientData, void *elementRecord,
-    Tk_Window tkwin, int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
+    void *clientData, void *elementRecord, Tk_Window tkwin,
+    int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
 {
     MenuIndicatorElement *mi = elementRecord;
     Tk_GetPixelsFromObj(NULL, tkwin, mi->widthObj, widthPtr);
@@ -646,8 +648,9 @@ MenuIndicatorElementGeometry(
 }
 
 static void
-MenuIndicatorElementDraw(void *clientData, void *elementRecord,
-    Tk_Window tkwin, Drawable d, Ttk_Box b, unsigned int state)
+MenuIndicatorElementDraw(
+    void *clientData, void *elementRecord, Tk_Window tkwin,
+    Drawable d, Ttk_Box b, unsigned int state)
 {
     MenuIndicatorElement *mi = elementRecord;
     Tk_3DBorder border = Tk_Get3DBorderFromObj(tkwin, mi->backgroundObj);
@@ -736,7 +739,7 @@ ArrowElementDraw(
     Tk_Fill3DRectangle(
 	tkwin, d, border, b.x, b.y, b.width, b.height, borderWidth, relief);
 
-    DrawArrow(Tk_Display(tkwin), d, Tk_GCForColor(arrowColor, d),
+    FillArrow(Tk_Display(tkwin), d, Tk_GCForColor(arrowColor, d),
 	Ttk_PadBox(b, ArrowPadding), direction);
 }
 
@@ -776,8 +779,8 @@ static Ttk_ElementOptionSpec TroughElementOptions[] =
 
 static void
 TroughElementGeometry(
-    void *clientData, void *elementRecord,
-    Tk_Window tkwin, int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
+    void *clientData, void *elementRecord, Tk_Window tkwin,
+    int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
 {
     TroughElement *troughPtr = elementRecord;
     int borderWidth = 2;
@@ -787,8 +790,9 @@ TroughElementGeometry(
 }
 
 static void
-TroughElementDraw(void *clientData, void *elementRecord,
-    Tk_Window tkwin, Drawable d, Ttk_Box b, unsigned int state)
+TroughElementDraw(
+    void *clientData, void *elementRecord, Tk_Window tkwin,
+    Drawable d, Ttk_Box b, unsigned int state)
 {
     TroughElement *troughPtr = elementRecord;
     Tk_3DBorder border = NULL;
@@ -821,7 +825,7 @@ static Ttk_ElementSpec TroughElementSpec =
 typedef struct
 {
     Tcl_Obj *orientObj;
-    Tcl_Obj *sizeObj;
+    Tcl_Obj *thicknessObj;
     Tcl_Obj *reliefObj;
     Tcl_Obj *borderObj;
     Tcl_Obj *borderWidthObj;
@@ -832,7 +836,7 @@ static Ttk_ElementOptionSpec ThumbElementOptions[] =
     { "-orient", TK_OPTION_ANY,
 	Tk_Offset(ThumbElement, orientObj), "horizontal" },
     { "-width", TK_OPTION_PIXELS,
-	Tk_Offset(ThumbElement,sizeObj), DEFAULT_ARROW_SIZE },
+	Tk_Offset(ThumbElement,thicknessObj), DEFAULT_ARROW_SIZE },
     { "-relief", TK_OPTION_RELIEF,
 	Tk_Offset(ThumbElement,reliefObj), "raised" },
     { "-background", TK_OPTION_BORDER,
@@ -848,17 +852,17 @@ ThumbElementGeometry(
     int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
 {
     ThumbElement *thumb = elementRecord;
-    int orient, size;
+    int orient, thickness;
 
-    Tk_GetPixelsFromObj(NULL, tkwin, thumb->sizeObj, &size);
+    Tk_GetPixelsFromObj(NULL, tkwin, thumb->thicknessObj, &thickness);
     Ttk_GetOrientFromObj(NULL, thumb->orientObj, &orient);
 
     if (orient == TTK_ORIENT_VERTICAL) {
-	*widthPtr = size;
+	*widthPtr = thickness;
 	*heightPtr = MIN_THUMB_SIZE;
     } else {
 	*widthPtr = MIN_THUMB_SIZE;
-	*heightPtr = size;
+	*heightPtr = thickness;
     }
 }
 
@@ -896,8 +900,8 @@ static Ttk_ElementSpec ThumbElementSpec =
 typedef struct
 {
     Tcl_Obj *orientObj;	     /* orientation of overall slider */
-    Tcl_Obj *lengthObj;      /* the length of the slider */
-    Tcl_Obj *widthObj;       /* the width of the slider (height if horizontal) */
+    Tcl_Obj *lengthObj;      /* slider length */
+    Tcl_Obj *thicknessObj;   /* slider thickness */
     Tcl_Obj *reliefObj;      /* the relief for this object */
     Tcl_Obj *borderObj;      /* the background color */
     Tcl_Obj *borderWidthObj; /* the size of the border */
@@ -906,9 +910,9 @@ typedef struct
 static Ttk_ElementOptionSpec SliderElementOptions[] =
 {
     { "-sliderlength", TK_OPTION_PIXELS, Tk_Offset(SliderElement,lengthObj),
-	DEFAULT_SLIDER_LENGTH },
-    { "-width", TK_OPTION_PIXELS, Tk_Offset(SliderElement,widthObj),
-	"3" },
+	"30" },
+    { "-sliderthickness",TK_OPTION_PIXELS,Tk_Offset(SliderElement,thicknessObj),
+	"15" },
     { "-sliderrelief", TK_OPTION_RELIEF, Tk_Offset(SliderElement,reliefObj),
 	"raised" },
     { "-borderwidth", TK_OPTION_PIXELS, Tk_Offset(SliderElement,borderWidthObj),
@@ -922,33 +926,33 @@ static Ttk_ElementOptionSpec SliderElementOptions[] =
 
 static void
 SliderElementGeometry(
-    void *clientData, void *elementRecord,
-    Tk_Window tkwin, int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
+    void *clientData, void *elementRecord, Tk_Window tkwin,
+    int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
 {
     SliderElement *slider = elementRecord;
-    int borderWidth, sliderwidth, sliderlength, orient;
+    int orient, length, thickness;
 
     Ttk_GetOrientFromObj(NULL, slider->orientObj, &orient);
-    Tk_GetPixelsFromObj(NULL, tkwin, slider->borderWidthObj, &borderWidth);
-    Tk_GetPixelsFromObj(NULL, tkwin, slider->lengthObj, &sliderlength);
-    Tk_GetPixelsFromObj(NULL, tkwin, slider->widthObj, &sliderwidth);
+    Tk_GetPixelsFromObj(NULL, tkwin, slider->lengthObj, &length);
+    Tk_GetPixelsFromObj(NULL, tkwin, slider->thicknessObj, &thickness);
 
     switch (orient) {
 	case TTK_ORIENT_VERTICAL:
-	    *widthPtr = sliderwidth;
-	    *heightPtr = sliderlength;
+	    *widthPtr = thickness;
+	    *heightPtr = length;
 	    break;
 
 	case TTK_ORIENT_HORIZONTAL:
-	    *heightPtr = sliderwidth;
-	    *widthPtr = sliderlength;
+	    *widthPtr = length;
+	    *heightPtr = thickness;
 	    break;
     }
 }
 
 static void
-SliderElementDraw(void *clientData, void *elementRecord,
-    Tk_Window tkwin, Drawable d, Ttk_Box b, unsigned int state)
+SliderElementDraw(
+    void *clientData, void *elementRecord, Tk_Window tkwin,
+    Drawable d, Ttk_Box b, unsigned int state)
 {
     SliderElement *slider = elementRecord;
     Tk_3DBorder border = NULL;
@@ -997,40 +1001,6 @@ static Ttk_ElementSpec SliderElementSpec =
     SliderElementDraw
 };
 
-/*
- *----------------------------------------------------------------------
- * +++ Progress bar element (old, for ttk::progress widget)
- *
- * A progress bar.
- */
-
-static void
-ProgressElementDraw(void *clientData, void *elementRecord,
-    Tk_Window tkwin, Drawable d, Ttk_Box b, unsigned int state)
-{
-    SliderElement *slider = elementRecord;
-    Tk_3DBorder border = NULL;
-    int relief, borderWidth, orient;
-
-    border = Tk_Get3DBorderFromObj(tkwin, slider->borderObj);
-    Ttk_GetOrientFromObj(NULL, slider->orientObj, &orient);
-    Tk_GetPixelsFromObj(NULL, tkwin, slider->borderWidthObj, &borderWidth);
-    Tk_GetReliefFromObj(NULL, slider->reliefObj, &relief);
-
-    Tk_Fill3DRectangle(tkwin, d, border,
-	b.x, b.y, b.width, b.height,
-	borderWidth, relief);
-}
-
-static Ttk_ElementSpec ProgressElementSpec =
-{
-    TK_STYLE_VERSION_2,
-    sizeof(SliderElement),
-    SliderElementOptions,
-    SliderElementGeometry,
-    ProgressElementDraw
-};
-
 /*------------------------------------------------------------------------
  * +++ Progress bar element:
  *	Draws the moving part of the progress bar.
@@ -1331,16 +1301,6 @@ TTK_BEGIN_LAYOUT(HorizontalScaleLayout)
 	TTK_NODE("Horizontal.Scale.slider", TTK_PACK_LEFT) )
 TTK_END_LAYOUT
 
-TTK_BEGIN_LAYOUT(VerticalProgressBarLayout)
-    TTK_GROUP("Vertical.Progress.trough", TTK_FILL_BOTH,
-	TTK_NODE("Vertical.Progress.bar", TTK_PACK_TOP))
-TTK_END_LAYOUT
-
-TTK_BEGIN_LAYOUT(HorizontalProgressBarLayout)
-    TTK_GROUP("Horizontal.Progress.trough", TTK_FILL_BOTH,
-	TTK_NODE("Horizontal.Progress.bar", TTK_PACK_LEFT))
-TTK_END_LAYOUT
-
 TTK_BEGIN_LAYOUT(SeparatorLayout)
     TTK_NODE("Separator.separator", TTK_FILL_BOTH)
 TTK_END_LAYOUT
@@ -1400,8 +1360,6 @@ void RegisterElements(Tcl_Interp *interp)
     Ttk_RegisterElement(interp, theme, "thumb", &ThumbElementSpec, NULL);
     Ttk_RegisterElement(interp, theme, "slider", &SliderElementSpec, NULL);
     Ttk_RegisterElement(interp, theme, "pbar", &PbarElementSpec, NULL);
-    Ttk_RegisterElement(interp, theme, "Progress.bar",
-	    &ProgressElementSpec, NULL);
 
     Ttk_RegisterElement(interp, theme, "separator",
 	    &SeparatorElementSpec, NULL);
@@ -1428,14 +1386,8 @@ void RegisterElements(Tcl_Interp *interp)
 	    "Vertical.TScrollbar", VerticalScrollbarLayout);
     Ttk_RegisterLayout(theme,
 	    "Horizontal.TScrollbar", HorizontalScrollbarLayout);
-    Ttk_RegisterLayout(theme,
-	    "Vertical.TScale", VerticalScaleLayout);
-    Ttk_RegisterLayout(theme,
-	    "Horizontal.TScale", HorizontalScaleLayout);
-    Ttk_RegisterLayout(theme,
-	    "Vertical.TProgress", VerticalProgressBarLayout);
-    Ttk_RegisterLayout(theme,
-	    "Horizontal.TProgress", HorizontalProgressBarLayout);
+    Ttk_RegisterLayout(theme, "Vertical.TScale", VerticalScaleLayout);
+    Ttk_RegisterLayout(theme, "Horizontal.TScale", HorizontalScaleLayout);
     Ttk_RegisterLayout(theme, "TSeparator", SeparatorLayout);
 }
 
