@@ -9,7 +9,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * tkTheme.c,v 1.82 2005/05/05 15:58:15 jenglish Exp
+ * tkTheme.c,v 1.86 2005/09/14 17:24:28 jenglish Exp
  */
 
 #include <stdlib.h>
@@ -24,8 +24,8 @@ EXTERN CONST Tk_OptionSpec *TkGetOptionSpec (CONST char *name,
 #include <tkInt.h>
 #endif
 
-/*
- * Styles --
+/*------------------------------------------------------------------------
+ * +++ Styles.
  */
 
 typedef struct Ttk_Style_
@@ -105,10 +105,11 @@ static Tcl_Obj *LookupDefault(Ttk_Style stylePtr, const char *optionName)
     return 0;
 }
 
-/*
- * array of Tk_OptionSpecs mapping widget options to element options
+/*------------------------------------------------------------------------
+ * +++ Elements.
  */
 typedef const Tk_OptionSpec **OptionMap;
+    /* array of Tk_OptionSpecs mapping widget options to element options */
 
 typedef struct Ttk_ElementImpl_		/* Element implementation */
 {
@@ -289,14 +290,14 @@ static void FreeElementImpl(ElementImpl *elementImpl)
     ckfree((ClientData)elementImpl);
 }
 
-/*
- * Default ThemeEnabledProc -- always return true
- */
-static int ThemeEnabled(Ttk_Theme theme, void *clientData) { return 1; }
 
-/*
- * Theme data structure:
+/*------------------------------------------------------------------------
+ * +++ Themes.
  */
+
+static int ThemeEnabled(Ttk_Theme theme, void *clientData) { return 1; }
+    /* Default ThemeEnabledProc -- always return true */
+
 typedef struct Ttk_Theme_
 {
     Ttk_Theme parentPtr;             	/* Parent theme. */
@@ -382,7 +383,7 @@ static void FreeTheme(Tcl_Interp *interp, Theme *themePtr)
 }
 
 /*
- * Element constructors:
+ * Element constructors.
  */
 typedef struct {
     Ttk_ElementFactory	factory;
@@ -398,8 +399,8 @@ typedef struct CleanupStruct {
     struct CleanupStruct *next;
 } Cleanup;
 
-/*
- * Master data structure.
+/*------------------------------------------------------------------------
+ * +++ Master style package data structure.
  */
 typedef struct
 {
@@ -415,18 +416,10 @@ typedef struct
 
 static void ThemeChangedProc(ClientData);	/* Forward */
 
-/*
- *---------------------------------------------------------------------------
- *
- * Ttk_StylePkgFree --
- *
- *	This procedure is called when the interp is deleted.  It
- *	deletes all the structures that were used by the style package
- *	for this application.
+/* Ttk_StylePkgFree --
+ *	Cleanup procedure for StylePackageData.
  */
-
-static void
-Ttk_StylePkgFree(ClientData clientData, Tcl_Interp *interp)
+static void Ttk_StylePkgFree(ClientData clientData, Tcl_Interp *interp)
 {
     StylePackageData *pkgPtr = (StylePackageData *)clientData;
     Tcl_HashSearch search;
@@ -490,11 +483,8 @@ Ttk_StylePkgFree(ClientData clientData, Tcl_Interp *interp)
  *	data provided here.
  *
  */
-void
-Ttk_RegisterCleanup(
-    Tcl_Interp *interp,
-    ClientData clientData,
-    Ttk_CleanupProc *cleanupProc)
+void Ttk_RegisterCleanup(
+    Tcl_Interp *interp, ClientData clientData, Ttk_CleanupProc *cleanupProc)
 {
     StylePackageData *pkgPtr = (StylePackageData *)
 	Tcl_GetAssocData(interp, "StylePackage", NULL);
@@ -542,15 +532,11 @@ static void ThemeChanged(StylePackageData *pkgPtr)
 }
 
 /*
- *---------------------------------------------------------------------------
- *
  * Ttk_CreateTheme --
  *	Create a new theme and register it in the global theme table.
  *
  * Returns:
  * 	Pointer to new Theme structure; NULL if named theme already exists.
- *
- * Side effects:
  * 	Leaves an error message in interp's result on error.
  */
 
@@ -586,12 +572,8 @@ Ttk_CreateTheme(
 
 
 /*
- *---------------------------------------------------------------------------
- *
  * Ttk_SetThemeEnabledProc --
- *
- *	Sets a procedure that is used to check that this theme is available
- *	for use.
+ *	Sets a procedure that is used to check that this theme is available.
  */
 
 void Ttk_SetThemeEnabledProc(
@@ -602,8 +584,6 @@ void Ttk_SetThemeEnabledProc(
 }
 
 /*
- *---------------------------------------------------------------------------
- *
  * Ttk_GetTheme --
  *	Retrieve a registered theme by name
  *
@@ -631,8 +611,6 @@ LookupTheme(
 }
 
 /*
- *---------------------------------------------------------------------------
- *
  * Ttk_GetTheme --
  *	Look up a Ttk_Theme by name.
  *
@@ -642,27 +620,22 @@ LookupTheme(
  *	message will be left in interp's result.
  */
 
-Ttk_Theme
-Ttk_GetTheme(
-    Tcl_Interp *interp,	/* Tcl interpreter to query */
-    const char *name)	/* Name of the theme to retrieve; NULL for default */
+Ttk_Theme Ttk_GetTheme(Tcl_Interp *interp, const char *themeName)
 {
     StylePackageData *pkgPtr = (StylePackageData *)
 	Tcl_GetAssocData(interp, "StylePackage", NULL);
 
-    return LookupTheme(interp, pkgPtr, name);
+    return LookupTheme(interp, pkgPtr, themeName);
 }
 
-Ttk_Theme
-Ttk_GetCurrentTheme(Tcl_Interp *interp)
+Ttk_Theme Ttk_GetCurrentTheme(Tcl_Interp *interp)
 {
     StylePackageData *pkgPtr = (StylePackageData *)
 	Tcl_GetAssocData(interp, "StylePackage", NULL);
     return pkgPtr->currentTheme;
 }
 
-Ttk_Theme
-Ttk_GetDefaultTheme(Tcl_Interp *interp)
+Ttk_Theme Ttk_GetDefaultTheme(Tcl_Interp *interp)
 {
     StylePackageData *pkgPtr = (StylePackageData *)
 	Tcl_GetAssocData(interp, "StylePackage", NULL);
@@ -671,10 +644,8 @@ Ttk_GetDefaultTheme(Tcl_Interp *interp)
 
 /*
  * Ttk_UseTheme --
- * 	Set the current theme, notify all widgets that
- * 	the theme has changed.
+ * 	Set the current theme, notify all widgets that the theme has changed.
  */
-
 int Ttk_UseTheme(Tcl_Interp *interp, Ttk_Theme  theme)
 {
     StylePackageData *pkgPtr = (StylePackageData *)
@@ -712,8 +683,7 @@ Ttk_GetResourceCache(Tcl_Interp *interp)
 /*
  * Register a new layout specification
  */
-
-static int Ttk_RegisterLayoutTemplate(
+static void Ttk_RegisterLayoutTemplate(
     Ttk_Theme themePtr,			/* Target theme */
     const char *layoutName,		/* Name of new layout */
     Ttk_LayoutTemplate layoutTemplate)	/* Template */
@@ -723,12 +693,10 @@ static int Ttk_RegisterLayoutTemplate(
 
     entryPtr=Tcl_CreateHashEntry(&themePtr->layoutTable,layoutName,&newEntry);
     if (!newEntry) {
-        return 0;
+	Ttk_LayoutTemplate oldTemplate = Tcl_GetHashValue(entryPtr);
+	Ttk_FreeLayoutTemplate(oldTemplate);
     }
-
     Tcl_SetHashValue(entryPtr, (ClientData)layoutTemplate);
-
-    return 1;
 }
 
 void Ttk_RegisterLayout(
@@ -797,7 +765,7 @@ Ttk_FindLayoutTemplate(Ttk_Theme themePtr, const char *layoutName)
     return NULL;
 }
 
-/*---------------------------------------------------------------------------
+/*
  * Ttk_GetElement --
  *	Look up an element implementation by name in a given theme.
  *	If not found, try generic element names in this theme, then
@@ -850,8 +818,6 @@ const char *Ttk_ElementName(ElementImpl *elementImpl)
 }
 
 /*
- *---------------------------------------------------------------------------
- *
  * Ttk_RegisterElementFactory --
  *	Register a new element factory.
  */
@@ -879,7 +845,8 @@ int Ttk_RegisterElementFactory(
 }
 
 
-/* (style element create $name) from $theme ?$element?
+/* Ttk_CloneElement -- element factory procedure.
+ * 	(style element create $name) "from" $theme ?$element?
  */
 static int Ttk_CloneElement(
     Tcl_Interp *interp, void *clientData,
@@ -971,6 +938,10 @@ int Ttk_RegisterElementSpec(Ttk_Theme theme,
 	   ? TCL_OK : TCL_ERROR;
 }
 
+/*------------------------------------------------------------------------
+ * +++ Element record initialization.
+ */
+
 /*
  * AllocateResource --
  * 	Extra initialization for element options like TK_OPTION_COLOR, etc.
@@ -1070,6 +1041,10 @@ int InitializeElementRecord(
     return 1;
 }
 
+/*------------------------------------------------------------------------
+ * +++ Public API.
+ */
+
 /*
  * Ttk_QueryStyle --
  * 	Look up a style option based on the current state.
@@ -1113,7 +1088,6 @@ Tcl_Obj *Ttk_QueryStyle(
 }
 
 /*
- *---------------------------------------------------------------------------
  * Ttk_ElementSize --
  *	Compute the requested size of the given element.
  */
@@ -1143,15 +1117,8 @@ Ttk_ElementSize(
 }
 
 /*
- *---------------------------------------------------------------------------
- *
  * Ttk_DrawElement --
- *
- *	This procedure draw the given widget element in a given drawable area.
- *
- * Results:
- *	Element is drawn.
- *
+ *	Draw the given widget element in a given drawable area.
  */
 
 void
@@ -1174,8 +1141,8 @@ Ttk_DrawElement(
 	tkwin, d, b, state);
 }
 
-/*========================================================================
- * Style ensemble implementation:
+/*------------------------------------------------------------------------
+ * +++ 'style' command ensemble procedures.
  */
 
 /*
@@ -1203,9 +1170,28 @@ static int EnumerateHashTable(Tcl_Interp *interp, Tcl_HashTable *ht)
     return TCL_OK;
 }
 
+/* HashTableToDict --
+ * 	Helper routine.  Converts a TCL_STRING_KEYS Tcl_HashTable 
+ * 	with Tcl_Obj * entries into a dictionary.
+ */
+static Tcl_Obj* HashTableToDict(Tcl_HashTable *ht)
+{
+    Tcl_HashSearch search;
+    Tcl_Obj *result = Tcl_NewListObj(0, NULL);
+    Tcl_HashEntry *entryPtr = Tcl_FirstHashEntry(ht, &search);
 
-/*
- * style map $style ? -resource statemap ... ?
+    while (entryPtr != NULL) {
+	Tcl_Obj *nameObj = Tcl_NewStringObj(Tcl_GetHashKey(ht, entryPtr),-1);
+	Tcl_Obj *valueObj = (Tcl_Obj*)Tcl_GetHashValue(entryPtr);
+	Tcl_ListObjAppendElement(NULL, result, nameObj);
+	Tcl_ListObjAppendElement(NULL, result, valueObj);
+	entryPtr = Tcl_NextHashEntry(&search);
+    }
+
+    return result;
+}
+
+/* style map $style ? -resource statemap ... ?
  *
  * 	Note that resource names are unconstrained; the Style
  * 	doesn't know what resources individual elements may use.
@@ -1223,14 +1209,33 @@ StyleMapCmd(
     Style *stylePtr;
     int i;
 
-    if (objc < 3 || objc % 2 != 1) {
-	Tcl_WrongNumArgs(interp,2,objv,"style ?-option statemap...?");
+    if (objc < 3) {
+usage:
+	Tcl_WrongNumArgs(interp,2,objv,"style ?-option ?value...??");
 	return TCL_ERROR;
     }
 
     styleName = Tcl_GetString(objv[2]);
     stylePtr = Ttk_GetStyle(theme, styleName);
 
+    /* NOTE: StateMaps are actually Tcl_Obj *s, so HashTableToDict works
+     * for settingsTable.
+     */
+    if (objc == 3) {		/* style map $styleName */
+	Tcl_SetObjResult(interp, HashTableToDict(&stylePtr->settingsTable));
+	return TCL_OK;
+    } else if (objc == 4) {	/* style map $styleName -option */
+	const char *optionName = Tcl_GetString(objv[3]);
+	Tcl_HashEntry *entryPtr = 
+	    Tcl_FindHashEntry(&stylePtr->settingsTable, optionName);
+	if (entryPtr) {
+	    Tcl_SetObjResult(interp, (Tcl_Obj*)Tcl_GetHashValue(entryPtr));
+	}
+	return TCL_OK;
+    } else if (objc % 2 != 1) {
+	goto usage;
+    }
+
     for (i = 3; i < objc; i += 2) {
 	const char *optionName = Tcl_GetString(objv[i]);
 	Tcl_Obj *stateMap = objv[i+1];
@@ -1256,11 +1261,9 @@ StyleMapCmd(
     return TCL_OK;
 }
 
-/*
- * style default $style -resource value...
+/* style configure $style -option ?value...
  */
-static int
-StyleDefaultCmd(
+static int StyleConfigureCmd(
     ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj * CONST objv[])
 {
     StylePackageData *pkgPtr = (StylePackageData *)clientData;
@@ -1269,14 +1272,30 @@ StyleDefaultCmd(
     Style *stylePtr;
     int i;
 
-    if (objc < 3 || objc % 2 != 1) {
-	Tcl_WrongNumArgs(interp,2,objv,"style ?-option statemap...?");
+    if (objc < 3) {
+usage:
+	Tcl_WrongNumArgs(interp,2,objv,"style ?-option ?value...??");
 	return TCL_ERROR;
     }
 
     styleName = Tcl_GetString(objv[2]);
     stylePtr = Ttk_GetStyle(theme, styleName);
 
+    if (objc == 3) {		/* style default $styleName */
+	Tcl_SetObjResult(interp, HashTableToDict(&stylePtr->defaultsTable));
+	return TCL_OK;
+    } else if (objc == 4) {	/* style default $styleName -option */
+	const char *optionName = Tcl_GetString(objv[3]);
+	Tcl_HashEntry *entryPtr = 
+	    Tcl_FindHashEntry(&stylePtr->defaultsTable, optionName);
+	if (entryPtr) {
+	    Tcl_SetObjResult(interp, (Tcl_Obj*)Tcl_GetHashValue(entryPtr));
+	}
+	return TCL_OK;
+    } else if (objc % 2 != 1) {
+	goto usage;
+    }
+
     for (i = 3; i < objc; i += 2) {
 	const char *optionName = Tcl_GetString(objv[i]);
 	Tcl_Obj *value = objv[i+1];
@@ -1296,8 +1315,7 @@ StyleDefaultCmd(
     return TCL_OK;
 }
 
-/*
- * style theme create name ?-parent $theme? ?-settings { script }?
+/* style theme create name ?-parent $theme? ?-settings { script }?
  */
 static int StyleThemeCreateCmd(
     ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj * CONST objv[])
@@ -1360,8 +1378,7 @@ static int StyleThemeCreateCmd(
     }
 }
 
-/*
- * style theme names --
+/* style theme names --
  * 	Return list of registered themes.
  */
 static int StyleThemeNamesCmd(
@@ -1371,8 +1388,7 @@ static int StyleThemeNamesCmd(
     return EnumerateHashTable(interp, &pkgPtr->themeTable);
 }
 
-/*
- * style theme settings $theme $script
+/* style theme settings $theme $script
  *
  * 	Temporarily sets the current theme to $themeName,
  * 	evaluates $script, then restores the old theme.
@@ -1489,11 +1505,9 @@ static int StyleElementOptionsCmd(
     return TCL_ERROR;
 }
 
-/*
- * style layout name ?spec?
+/* style layout name ?spec?
  */
-static int
-StyleLayoutCmd(
+static int StyleLayoutCmd(
     ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj * CONST objv[])
 {
     StylePackageData *pkgPtr = (StylePackageData *)clientData;
@@ -1501,26 +1515,33 @@ StyleLayoutCmd(
     const char *layoutName;
     Ttk_LayoutTemplate layoutTemplate;
 
-    if (objc != 4) {
-	Tcl_WrongNumArgs(interp, 2, objv, "name spec");
+    if (objc < 3 || objc > 4) {
+	Tcl_WrongNumArgs(interp, 2, objv, "name ?spec?");
 	return TCL_ERROR;
     }
 
     layoutName = Tcl_GetString(objv[2]);
-    layoutTemplate = Ttk_ParseLayoutTemplate(interp, objv[3]);
-    if (!layoutTemplate) {
-	return TCL_ERROR;
-    }
 
-    if (!Ttk_RegisterLayoutTemplate(theme, layoutName, layoutTemplate)) {
-	Tcl_AppendResult(interp,"Layout ",layoutName," already defined",NULL);
-	return TCL_ERROR;
+    if (objc == 3) {
+	layoutTemplate = Ttk_FindLayoutTemplate(theme, layoutName);
+	if (!layoutTemplate) {
+	    Tcl_AppendResult(interp, "Layout ", layoutName, " not found",
+		    NULL);
+	    return TCL_ERROR;
+	}
+	Tcl_SetObjResult(interp, Ttk_UnparseLayoutTemplate(layoutTemplate));
+    } else {
+	layoutTemplate = Ttk_ParseLayoutTemplate(interp, objv[3]);
+	if (!layoutTemplate) {
+	    return TCL_ERROR;
+	}
+	Ttk_RegisterLayoutTemplate(theme, layoutName, layoutTemplate);
+	ThemeChanged(pkgPtr);
     }
     return TCL_OK;
 }
 
-/*
- *  style theme use $theme --
+/*  style theme use $theme --
  *  	Sets the current theme to $theme
  */
 static int
@@ -1546,8 +1567,7 @@ StyleThemeUseCmd(
     return Ttk_UseTheme(interp, theme);
 }
 
-/*---------------------------------------------------------------------------
- *
+/*
  * StyleObjCmd --
  *	Implementation of the [style] command.
  */
@@ -1574,7 +1594,8 @@ struct Ensemble StyleElementEnsemble[] = {
 };
 
 struct Ensemble StyleEnsemble[] = {
-    { "default", StyleDefaultCmd, 0 },
+    { "configure", StyleConfigureCmd, 0 },
+    { "default", StyleConfigureCmd, 0 }, /* TEMP: for pre-0.7 compatibility */
     { "map", StyleMapCmd, 0 },
     { "layout", StyleLayoutCmd, 0 },
     { "theme", 0, StyleThemeEnsemble },
