@@ -4,7 +4,7 @@
  *
  * Copyright (c) 2003 Joe English.  Freely redistributable.
  *
- * tkTheme.h,v 1.103 2005/05/31 01:50:07 jenglish Exp
+ * tkTheme.h,v 1.110 2005/10/15 02:15:47 jenglish Exp
  */
 
 #ifndef TKTHEME_H
@@ -21,8 +21,8 @@ extern "C" {
 #	define TTKAPI DLLIMPORT
 #endif
 
-#define TILE_VERSION	"0.6"
-#define TILE_PATCHLEVEL	"0.6.4"
+#define TILE_VERSION	"0.7.2"
+#define TILE_PATCHLEVEL	TILE_VERSION
 
 /*------------------------------------------------------------------------
  * +++ Defaults for element option specifications.
@@ -168,6 +168,7 @@ TTKAPI Ttk_Box Ttk_PlaceBox(Ttk_Box *cavity, int w, int h, Ttk_Side side, Ttk_St
 #define TTK_FILL_BOTH	(0xF)	/* -sticky nswe */
 
 TTKAPI int Ttk_GetStickyFromObj(Tcl_Interp *, Tcl_Obj *, Ttk_Sticky *);
+TTKAPI Tcl_Obj *Ttk_NewStickyObj(Ttk_Sticky);
 
 /*
  * Extra bits for layout specifications:
@@ -182,6 +183,9 @@ TTKAPI int Ttk_GetStickyFromObj(Tcl_Interp *, Tcl_Obj *, Ttk_Sticky *);
 #define TTK_CHILDREN	(0x1000)/* for LayoutSpecs -- children follow */
 #define TTK_LAYOUT_END	(0x2000)/* for LayoutSpecs -- end of child list */
 
+#define _TTK_MASK_STICK (0x0F)	/* See Ttk_UnparseLayout() */
+#define _TTK_MASK_PACK	(0xF0)	/* See Ttk_UnparseLayout(), packStrings */
+
 /*------------------------------------------------------------------------
  * +++ Themes.
  */
@@ -308,8 +312,7 @@ extern void Ttk_LayoutSize(Ttk_Layout,Ttk_State,int *widthPtr,int *heightPtr);
 extern void Ttk_PlaceLayout(Ttk_Layout, Ttk_State, Ttk_Box);
 extern void Ttk_DrawLayout(Ttk_Layout, Ttk_State, Drawable);
 
-extern void Ttk_RebindLayout(
-    Ttk_Layout, void *recordPtr, Tk_OptionTable, Tk_Window tkwin);
+extern void Ttk_RebindSublayout(Ttk_Layout, void *recordPtr);
 
 extern Ttk_LayoutNode *Ttk_LayoutIdentify(Ttk_Layout, int x, int y);
 extern Ttk_LayoutNode *Ttk_LayoutFindNode(Ttk_Layout, const char *nodeName);
@@ -358,17 +361,17 @@ extern int Ttk_GetButtonDefaultStateFromObj(Tcl_Interp *, Tcl_Obj *, int *);
 
 typedef enum 			/* -compound option values */
 {
-    TK_COMPOUND_LAYOUT_NONE,	/* image if specified, otherwise text */
-    TK_COMPOUND_LAYOUT_TEXT,	/* text only */
-    TK_COMPOUND_LAYOUT_IMAGE,	/* image only */
-    TK_COMPOUND_LAYOUT_CENTER,	/* text overlays image */
-    TK_COMPOUND_LAYOUT_TOP,	/* image above text */
-    TK_COMPOUND_LAYOUT_BOTTOM,	/* image below text */
-    TK_COMPOUND_LAYOUT_LEFT,	/* image to left of text */
-    TK_COMPOUND_LAYOUT_RIGHT 	/* image to right of text */
-} Ttk_CompoundLayout;
+    TTK_COMPOUND_NONE,  	/* image if specified, otherwise text */
+    TTK_COMPOUND_TEXT,  	/* text only */
+    TTK_COMPOUND_IMAGE,  	/* image only */
+    TTK_COMPOUND_CENTER,	/* text overlays image */
+    TTK_COMPOUND_TOP,   	/* image above text */
+    TTK_COMPOUND_BOTTOM,	/* image below text */
+    TTK_COMPOUND_LEFT,   	/* image to left of text */
+    TTK_COMPOUND_RIGHT  	/* image to right of text */
+} Ttk_Compound;
 
-extern int Ttk_GetCompoundLayoutFromObj(Tcl_Interp *, Tcl_Obj *, int *);
+extern int Ttk_GetCompoundFromObj(Tcl_Interp *, Tcl_Obj *, int *);
 
 typedef enum { 		/* -orient option values */
     TTK_ORIENT_HORIZONTAL,
@@ -388,6 +391,7 @@ typedef enum { 		/* -orient option values */
 typedef enum { ARROW_UP, ARROW_DOWN, ARROW_LEFT, ARROW_RIGHT } ArrowDirection;
 extern void ArrowSize(int h, ArrowDirection, int *widthPtr, int *heightPtr);
 extern void DrawArrow(Display *, Drawable, GC, Ttk_Box, ArrowDirection);
+extern void FillArrow(Display *, Drawable, GC, Ttk_Box, ArrowDirection);
 
 #ifdef __cplusplus
 }
