@@ -1,5 +1,5 @@
 /*
- * treeview.c,v 1.13 2005/04/20 16:43:45 tallniel Exp
+ * treeview.c,v 1.20 2005/10/08 14:02:47 jenglish Exp
  * Copyright (c) 2004, Joe English
  *
  * Tile widget set: treeview widget.
@@ -126,7 +126,7 @@ static void DetachItem(TreeItem *item)
  * 	Insert an item into the tree after the specified item.
  *
  * Preconditions:
- * 	+ item is currently unlinked
+ * 	+ item is currently detached
  * 	+ prev != NULL ==> prev->parent == parent.
  */
 static void InsertItem(TreeItem *parent, TreeItem *prev, TreeItem *item)
@@ -181,12 +181,12 @@ typedef struct {
     Tcl_Obj 	*headingObj;		/* Heading label */
     Tcl_Obj	*headingImageObj;	/* Heading image */
     Tcl_Obj	*headingCommandObj;	/* Command to execute */
-    Tcl_Obj 	*headingStateObj;	/* @@@ testing ... */ 
+    Tcl_Obj 	*headingStateObj;	/* @@@ testing ... */
     Ttk_State	headingState;		/* ... */
 
-    /* Temporary storage for cell data 
+    /* Temporary storage for cell data
      */
-    Tcl_Obj 	*data;		
+    Tcl_Obj 	*data;
 } TreeColumn;
 
 static void InitColumn(TreeColumn *column)
@@ -221,7 +221,7 @@ static Tk_OptionSpec ColumnOptionSpecs[] =
 {
     {TK_OPTION_INT, "-width", "width", "Width",
 	DEF_COLWIDTH, -1, Tk_Offset(TreeColumn,width),
-	0,0,0 },
+	0,0,0/*|GEOMETRY_CHANGED*/ },
     {TK_OPTION_ANCHOR, "-anchor", "anchor", "Anchor",
 	"w", Tk_Offset(TreeColumn,anchorObj), -1,
 	0,0,0 },
@@ -277,16 +277,16 @@ static Tk_OptionSpec RowOptionSpecs[] =
 
 /*------------------------------------------------------------------------
  * +++ -show option:
- * TODO: Implement SHOW_BRANCHES. 
+ * TODO: Implement SHOW_BRANCHES.
  */
 
 #define SHOW_TREE 	(0x1) 	/* Show tree column? */
-#define SHOW_HEADINGS	(0x2)	/* Show heading row? */	
+#define SHOW_HEADINGS	(0x2)	/* Show heading row? */
 
 #define DEFAULT_SHOW	"tree headings"
 
 static const char *showStrings[] = {
-    "tree", "headings", NULL 
+    "tree", "headings", NULL
 };
 
 static int GetEnumSetFromObj(
@@ -305,7 +305,7 @@ static int GetEnumSetFromObj(
     for (i = 0; i < objc; ++i) {
 	int index;
 	if (TCL_OK != Tcl_GetIndexFromObj(
-		    interp, objv[i], table, "value", TCL_EXACT, &index)) 
+		    interp, objv[i], table, "value", TCL_EXACT, &index))
 	{
 	    return TCL_ERROR;
 	}
@@ -356,7 +356,7 @@ typedef struct
     Tcl_Obj *columnsObj;	/* List of symbolic column names */
     Tcl_Obj *displayColumnsObj;	/* List of columns to display */
 
-    Tcl_Obj *heightObj;		/* height (pixels) */
+    Tcl_Obj *heightObj;		/* height (rows) */
     Tcl_Obj *paddingObj;	/* internal padding */
 
     Tcl_Obj *showObj;		/* -show list */
@@ -370,7 +370,7 @@ typedef struct
     int nColumns; 		/* #columns */
     unsigned showFlags;		/* bitmask of subparts to display */
 
-    int *displayColumns;	/* List of column indices for display */
+    TreeColumn **displayColumns; /* List of columns for display (incl tree) */
     int nDisplayColumns;	/* #display columns */
     Ttk_Box headingArea;	/* Display area for column headings */
     Ttk_Box treeArea;   	/* Display area for tree */
@@ -398,7 +398,7 @@ static Tk_OptionSpec TreeviewOptionSpecs[] =
     {TK_OPTION_STRING, "-displaycolumns","displayColumns","DisplayColumns",
 	"", Tk_Offset(Treeview,tree.displayColumnsObj), -1,
 	0,0,DCOLUMNS_CHANGED | GEOMETRY_CHANGED },
-    {TK_OPTION_STRING, "-show", "show", "Show", 
+    {TK_OPTION_STRING, "-show", "show", "Show",
 	DEFAULT_SHOW, Tk_Offset(Treeview,tree.showObj), -1,
 	0,0,SHOW_CHANGED | GEOMETRY_CHANGED },
 
@@ -450,11 +450,9 @@ static Tcl_Obj *unshare(Tcl_Obj *objPtr)
  * 	Rebind, place, and draw a layout + object combination.
  */
 static void DisplayLayout(
-    Ttk_Layout layout,
-    void *recordPtr, Tk_OptionTable optionTable, Tk_Window tkwin,
-    Ttk_State state, Ttk_Box b, Drawable d)
+    Ttk_Layout layout, void *recordPtr, Ttk_State state, Ttk_Box b, Drawable d)
 {
-    Ttk_RebindLayout(layout, recordPtr, optionTable, tkwin);
+    Ttk_RebindSublayout(layout, recordPtr);
     Ttk_PlaceLayout(layout, state, b);
     Ttk_DrawLayout(layout, state, d);
 }
@@ -574,15 +572,14 @@ static TreeColumn *FindColumn(
 
     if (sscanf(Tcl_GetString(columnIDObj), "#%d", &column) == 1)
     {	/* Display column specification, #n */
-	if (column == 0) {
-	    return &tv->tree.column0;
-	} else if (column > 0 && column <= tv->tree.nDisplayColumns) {
-	    return tv->tree.columns + tv->tree.displayColumns[column-1];
+	if (column >= 0 && column < tv->tree.nDisplayColumns) {
+	    return tv->tree.displayColumns[column];
 	}
 	/* else */
 	Tcl_ResetResult(interp);
 	Tcl_AppendResult(interp,
-	    "Column ", Tcl_GetString(columnIDObj), " out of range", NULL);
+	    "Column ", Tcl_GetString(columnIDObj), " out of range",
+	    NULL);
 	return NULL;
     }
 
@@ -664,40 +661,47 @@ static int TreeviewInitColumns(Tcl_Interp *interp, Treeview *tv)
 }
 
 /* + TreeviewInitDisplayColumns --
+ * 	Initializes the 'displayColumns' array.
+ *
+ * 	Note that displayColumns[0] is always the tree column,
+ * 	even when SHOW_TREE is not set.
+ *
+ * @@@ TODO: disallow duplicated columns
  */
 static int TreeviewInitDisplayColumns(Tcl_Interp *interp, Treeview *tv)
 {
     Tcl_Obj **dcolumns;
     int index, ndcols;
-    int *displayColumns = 0;
+    TreeColumn **displayColumns = 0;
 
     if (Tcl_ListObjGetElements(interp,
-		tv->tree.displayColumnsObj, &ndcols, &dcolumns) != TCL_OK) {
+	    tv->tree.displayColumnsObj, &ndcols, &dcolumns) != TCL_OK) {
 	return TCL_ERROR;
     }
 
     if (ndcols == 0) {
 	ndcols = tv->tree.nColumns;
-	displayColumns = (int*)ckalloc(ndcols * sizeof(int));
+	displayColumns = (TreeColumn**)ckalloc((ndcols+1)*sizeof(TreeColumn*));
 	for (index = 0; index < ndcols; ++index) {
-	    displayColumns[index] = index;
+	    displayColumns[index+1] = tv->tree.columns + index;
 	}
     } else {
-	displayColumns = (int*)ckalloc(ndcols * sizeof(int));
+	displayColumns = (TreeColumn**)ckalloc((ndcols+1)*sizeof(TreeColumn*));
 	for (index = 0; index < ndcols; ++index) {
 	    int columnIndex = ColumnIndex(interp, tv, dcolumns[index]);
 	    if (columnIndex == -1) {
 		ckfree((ClientData)displayColumns);
 		return TCL_ERROR;
 	    }
-	    displayColumns[index] = columnIndex;
+	    displayColumns[index+1] = tv->tree.columns + columnIndex;
 	}
     }
+    displayColumns[0] = &tv->tree.column0;
 
     if (tv->tree.displayColumns)
 	ckfree((ClientData)tv->tree.displayColumns);
     tv->tree.displayColumns = displayColumns;
-    tv->tree.nDisplayColumns = ndcols;
+    tv->tree.nDisplayColumns = ndcols + 1;
 
     return TCL_OK;
 }
@@ -721,10 +725,10 @@ static int TreeviewInitialize(Tcl_Interp *interp, void *recordPtr)
 	Tk_CreateOptionTable(interp, HeadingOptionSpecs);
     tv->tree.rowOptionTable = Tk_CreateOptionTable(interp, RowOptionSpecs);
 
-    tv->tree.itemLayout 
-	= tv->tree.cellLayout 
-	= tv->tree.headingLayout 
-	= tv->tree.rowLayout 
+    tv->tree.itemLayout
+	= tv->tree.cellLayout
+	= tv->tree.headingLayout
+	= tv->tree.rowLayout
 	= 0;
 
     Tcl_InitHashTable(&tv->tree.columnNames, TCL_STRING_KEYS);
@@ -863,7 +867,7 @@ static int ConfigureItem(
     /* All OK.
      */
     Tk_FreeSavedOptions(&savedOptions);
-    WidgetChanged(&tv->core, REDISPLAY_REQUIRED);
+    TtkRedisplayWidget(&tv->core);
     return TCL_OK;
 
 error:
@@ -879,7 +883,7 @@ static int ConfigureColumn(
     int objc, Tcl_Obj *const objv[])
 {
     Tk_SavedOptions savedOptions;
-    unsigned int mask;
+    int mask;
 
     if (Tk_SetOptions(interp, (ClientData)column,
 	    tv->tree.columnOptionTable, objc, objv, tv->core.tkwin,
@@ -894,7 +898,11 @@ static int ConfigureColumn(
 	goto error;
     }
 
-    WidgetChanged(&tv->core, RELAYOUT_REQUIRED);
+    if (mask & GEOMETRY_CHANGED) {
+	TtkResizeWidget(&tv->core);
+    }
+    TtkRedisplayWidget(&tv->core);
+
     Tk_FreeSavedOptions(&savedOptions);
     return TCL_OK;
 
@@ -911,7 +919,7 @@ static int ConfigureHeading(
     int objc, Tcl_Obj *const objv[])
 {
     Tk_SavedOptions savedOptions;
-    unsigned int mask;
+    int mask;
 
     if (Tk_SetOptions(interp, (ClientData)column,
 	    tv->tree.headingOptionTable, objc, objv, tv->core.tkwin,
@@ -924,7 +932,7 @@ static int ConfigureHeading(
     if ((mask & STATE_CHANGED) && column->headingStateObj) {
 	Ttk_StateSpec stateSpec;
 	if (Ttk_GetStateSpecFromObj(
-		interp, column->headingStateObj, &stateSpec) != TCL_OK)  
+		interp, column->headingStateObj, &stateSpec) != TCL_OK)
 	{
 	    goto error;
 	}
@@ -934,7 +942,7 @@ static int ConfigureHeading(
 	Tcl_IncrRefCount(column->headingStateObj);
     }
 
-    WidgetChanged(&tv->core, REDISPLAY_REQUIRED);
+    TtkRedisplayWidget(&tv->core);
     Tk_FreeSavedOptions(&savedOptions);
     return TCL_OK;
 
@@ -965,19 +973,39 @@ static int CountRows(TreeItem *item)
 }
 
 /* + TreeWidth --
- * 	Compute the requested tree width from the sum of column widths.
+ * 	Compute the requested tree width from the sum of visible column widths.
  */
 static int TreeWidth(Treeview *tv)
 {
-    int width = (tv->tree.showFlags&SHOW_TREE) ? tv->tree.column0.width : 0;
-    int i;
+    int i = (tv->tree.showFlags&SHOW_TREE) ? 0 : 1;
+    int width = 0;
 
-    for (i = 0; i < tv->tree.nDisplayColumns; ++i) {
-	width += tv->tree.columns[tv->tree.displayColumns[i]].width;
+    while (i < tv->tree.nDisplayColumns) {
+	width += tv->tree.displayColumns[i++]->width;
     }
     return width;
 }
 
+/* + PlaceColumns -
+ * 	Adjust final column width to fill available space.
+ * 	@@@ NB: still not right -- see paned.c for correct algorithm
+ */
+static void PlaceColumns(Treeview *tv, int availableWidth)
+{
+#   define MIN_WIDTH 24
+    int colno = (tv->tree.showFlags & SHOW_TREE) ? 0 : 1;
+    TreeColumn *column = tv->tree.displayColumns[colno];
+
+    while (++colno < tv->tree.nDisplayColumns) {
+	availableWidth -= column->width;
+	column = tv->tree.displayColumns[colno];
+    }
+    column->width = availableWidth;
+    if (column->width < MIN_WIDTH) {
+	column->width = MIN_WIDTH;
+    }
+}
+
 /* + IdentifyRow --
  * 	Recursive search for item at specified y position.
  * 	Main work routine for IdentifyItem(()
@@ -1027,25 +1055,17 @@ static TreeItem *IdentifyItem(Treeview *tv, int y, Ttk_Box *itemPos)
  */
 static int IdentifyDisplayColumn(Treeview *tv, int x, int *x1)
 {
-    TreeColumn *column =  &tv->tree.column0;
-    int columnNumber = 0;
+    int colno = (tv->tree.showFlags & SHOW_TREE) ? 0 : 1;
     int xpos = tv->tree.treeArea.x;
 
-    if (!(tv->tree.showFlags & SHOW_TREE)) {
-	if (!tv->tree.nDisplayColumns) 
-	    return -1;
-	column = tv->tree.columns + tv->tree.displayColumns[columnNumber];
-	++columnNumber;
-    }
-
-    while (columnNumber <= tv->tree.nDisplayColumns) {
+    while (colno < tv->tree.nDisplayColumns) {
+	TreeColumn *column = tv->tree.displayColumns[colno];
 	int next_xpos = xpos + column->width;
 	if (xpos <= x && x <= next_xpos + HALO) {
 	    *x1 = next_xpos;
-	    return columnNumber;
+	    return colno;
 	}
-	column = tv->tree.columns + tv->tree.displayColumns[columnNumber];
-	++columnNumber;
+	++colno;
 	xpos = next_xpos;
     }
 
@@ -1110,6 +1130,7 @@ static Ttk_Layout TreeviewGetLayout(
  * Side effects:
  * 	Computes headingArea and treeArea.
  * 	Computes subtree height.
+ * 	Invokes scroll callbacks.
  */
 static void TreeviewDoLayout(void *clientData)
 {
@@ -1122,6 +1143,8 @@ static void TreeviewDoLayout(void *clientData)
 	? Ttk_LayoutNodeInternalParcel(tv->core.layout,clientNode)
 	: Ttk_WinBox(tv->core.tkwin) ;
 
+    PlaceColumns(tv, tv->tree.treeArea.width);
+
     if (showFlags & SHOW_HEADINGS) {
 	tv->tree.headingArea = Ttk_PackBox(
 	    &tv->tree.treeArea, 1, HEADINGHEIGHT, TTK_SIDE_TOP);
@@ -1180,32 +1203,16 @@ static Ttk_State ItemState(Treeview *tv, TreeItem *item)
  */
 static void DrawHeadings(Treeview *tv, Drawable d, Ttk_Box b)
 {
-    Ttk_Layout layout = tv->tree.headingLayout;
+    int i = (tv->tree.showFlags & SHOW_TREE) ? 0 : 1;
     int x = 0;
-    int height = HEADINGHEIGHT;
-    Ttk_Box parcel;
-    int i;
 
-    if (tv->tree.showFlags & SHOW_TREE) {
-	/* Draw #0:
-	 */
-	DisplayLayout(layout,
-		&tv->tree.column0, tv->tree.headingOptionTable, tv->core.tkwin,
-		tv->tree.column0.headingState,
-		Ttk_MakeBox(b.x, b.y, tv->tree.column0.width, b.height), d);
-	x = tv->tree.column0.width;
-    }
-
-    /* Draw data column headings:
-     */
-    for (i = 0; i < tv->tree.nDisplayColumns; ++i) {
-	int dataColumn = tv->tree.displayColumns[i];
-	TreeColumn *column = &tv->tree.columns[dataColumn];
-	parcel = Ttk_MakeBox(b.x+x, b.y, column->width, height);
-	DisplayLayout(layout,
-	    column, tv->tree.headingOptionTable, tv->core.tkwin,
-	    column->headingState, parcel, d);
+    while (i < tv->tree.nDisplayColumns) {
+	TreeColumn *column = tv->tree.displayColumns[i];
+	Ttk_Box parcel = Ttk_MakeBox(b.x+x, b.y, column->width, b.height);
+	DisplayLayout(tv->tree.headingLayout,
+	    column, column->headingState, parcel, d);
 	x += column->width;
+	++i;
     }
 }
 
@@ -1216,7 +1223,6 @@ static void DrawCells(
     Treeview *tv, TreeItem *item, Drawable d, Ttk_Box b, int x, int y)
 {
     Ttk_Layout layout = tv->tree.cellLayout;
-    Tk_OptionTable optionTable = tv->tree.cellOptionTable;
     Ttk_State state = ItemState(tv, item);
     Ttk_Padding cellPadding = {4, 0, 4, 0};
     int height = ROWHEIGHT;
@@ -1228,18 +1234,15 @@ static void DrawCells(
 	return;
 
     Tcl_ListObjGetElements(NULL, item->valuesObj, &nValues, &values);
+    for (i = 0; i < tv->tree.nColumns; ++i) {
+	tv->tree.columns[i].data = (i < nValues) ? values[i] : 0;
+    }
 
-    for (i = 0; i < tv->tree.nDisplayColumns; ++i) {
-	int dataColumn = tv->tree.displayColumns[i];
-	TreeColumn *column = &tv->tree.columns[dataColumn];
-
-	if (dataColumn < nValues) {
-	    Ttk_Box parcel = Ttk_PadBox(
-		Ttk_MakeBox(b.x+x, b.y+y, column->width, height), cellPadding);
-	    column->data = values[dataColumn];
-	    DisplayLayout(layout, column, optionTable, tv->core.tkwin,
-			 state, parcel, d);
-	}
+    for (i = 1; i < tv->tree.nDisplayColumns; ++i) {
+	TreeColumn *column = tv->tree.displayColumns[i];
+	Ttk_Box parcel = Ttk_PadBox(
+	    Ttk_MakeBox(b.x+x, b.y+y, column->width, height), cellPadding);
+	DisplayLayout(layout, column, state, parcel, d);
 	x += column->width;
     }
 }
@@ -1264,19 +1267,16 @@ static void DrawItem(
     {
 	Ttk_Box rowBox = Ttk_MakeBox(b.x, b.y+y, TreeWidth(tv), height);
 	Ttk_Layout rowLayout =  tv->tree.rowLayout;
-	TreeRow row; 
-	DisplayLayout(rowLayout,&row,tv->tree.rowOptionTable,tv->core.tkwin,
-			state, rowBox, d);
+	TreeRow row;	/* @@@ double-check this. */
+	DisplayLayout(rowLayout, &row, state, rowBox, d);
     }
 
-
     /* Draw tree label:
      */
     if (tv->tree.showFlags & SHOW_TREE) {
 	int colwidth = tv->tree.column0.width;
 	Ttk_Box parcel = Ttk_MakeBox(b.x + x, b.y + y, colwidth - x, height);
-	DisplayLayout(layout, item, tv->tree.itemOptionTable, tv->core.tkwin,
-		    state, parcel, d);
+	DisplayLayout(layout, item, state, parcel, d);
 	x = colwidth;
     } else {
 	x = 0;
@@ -1315,7 +1315,7 @@ static int DrawSubtree(
  * 	Draw a sequence of items and their visible descendants.
  *
  * Returns:
- * 	Row number of the last item drawn. 
+ * 	Row number of the last item drawn.
  */
 static int DrawForest(
     Treeview *tv, TreeItem *item, Drawable d, Ttk_Box b, int depth, int row)
@@ -1393,7 +1393,7 @@ static int AncestryCheck(
 	    Tcl_ResetResult(interp);
 	    Tcl_AppendResult(interp,
 		    "Cannot insert ", ItemName(tv, item),
-		    " as a child of ", ItemName(tv, parent),
+		    " as a descendant of ", ItemName(tv, parent),
 		    NULL);
 	    return 0;
 	}
@@ -1445,7 +1445,7 @@ static int RowNumber(Treeview *tv, TreeItem *item)
 	} else {
 	    while (!p->next && p && p->parent)
 		p = p->parent;
-	    if (p) 
+	    if (p)
 		p = p->next;
 	}
     }
@@ -1508,18 +1508,30 @@ static int TreeviewChildrenCommand(
 	    child = next;
 	}
 
-	/* Insert new children:
+	/* Detach new children from their current locations:
+	 */
+	for (i=0; newChildren[i]; ++i) {
+	    DetachItem(newChildren[i]);
+	}
+
+	/* Reinsert new children:
 	 * Note: it is not an error for an item to be listed more than once,
 	 * though it probably should be...
 	 */
 	child = 0;
 	for (i=0; newChildren[i]; ++i) {
+	    if (newChildren[i]->parent) {
+		/* This is a duplicate element which has already been
+		 * inserted.  Ignore it.
+		 */
+		continue;
+	    }
 	    InsertItem(item, child, newChildren[i]);
 	    child = newChildren[i];
 	}
 
 	ckfree((ClientData)newChildren);
-	WidgetChanged(&tv->core, REDISPLAY_REQUIRED);
+	TtkRedisplayWidget(&tv->core);
     }
 
     return TCL_OK;
@@ -1707,8 +1719,7 @@ static int TreeviewIdentifyCommand(
 	} else if (item) {
 	    Ttk_Layout layout = tv->tree.itemLayout;
 	    Ttk_LayoutNode *element;
-	    Ttk_RebindLayout(layout, item,
-		    tv->tree.itemOptionTable, tv->core.tkwin);
+	    Ttk_RebindSublayout(layout, item);
 	    Ttk_PlaceLayout(layout, ItemState(tv,item), itemBox);
 	    element = Ttk_LayoutIdentify(layout, x, y);
 
@@ -1823,7 +1834,7 @@ static int TreeviewHeadingCommand(
     }
 }
 
-/* + $tv set $item $column ?value?
+/* + $tv set $item ?$column ?value??
  * 	Query or configure cell values
  */
 static int TreeviewSetCommand(
@@ -1834,12 +1845,39 @@ static int TreeviewSetCommand(
     TreeColumn *column;
     int columnNumber;
 
-    if (objc < 4 || objc > 5) {
-	Tcl_WrongNumArgs(interp, 2, objv, "item column ?value?");
+    if (objc < 3 || objc > 5) {
+	Tcl_WrongNumArgs(interp, 2, objv, "item ?column ?value??");
 	return TCL_ERROR;
     }
     if (!(item = FindItem(interp, tv, objv[2])))
 	return TCL_ERROR;
+
+    /* Make sure -values exists:
+     */
+    if (!item->valuesObj) {
+	item->valuesObj = Tcl_NewListObj(0,0);
+	Tcl_IncrRefCount(item->valuesObj);
+    }
+
+    if (objc == 3) {
+	/* Return dictionary:
+	 */
+	Tcl_Obj *result = Tcl_NewListObj(0,0);
+	Tcl_Obj *value;
+	for (columnNumber=0; columnNumber<tv->tree.nColumns; ++columnNumber) {
+	    Tcl_ListObjIndex(interp, item->valuesObj, columnNumber, &value);
+	    if (value) {
+		Tcl_ListObjAppendElement(interp, result,
+			tv->tree.columns[columnNumber].idObj);
+		Tcl_ListObjAppendElement(interp, result, value);
+	    }
+	}
+	Tcl_SetObjResult(interp, result);
+	return TCL_OK;
+    }
+
+    /* else -- get or set column
+     */
     if (!(column = FindColumn(interp, tv, objv[3])))
 	return TCL_ERROR;
 
@@ -1853,15 +1891,19 @@ static int TreeviewSetCommand(
      * since item->valuesObj is guaranteed to be a list.
      */
     columnNumber = column - tv->tree.columns;
-    if (objc == 5) {
+
+    if (objc == 4) {	/* get column */
+	Tcl_Obj *result = 0;
+	Tcl_ListObjIndex(interp, item->valuesObj, columnNumber, &result);
+	if (!result) {
+	    result = Tcl_NewStringObj("",0);
+	}
+	Tcl_SetObjResult(interp, result);
+	return TCL_OK;
+    } else {		/* set column */
 	int length;
 
-	if (item->valuesObj) {
-	    item->valuesObj = unshare(item->valuesObj);
-	} else {
-	    item->valuesObj = Tcl_NewListObj(0,0);
-	    Tcl_IncrRefCount(item->valuesObj);
-	}
+	item->valuesObj = unshare(item->valuesObj);
 
 	/* Make sure -values is fully populated:
 	 */
@@ -1875,13 +1917,7 @@ static int TreeviewSetCommand(
 	/* Set value:
 	 */
 	Tcl_ListObjReplace(interp,item->valuesObj,columnNumber,1,1,objv+4);
-	WidgetChanged(&tv->core, REDISPLAY_REQUIRED);
-	return TCL_OK;
-    } else {
-	Tcl_Obj *result = 0;
-	Tcl_ListObjIndex(interp, item->valuesObj, columnNumber, &result);
-	Tcl_SetObjResult(interp, result);
-
+	TtkRedisplayWidget(&tv->core);
 	return TCL_OK;
     }
 }
@@ -1961,7 +1997,7 @@ static int TreeviewInsertCommand(
     Tcl_SetHashValue(entryPtr, newItem);
     newItem->entryPtr = entryPtr;
     InsertItem(parent, sibling, newItem);
-    WidgetChanged(&tv->core, REDISPLAY_REQUIRED);
+    TtkRedisplayWidget(&tv->core);
 
     Tcl_SetObjResult(interp, ItemID(tv, newItem));
     return TCL_OK;
@@ -1998,7 +2034,7 @@ static int TreeviewDetachCommand(
 	DetachItem(items[i]);
     }
 
-    WidgetChanged(&tv->core, REDISPLAY_REQUIRED);
+    TtkRedisplayWidget(&tv->core);
     ckfree((ClientData)items);
     return TCL_OK;
 }
@@ -2059,7 +2095,7 @@ static int TreeviewDeleteCommand(
     }
 
     ckfree((ClientData)items);
-    WidgetChanged(&tv->core, REDISPLAY_REQUIRED);
+    TtkRedisplayWidget(&tv->core);
     return TCL_OK;
 }
 
@@ -2099,12 +2135,18 @@ static int TreeviewMoveCommand(
     if (!AncestryCheck(interp, tv, item, parent))
 	return TCL_ERROR;
 
+    /* Moving an item after itself is a no-op:
+     */
+    if (item == sibling) {
+	return TCL_OK;
+    }
+
     /* Move item:
      */
     DetachItem(item);
     InsertItem(parent, sibling, item);
 
-    WidgetChanged(&tv->core, REDISPLAY_REQUIRED);
+    TtkRedisplayWidget(&tv->core);
     return TCL_OK;
 }
 
@@ -2159,7 +2201,7 @@ static int TreeviewSeeCommand(
 	return TCL_OK;
     }
 
-    WidgetChanged(&tv->core, REDISPLAY_REQUIRED);
+    TtkRedisplayWidget(&tv->core);
     return TCL_OK;
 }
 
@@ -2184,7 +2226,7 @@ static int TreeviewFocusCommand(
 	if (!newFocus)
 	    return TCL_ERROR;
 	tv->tree.focus = newFocus;
-	WidgetChanged(&tv->core, REDISPLAY_REQUIRED);
+	TtkRedisplayWidget(&tv->core);
 	return TCL_OK;
     } else {
 	Tcl_WrongNumArgs(interp, 2, objv, "?newFocus?");
@@ -2197,8 +2239,8 @@ static int TreeviewFocusCommand(
 static int TreeviewSelectionCommand(
     Tcl_Interp *interp, int objc, Tcl_Obj *const objv[], void *recordPtr)
 {
-    enum { 
-	SELECTION_SET, SELECTION_ADD, SELECTION_REMOVE, SELECTION_TOGGLE 
+    enum {
+	SELECTION_SET, SELECTION_ADD, SELECTION_REMOVE, SELECTION_TOGGLE
     };
     static const char *selopStrings[] = {
 	"set", "add", "remove", "toggle", NULL
@@ -2234,7 +2276,7 @@ static int TreeviewSelectionCommand(
 	return TCL_ERROR;
     }
 
-    switch (selop) 
+    switch (selop)
     {
 	case SELECTION_SET:
 	    for (item=tv->tree.root; item; item=NextPreorder(item)) {
@@ -2260,7 +2302,7 @@ static int TreeviewSelectionCommand(
 
     ckfree((ClientData)items);
     SendVirtualEvent(tv->core.tkwin, "TreeviewSelect");
-    WidgetChanged(&tv->core, REDISPLAY_REQUIRED);
+    TtkRedisplayWidget(&tv->core);
 
     return TCL_OK;
 }
@@ -2316,8 +2358,7 @@ WidgetSpec TreeviewWidgetSpec =
     TreeviewGetLayout, 		/* getLayoutProc */
     TreeviewSize, 		/* sizeProc */
     TreeviewDoLayout,		/* layoutProc */
-    TreeviewDisplay,		/* displayProc */
-    WIDGET_SPEC_END		/* sentinel */
+    TreeviewDisplay		/* displayProc */
 };
 
 /*------------------------------------------------------------------------
@@ -2367,71 +2408,68 @@ static const int WIN32_XDRAWLINE_HACK = 0;
 typedef struct
 {
     Tcl_Obj *colorObj;
-    Tcl_Obj *marginObj;
-    Tcl_Obj *diameterObj;
-} ItemIndicator;
+    Tcl_Obj *sizeObj;
+    Tcl_Obj *marginsObj;
+} TreeitemIndicator;
 
-static Ttk_ElementOptionSpec ItemIndicatorOptions[] =
+static Ttk_ElementOptionSpec TreeitemIndicatorOptions[] =
 {
     { "-foreground", TK_OPTION_COLOR,
-	Tk_Offset(ItemIndicator,colorObj), DEFAULT_FOREGROUND },
-    { "-diameter", TK_OPTION_PIXELS,
-	Tk_Offset(ItemIndicator,diameterObj), "6" },
+	Tk_Offset(TreeitemIndicator,colorObj), DEFAULT_FOREGROUND },
+    { "-indicatorsize", TK_OPTION_PIXELS,
+	Tk_Offset(TreeitemIndicator,sizeObj), "12" },
     { "-indicatormargins", TK_OPTION_STRING,
-	Tk_Offset(ItemIndicator,marginObj), "0 2 4 2" },
+	Tk_Offset(TreeitemIndicator,marginsObj), "2 2 4 2" },
     {NULL}
 };
 
-static void ItemIndicatorSize(
+static void TreeitemIndicatorSize(
     void *clientData, void *elementRecord, Tk_Window tkwin,
     int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
 {
-    ItemIndicator *indicator = elementRecord;
-    int diameter = 0;
-    Ttk_GetPaddingFromObj(NULL, tkwin, indicator->marginObj, paddingPtr);
-    Tk_GetPixelsFromObj(NULL, tkwin, indicator->diameterObj, &diameter);
-    *widthPtr = *heightPtr = diameter;
+    TreeitemIndicator *indicator = elementRecord;
+    int size = 0;
+
+    Ttk_GetPaddingFromObj(NULL, tkwin, indicator->marginsObj, paddingPtr);
+    Tk_GetPixelsFromObj(NULL, tkwin, indicator->sizeObj, &size);
+
+    *widthPtr = *heightPtr = size;
 }
 
-static void ItemIndicatorDraw(
+static void TreeitemIndicatorDraw(
     void *clientData, void *elementRecord, Tk_Window tkwin,
     Drawable d, Ttk_Box b, Ttk_State state)
 {
-    ItemIndicator *indicator = elementRecord;
-    XColor *color = Tk_GetColorFromObj(tkwin, indicator->colorObj);
-    GC gc = Tk_GCForColor(color, d);
-    Ttk_Padding padding = Ttk_UniformPadding(0);
-    int w = WIN32_XDRAWLINE_HACK;
-    int cx, cy;
+    TreeitemIndicator *indicator = elementRecord;
+    ArrowDirection direction = 
+	(state & TTK_STATE_OPEN) ? ARROW_DOWN : ARROW_RIGHT;
+    Ttk_Padding margins;
+    XColor *borderColor = Tk_GetColorFromObj(tkwin, indicator->colorObj);
+    XGCValues gcvalues; GC gc; unsigned mask;
 
-    if (state & TTK_STATE_LEAF) {
-	/* don't draw anything ... */
+    if (state & TTK_STATE_LEAF) /* don't draw anything */
 	return;
-    }
 
-    Ttk_GetPaddingFromObj(NULL,tkwin,indicator->marginObj,&padding);
-    b = Ttk_PadBox(b, padding);
+    Ttk_GetPaddingFromObj(NULL,tkwin,indicator->marginsObj,&margins);
+    b = Ttk_PadBox(b, margins);
 
-    XDrawRectangle(Tk_Display(tkwin), d, gc,
-	    b.x, b.y, b.width, b.height);
+    gcvalues.foreground = borderColor->pixel;
+    gcvalues.line_width = 1;
+    mask = GCForeground | GCLineWidth;
+    gc = Tk_GetGC(tkwin, mask, &gcvalues);
 
-    cx = b.x + b.width / 2;
-    cy = b.y + b.height / 2;
-    XDrawLine(Tk_Display(tkwin), d, gc, b.x+2, cy, b.x+b.width-2+w, cy);
+    DrawArrow(Tk_Display(tkwin), d, gc, b, direction);
 
-    if (!(state & TTK_STATE_OPEN)) {
-	/* turn '-' into a '+' */
-	XDrawLine(Tk_Display(tkwin), d, gc, cx, b.y+2, cx, b.y+b.height-2+w);
-    }
+    Tk_FreeGC(Tk_Display(tkwin), gc);
 }
 
-static Ttk_ElementSpec ItemIndicatorElementSpec =
+static Ttk_ElementSpec TreeitemIndicatorElementSpec =
 {
     TK_STYLE_VERSION_2,
-    sizeof(ItemIndicator),
-    ItemIndicatorOptions,
-    ItemIndicatorSize,
-    ItemIndicatorDraw
+    sizeof(TreeitemIndicator),
+    TreeitemIndicatorOptions,
+    TreeitemIndicatorSize,
+    TreeitemIndicatorDraw
 };
 
 /*------------------------------------------------------------------------
@@ -2483,7 +2521,7 @@ DLLEXPORT int Treeview_Init(Tcl_Interp *interp)
     RegisterWidget(interp, "ttk::treeview", &TreeviewWidgetSpec);
 
     Ttk_RegisterElement(interp, theme,
-	    "Treeitem.indicator", &ItemIndicatorElementSpec, 0);
+	    "Treeitem.indicator", &TreeitemIndicatorElementSpec, 0);
     Ttk_RegisterElement(interp, theme, "Treeitem.row", &RowElementSpec, 0);
     Ttk_RegisterElement(interp, theme, "Treeheading.cell", &RowElementSpec, 0);
 
