@@ -1,4 +1,4 @@
-/* widget.h,v 1.43 2005/03/31 17:27:11 jenglish Exp
+/* widget.h,v 1.47 2005/10/08 14:02:47 jenglish Exp
  * Copyright (c) 2003, Joe English
  *
  * Helper routines for widget implementations.
@@ -14,9 +14,7 @@
  * need reconfigure, need redisplay, redisplay pending
  */
 #define WIDGET_DESTROYED	0x0001
-#define UPDATE_PENDING 		0x0002
-#define REDISPLAY_REQUIRED	0x0004	/* widget to be redisplayed */
-#define RELAYOUT_REQUIRED	0x0008	/* recompute geometry */
+#define REDISPLAY_PENDING 	0x0002	/* scheduled call to RedisplayWidget */
 #define WIDGET_REALIZED		0x0010	/* set at first ConfigureNotify */
 #define CURSOR_ON 		0x0020	/* See BlinkCursor() */
 #define WIDGET_USER_FLAG        0x0100  /* 0x0100 - 0x8000 for user flags */
@@ -90,8 +88,6 @@ struct WidgetSpec_
     int 	(*sizeProc)(void *recordPtr, int *widthPtr, int *heightPtr);
     void	(*layoutProc)(void *recordPtr);
     void	(*displayProc)(void *recordPtr, Drawable d);
-
-    enum { WIDGET_SPEC_END } sentinel;	/* to help the compiler catch errors */
 };
 
 /*
@@ -151,7 +147,8 @@ extern int WidgetDestroyed(WidgetCore *);
 extern void WidgetChangeState(WidgetCore *,
 	unsigned int setBits, unsigned int clearBits);
 
-extern void WidgetChanged(WidgetCore *, unsigned flags);
+extern void TtkRedisplayWidget(WidgetCore *);
+extern void TtkResizeWidget(WidgetCore *);
 
 extern void TrackElementState(WidgetCore *);
 extern void BlinkCursor(WidgetCore *);
@@ -164,15 +161,13 @@ extern void CheckStateOption(WidgetCore *, Tcl_Obj *);
 /*
  * Variable traces:
  */
+typedef void (*Ttk_TraceProc)(void *recordPtr, const char *value);
+typedef struct TtkTraceHandle_ Ttk_TraceHandle;
 
-typedef int (*TraceProc)(Tcl_Interp *, void *recordPtr, const char *value);
-typedef struct TraceHandle_ TraceHandle;
-
-extern TraceHandle *TraceVariable(
-    Tcl_Interp *, Tcl_Obj *varnameObj, TraceProc callback, void *clientData);
-extern void UntraceVariable(TraceHandle *);
-extern int FireTrace(TraceHandle *);
-
+extern Ttk_TraceHandle *Ttk_TraceVariable(
+    Tcl_Interp*, Tcl_Obj *varnameObj, Ttk_TraceProc callback, void *clientData);
+extern void Ttk_UntraceVariable(Ttk_TraceHandle *);
+extern int Ttk_FireTrace(Ttk_TraceHandle *);
 
 /*
  * Utility routines for managing -image option:
