@@ -1,14 +1,17 @@
 #
-# aquaTheme.tcl,v 1.12 2005/04/01 17:45:23 jenglish Exp
+# aquaTheme.tcl,v 1.14 2005/09/29 15:34:54 jenglish Exp
 #
 # Tile widget set: Aqua theme (OSX native look and feel)
 #
+#
+# TODO: panedwindow sashes should be 9 pixels (HIG:Controls:Split Views)
+#
 
 namespace eval tile {
 
     style theme settings aqua {
 
-	style default . \
+	style configure . \
 	    -font System \
 	    -background White \
 	    -foreground Black \
@@ -27,14 +30,14 @@ namespace eval tile {
 	# Actually, on Aqua we probably shouldn't stipple images in
 	# disabled buttons even if it did work...
 	#
-	style default . -stipple {}
+	style configure . -stipple {}
 
-	style default TButton -padding {0 2} -width -6
-	style default Toolbutton -padding 4
-	style default TNotebook.Tab -padding {10 2 10 2}
+	style configure TButton -padding {0 2} -width -6
+	style configure Toolbutton -padding 4
+	style configure TNotebook.Tab -padding {10 2 10 2}
 
 	# Enable animation for ttk::progressbar widget:
-	style default TProgressbar -period 100 -maxphase 255
+	style configure TProgressbar -period 100 -maxphase 255
 
 	# A hack:
 	# Aqua doesn't use underlines to indicate mnemonics;
@@ -58,7 +61,7 @@ namespace eval tile {
 	# with a 14 pixel inset and 4 pixels spacing 
 	# (ref: Apple Human Interface Guidelines / Controls / Grouping Controls)
 	#
-    	style default TLabelframe \
+    	style configure TLabelframe \
 		-labeloutside true -labelinset 14 -labelspace 4
     }
 }
