@@ -1,5 +1,5 @@
 /*
- * aquaTheme.c,v 1.30 2005/04/28 21:21:05 tallniel Exp
+ * aquaTheme.c,v 1.32 2005/10/07 22:34:09 jenglish Exp
  *
  * Tk theme engine for Mac OSX, using the Appearance Manager API.
  *
@@ -463,8 +463,6 @@ typedef struct {	/* TrackElement client data */
     SInt32		thicknessMetric;
 } TrackElementData;
 
-static TrackElementData ProgressData =
-    { kThemeProgressBar, kThemeMetricLargeProgressBarThickness };
 static TrackElementData ScaleData =
     { kThemeSlider, kThemeMetricHSliderHeight };
 
@@ -528,7 +526,7 @@ static void TrackElementDraw(
 	break;
     case kThemeSlider:
     	drawInfo.trackInfo.slider.pressState = 0; /* @@@ fill this in */
-    	drawInfo.trackInfo.slider.thumbDir =  kThemeThumbDownward;
+    	drawInfo.trackInfo.slider.thumbDir =  kThemeThumbPlain;
 	    /* kThemeThumbUpward, kThemeThumbDownward, kThemeThumbPlain  */
 	break;
     }
@@ -933,18 +931,13 @@ int AquaTheme_Init(Tcl_Interp *interp)
     Ttk_RegisterElementSpec(themePtr, "vseparator",&SeparatorElementSpec,0);
 
     /* <<NOTE-TRACKS>>
-     * The Progressbar widget adjusts the size of the bar element.
-     * In the Aqua theme, the appearance manager computes the bar geometry,
-     * so we make *.bar a no-op and do all the drawing in the *.trough
+     * The Progressbar widget adjusts the size of the pbar element.
+     * In the Aqua theme, the appearance manager computes the bar geometry;
+     * we do all the drawing in the ".track" element and leave the .pbar out.
      */
-    Ttk_RegisterElementSpec(themePtr,"Progress.trough",
-    	&TrackElementSpec, &ProgressData);
     Ttk_RegisterElementSpec(themePtr,"Scale.trough",
     	&TrackElementSpec, &ScaleData);
-    Ttk_RegisterElementSpec(themePtr,"Progress.bar",&NullElementSpec,0);
     Ttk_RegisterElementSpec(themePtr,"Scale.slider",&SliderElementSpec,0);
-
-    /* <<NOTE-TRACKS>> -- same for new Progressbar */
     Ttk_RegisterElementSpec(themePtr,"Progressbar.track", &PbarElementSpec, 0);
 
     /* Layouts:
