@@ -1,5 +1,5 @@
 /*
- * xpTheme.c,v 1.77 2005/03/24 18:01:16 jenglish Exp
+ * xpTheme.c,v 1.81 2005/10/08 16:39:09 jenglish Exp
  *
  * Tk theme engine which uses the Windows XP "Visual Styles" API
  * Adapted from Georgios Petasis' XP theme patch.
@@ -17,11 +17,18 @@
  *  	shellcc/platform/commctls/userex/refentry.asp >
  */
 
-#ifdef _MSC_VER
-#define WIN32_LEAN_AND_MEAN
-#endif
+#ifndef HAVE_UXTHEME_H
+/* Stub for platforms that lack the XP theme API headers: */
+#include <windows.h>
+#include <tcl.h>
+int XPTheme_Init(Tcl_Interp *interp, HWND hwnd) { return TCL_OK; }
+#else
+
+#define WINVER 0x0501	/* Requires Windows XP APIs */
 
 #include <windows.h>
+#include <uxtheme.h>
+#include <tmschema.h>
 
 #ifdef NO_PRIVATE_HEADERS
 #include <tk.h>
@@ -32,22 +39,6 @@
 
 #include "tkTheme.h"
 
-/*
- * This #if chunk makes it possible to build this library using older
- * versions of the Platform SDK that don't support the uxtheme library.
- */
-#ifndef HAVE_UXTHEME_H
-
-#if _MSC_VER > 100
-#pragma message("warning: XP theme is unavailable.")
-#endif
-int XPTheme_Init(Tcl_Interp *interp, HWND hwnd) { return TCL_OK; }
-
-#else
-
-#include <uxtheme.h>
-#include <tmschema.h>
-
 typedef HTHEME  (STDAPICALLTYPE OpenThemeDataProc)(HWND hwnd,
 		 LPCWSTR pszClassList);
 typedef HRESULT (STDAPICALLTYPE CloseThemeDataProc)(HTHEME hTheme);
@@ -905,7 +896,7 @@ static ElementInfo ElementInfoTable[] = {
     	TKP_TRACK, scale_statemap, NOPAD, 0 },
     { "Vertical.Scale.track", &GenericElementSpec, L"TRACKBAR",
     	TKP_TRACKVERT, scale_statemap, NOPAD, 0 },
-    /* for ttk::progressbar: */
+    /* ttk::progressbar elements */
     { "Horizontal.Progressbar.pbar", &PbarElementSpec, L"PROGRESS",
     	PP_CHUNK, null_statemap, NOPAD, 0 },
     { "Vertical.Progressbar.pbar", &PbarElementSpec, L"PROGRESS",
@@ -914,15 +905,7 @@ static ElementInfo ElementInfoTable[] = {
     	PP_BAR, null_statemap, PAD(3,3,3,3), IGNORE_THEMESIZE },
     { "Vertical.Progressbar.trough", &GenericElementSpec, L"PROGRESS",
     	PP_BARVERT, null_statemap, PAD(3,3,3,3), IGNORE_THEMESIZE },
-    /* for ttk::progress: */
-    { "Horizontal.Progress.bar", &GenericElementSpec, L"PROGRESS",
-    	PP_CHUNK, null_statemap, NOPAD, 0 },
-    { "Vertical.Progress.bar", &GenericElementSpec, L"PROGRESS",
-    	PP_CHUNKVERT, null_statemap, NOPAD, 0 },
-    { "Horizontal.Progress.trough", &GenericElementSpec, L"PROGRESS",
-    	PP_BAR, null_statemap, PAD(3,3,3,3), IGNORE_THEMESIZE },
-    { "Vertical.Progress.trough", &GenericElementSpec, L"PROGRESS",
-    	PP_BARVERT, null_statemap, PAD(3,3,3,3), IGNORE_THEMESIZE },
+    /* ttk::notebook */
     { "tab", &TabElementSpec, L"TAB",
     	TABP_TABITEM, tabitem_statemap, PAD(3,3,3,0), 0 },
     { "client", &GenericElementSpec, L"TAB", 
@@ -938,7 +921,7 @@ static ElementInfo ElementInfoTable[] = {
     { "Treeitem.indicator", &TreeIndicatorElementSpec, L"TREEVIEW",
     	TVP_GLYPH, tvpglyph_statemap, PAD(1,1,6,0), 0 },
     { "Treeheading.border", &GenericElementSpec, L"HEADER", 
-    	HP_HEADERITEM, header_statemap, NOPAD,0 },
+    	HP_HEADERITEM, header_statemap, PAD(4,0,4,0),0 },
 
 #if BROKEN_TEXT_ELEMENT
     { "Labelframe.text", &TextElementSpec, L"BUTTON", 
@@ -1015,4 +998,4 @@ int XPTheme_Init(Tcl_Interp *interp, HWND hwnd)
     return TCL_OK;
 }
 
-#endif /* WINVER < 0x0501 */
+#endif /* HAVE_UXTHEME_H */
